set gamma 1

load_ph

# globals:
# members - database in canonical order
# ws - working set (reordering of members)
# ws_size
# od - current directory
# db - db directory
# features - feature directory
# within - within-image hier directory
# across - across-image hier directory
# within_list/across_list - list of all hier names
# use_within/across(h) - 1 if hier is currently used
# view - current view object
# metric - current comparison object
# label_index - array of index corresponding to label name
# label_count - next label index
# label_color - array of color corresponding to label name
# current_label - current label name
# label_mode - 0 means click to search

# reorders the working set according to similarity with the query image
proc query {index} {
    global members ws ws_size metric start distances

    if {![info exists metric]} {
	puts "no metric defined"
	return
    }
    set query [lindex $ws [expr $index+$start]]
    puts "query: $query"

    # get a vector of distances
    #boa print $metric
    set bdists [boa getu $metric $query]

    boa tcl_var $bdists distances
    set ws $members
    sort -real distances ws
    #unset distances
    set start 0
    refresh
}

proc shuffle {} {
    global start ws distances
    set dists ""
    foreach m $ws {
	lappend dists [ran]
    }
    sort -real dists ws
    set start 0
    if {[info exists distances]} {unset distances}
    refresh
}

proc canonicalize {} {
    global start members ws distances
    set ws $members
    set start 0
    if {[info exists distances]} {unset distances}
    refresh
}

# Menu bar ####################################################################
set tearoff 1
frame .menu 
#-relief raised -borderwidth 2
pack .menu -side top -fill x 

menubutton .menu.db -text "Database" -menu .menu.db.m -relief raised
menu .menu.db.m -tearoff $tearoff

menubutton .menu.view -text "View" -menu .menu.view.m -relief raised
menu .menu.view.m -tearoff $tearoff
menubutton .menu.metric -text "Metric" -menu .menu.metric.m -relief raised
menu .menu.metric.m -tearoff $tearoff

menubutton .menu.within -text "Within" -menu .menu.within.m -relief raised
menu .menu.within.m -tearoff $tearoff
menubutton .menu.across -text "Across" -menu .menu.across.m -relief raised
menu .menu.across.m -tearoff $tearoff

pack .menu.db .menu.view .menu.metric .menu.within .menu.across -side left -fill x -expand 1

# stop and restart across-image labeling
# use only the selected acrosses; use all withins
proc across_init {} {
    global across use_across across_list within within_list
    if {![info exists across_list]} {return}
    ph across destroy
    photo_cursor watch
    ph across init $across $across_list $within $within_list
    # now enable according to use_across
    set index 0
    foreach name $across_list {
	ph across enable $index $use_across($name)
	incr index
    }
    photo_cursor ""
}

proc db_destroy {} {
    global metric view features db cache distances
    global across across_list use_across within within_list use_within 
    if [info exists metric] {boa free $metric; unset metric}
    if [info exists view] {boa free $view; unset view}
    if [info exists across] {boa free $across; ph across destroy; unset across across_list}
    if [info exists use_across] {unset use_across}
    if [info exists within] {boa free $within; unset within within_list}
    if [info exists use_within] {unset use_within}
    if [info exists features] {boa free $features}
    if [info exists distances] {unset distances}
    boa free $cache; unset cache
    boa free $db; unset db
}

proc db_open {name} {
    global db cache features members ws ws_size
    global within use_within within_list across use_across across_list
    global tearoff env

    if [info exists db] {db_destroy}
    set db [boa create [boa type objDir]]
    boa set $db _path [boa string $env(PHOTOBOOK_DB_DIR)/$name]
    set cache [boa get $db cache]
    set index [boa get $cache index]
    boa hold $index
    ph set_index $index
    boa tcl_var $index members
    set ws $members
    set ws_size [llength $ws]
    catch {set features [boa get $cache feature]}

    # clear existing menu
    .menu.view.m delete 0 last
    # read the views
    if {![catch {boa tcl_var [boa get $db view_list] list}] ||
        ![catch {boa tcl_var [boa get $db view/_index] list}]} {
	puts "views: $list"
	foreach name $list {
	    .menu.view.m add radio -label $name -variable view_name \
		    -command "setview view/$name"
	}
	# set the view to be the first view (calls setview)
	# first menu entry comes after the tearoff button
	.menu.view.m invoke $tearoff
    } else {
	# if there is no view, force a refresh to show the labels
	refresh
    }

    .menu.metric.m delete 0 last
    # read the metrics
    if {![catch {boa tcl_var [boa get $db metric/_index] list}]} {
	puts "metrics: $list"
	foreach name $list {
	    .menu.metric.m add radio -label $name -variable metric_name \
		    -command "setmetric metric/$name"
	}
	# set the metric to be the first metric (calls setmetric)
	.menu.metric.m invoke $tearoff
    }

    .menu.within.m delete 0 last
    # read the withins
    if {![catch {boa tcl_var [boa get $db within_list] list}] ||
	![catch {boa tcl_var [boa get $db within/_index] list}]} {
	puts "withins: $list"
	# withins are cached
	set within [boa get $cache within]
	foreach name $list {
	    .menu.within.m add check -label $name \
		    -variable use_within($name)
	    .menu.within.m invoke $name
	}
        set within_list $list
    }
    
    .menu.across.m delete 0 last
    # read the acrosses
    if {![catch {boa tcl_var [boa get $db across_list] list}] ||
        ![catch {boa tcl_var [boa get $db across/_index] list}]} {
	puts "acrosses: $list"
	# acrosses are not cached
	set across [boa get $db across]
	foreach name $list {
	    .menu.across.m add check -label $name \
		    -variable use_across($name) -command "across_toggle $name"
	}
	set across_list $list
	across_init
	foreach name $list {
	    .menu.across.m invoke $name
	}
    }
}
# callback for view menu
proc setview {name} {
    global cache view
    if [info exists view] {boa free $view}
    # should cache these in Tcl instead of in TpmImages
    if [catch {set view [boa get $cache $name]} msg] {
	puts stderr $msg
    }
    refresh
}
# callback for metric menu
proc setmetric {name} {
    global cache metric features members ws_size
    if [info exists metric] {boa free $metric}
    set metric [boa get $cache $name]
    boa hold $features
    boa set $metric features $features
    # mapCon requires an array, not an index
    set tests [boa parse tcl [boa parse type 0 "(array $ws_size string)"] \{$members\}]
    boa set $metric test $tests
}
# callback for across menu
proc across_toggle name {
    global use_across tearoff
    set index [expr [.menu.across.m index $name]-$tearoff]
    puts "toggle across: $index"
    ph across enable $index $use_across($name)
    refresh_labels
}

# Photo grid ##################################################################
frame .center-frame
pack .center-frame -fill both -expand 1

frame .grid-frame
pack .grid-frame -fill both  -side left -in .center-frame

# remove the highest-numbered photo
proc remove_photo {} {
    global total
    incr total -1
    set index $total

    destroy .frame$index
    destroy .photo$index
    destroy .label$index
    image delete image$index
}
proc add_photo {} {
    global total gamma
    set index $total
    incr total

    # make a frame for image and label
    frame .frame$index -relief sunken -borderwidth 2

    # make a canvas for the image, so we can draw on it
    image create photo image$index -gamma $gamma
    canvas .photo$index -width 64 -height 64
    .photo$index create image 1 1 -anchor nw -image image$index
    bind .photo$index <Enter> "photoEnter .frame$index"
    bind .photo$index <Leave> "photoLeave .frame$index"
    bind .photo$index <1> "photoWithin .frame$index %x %y"
    bind .photo$index <2> "photoDown .frame$index"
    bind .photo$index <ButtonRelease-2> "photoUp .frame$index"
    bind .photo$index <Control-3> "across_init; refresh"
    bind .photo$index <Shift-1> "showSeg_init $index"
    
    # make a label for the image name
    label .label$index
    # canvas should not fill x
    pack .photo$index -in .frame$index
    pack .label$index -in .frame$index -fill x 
}

# buttons for adding/removing rows and columns
frame .row-frame
pack .row-frame -fill y -side left -in .grid-frame
button .sub-row -text "-" -padx 0 -command {
    if {$rows > 1} {
	incr rows -1
	for {set col 0} {$col < $cols} {incr col} {
	    remove_photo
	}
    }
}
pack .sub-row -side top -fill y -expand 1 -in .row-frame
button .add-row -text "+" -padx 0 -command {
    incr rows
    for {set col 0} {$col < $cols} {incr col} {
	add_photo
    }
    layout_photos
    refresh
}
pack .add-row -side top -fill y -expand 1 -in .row-frame

frame .col-frame
pack .col-frame -fill x  -side bottom -in .grid-frame
button .sub-col -text "-" -pady 0 -command {
    incr cols -1
    for {set row 0} {$row < $rows} {incr row} {
	remove_photo
    }
    layout_photos
    refresh
}
pack .sub-col -side left -fill x -expand 1 -in .col-frame
button .add-col -text "+" -pady 0 -command {
    incr cols
    # add photos to the grid bottom
    for {set row 0} {$row < $rows} {incr row} {
	add_photo
    }
    layout_photos
    refresh
}
pack .add-col -side left -fill x -expand 1 -in .col-frame

# grid of photos
# must create this before the photos, so it will be underneath them
frame .grid
pack .grid -fill both  -in .grid-frame

# create the initial set of photos
set rows 2
set cols 2
set total 0
set start 0
for {set row 0} {$row < $rows} {incr row} {
    for {set col 0} {$col < $cols} {incr col} {
	add_photo
    }
}

# arrange photos in a grid
proc layout_photos {} {
    global rows cols
    for {set row 0} {$row < $rows} {incr row} {
	for {set col 0} {$col < $cols} {incr col} {
	    set index [expr $cols*$row+$col]
	    grid .frame$index -row $row -column $col -in .grid
	}
    }
}
layout_photos

# Photo callbacks #############################################################
proc photoDown w {
    global active phPriv
    if $active($w) {
	set phPriv(window) $w
	$w configure -relief raised
    }
}
proc photoUp w {
    global active phPriv
    if $active($w) {
	if {$phPriv(window) == $w} {
	    $w configure -relief sunken
	    photoInvoke $w
	}
    }
}
proc photoInvoke w {
    update
    query [string range $w 6 end]
}
set phPriv(window) ""
proc photoEnter w {
    global phPriv
}
proc photoLeave w {
    global phPriv
    if {$phPriv(window) == $w} {
	$w configure -relief sunken
	set phPriv(window) ""
    }
}
proc photoWithin {w x y} {
    global active
    if $active($w) {
	within_init [string range $w 6 end] $x $y
    }
}
# start within-image labeling mode
proc within_init {index x y} {
    global ws canvas db 
    global within within_list use_within 
    global start normal_binds

    if {![info exists within_list]} {
	puts "this database has no withins"
	return
    }
    set member [lindex $ws [expr $start+$index]]
    puts "within $member"
    set list ""
    foreach name $within_list {
	if {$use_within($name)} {
	    lappend list "$name/$member"
	}
    }
    ph within init $within $list
    set canvas .photo$index
    # save the bindings we will override
    set normal_binds {
	Button-1 B1-Motion Control-1 \
	Button-2 Control-2 \
	Button-3
    }
    get_bindings $canvas normal_binds
    within_ex pos $x $y
    bind $canvas <1> "within_ex pos %x %y"
    bind $canvas <B1-Motion> "within_ex pos %x %y"
    bind $canvas <2> "within_ex neg %x %y"
    bind $canvas <Control-1> "attach_label pos $index"
    bind $canvas <Control-2> "attach_label neg $index"
    bind $canvas <3> "within_clear"
}
# stop within-image labeling mode
proc within_clear {} {
    global canvas normal_binds
    ph within destroy 
    set_bindings $canvas normal_binds
    # clear the canvas
    $canvas delete line
    refresh_labels
}
# attach a label to the within-image region
# also stops within-image labeling mode
proc attach_label {type index} {
    global start label_index current_label ws
    set member [lindex $ws [expr $start+$index]]
    ph across $type [ph member $member] $label_index($current_label) \
	    [ph within get]
    within_clear
}
# add a pos/neg example for the within-image region
# i.e. a position which should be covered/uncovered
proc within_ex {type x y} {
    global canvas
    ph within $type $canvas $x $y
    $canvas delete line
    $canvas coords image
    ph seg show $canvas [ph within get] white
}

proc photo_cursor {cursor} {
    global total
    for {set i 0} {$i < $total} {incr i} {
	.photo$i configure -cursor $cursor
    }
}

# draw the across-image labels on the photos
proc refresh_labels {} {
    global start total ws ws_size label_color label_index across_list
    if {![info exists across_list]} {return}
    photo_cursor watch
    for {set i 0} {$i < $total} {incr i} {
	set pos [expr $i + $start]
	if {$pos >= $ws_size} {break}
	set member [lindex $ws $pos]
        #puts "refresh_label: $member"
        .photo$i delete line
	foreach label [array names label_color] {
	    #puts $label
	    ph seg show .photo$i \
		    [ph across seg [ph member $member] $label_index($label)] \
		    $label_color($label)
	    update
	}
    }
    photo_cursor ""
}

proc save_labels {} {
    global members features label_index label_color
    #exec mkdir [boa get $features _path]/labels
    if {[catch {set labeldir [boa get $features labels/seg]} msg]} {
	puts stderr $msg
	return
    }
    foreach member $members {
	puts stderr $member
	set segs [boa create [boa type ord]]
	foreach label [array names label_index] {
	    set segmask [ph across seg [ph member $member] \
		    $label_index($label)]
	    boa set $segs "$label $label_color($label)" \
		    [ph seg fromTcl $segmask]
	}
	boa set $labeldir $member $segs
    }
    boa free $labeldir
    # flush the feature cache (kludge until we have cache coherence)
    boa set $features _cache [boa create [boa type dict]]
}

proc load_labels {} {
    global members label_index features
    if {[catch {set labeldir [boa get $features labels/seg]} msg]} {
	puts stderr $msg
	return
    }
    foreach member $members {
	puts stderr $member
	set ord [boa get $labeldir $member]
	boa tcl_var [boa get $ord _length] length
	for {set i 0} {$i < $length} {incr i} {
	    set segmask [ph seg toTcl [boa intGet $ord $i]]
	    boa tcl_var [boa keyGet $ord $i] lc
	    add_label $lc
	    set label [lindex $lc 0]
	    ph across pos [ph member $member] $label_index($label) $segmask
	}
	boa free $ord
    }
    refresh_labels
}

# error image
image create photo imageE -file $env(PHOTOBOOK_ROOT)/lib/common/error.ppm

# bind images to grid entries, i.e. photos
# if a read error occurs, use the error image instead
# calls refresh_labels
proc refresh {} {
    global view start total ws ws_size active annotations distances
    photo_cursor watch
    # blank the grid
    for {set i 0} {$i < $total} {incr i} {
	# fill label with blanks, but keep same length
	#.label$i configure -text [repeat " " [expr 0.7*[string length [.label$i configure -text]]]]
	.photo$i delete line
	set active(.frame$i) 0
	image$i blank
    }
    # fill in the grid
    for {set i 0} {$i < $total} {incr i} {
	set pos [expr $i + $start]
	if {$pos >= $ws_size} {break}
	set name [lindex $ws $pos]
	set text $name
	if {[info exists distances]} {
	    append text "\n[lindex $distances $pos]"
	}
	if {[info exists annotations]} {
	    if {![catch {boa tcl_var [boa getu $annotations $name] ann}]} {
		append text "\n$ann"
	    }
	}
	.label$i configure -text $text

	set active(.frame$i) 1
	#puts "image: $name"
	if [catch {set img [boa get $view $name]} msg] {
	    image$i copy imageE -shrink
	    puts stderr $msg
	} else {
	    boa tk_image $img image$i
	}
	.photo$i configure -width [image width image$i] \
		-height [image height image$i]
	update
    }
    refresh_labels
    photo_cursor ""
}

proc showSeg_init index {
    global canvas showSeg showSeg_segs 
    global ws start normal_binds within_list use_within

    if {![info exists within_list]} {
	puts "this database has no withins"
	return
    }
    set member [lindex $ws [expr $start+$index]]
    set list ""
    foreach name $within_list {
	if {$use_within($name)} {
	    lappend list "$name/$member"
	}
    }
    set showSeg(list) $list
    set showSeg(within) 0

    set canvas .photo$index
    # save the bindings we will override
    set normal_binds {
	Button-1 Shift-1 \
	Button-2 \
	Button-3 
    }
    get_bindings $canvas normal_binds
 
    bind $canvas <1> {showSeg_incr 1}
    bind $canvas <2> {showSeg_incr -1}
    bind $canvas <3> {showSeg_clear}
    bind $canvas <Shift-1> {showSeg_within_incr 1}

    showSeg_within_incr 0
}
proc showSeg_within_incr inc {
    global canvas showSeg showSeg_segs within
    global status
    incr showSeg(within) $inc
    set showSeg(within) [expr $showSeg(within) % [llength $showSeg(list)]]
    set status [lindex $showSeg(list) $showSeg(within)]
    while {[catch {set obj [boa get $within $status]}]} {
	incr showSeg(within) 1
	set showSeg(within) [expr $showSeg(within) % [llength $showSeg(list)]]
	set status [lindex $showSeg(list) $showSeg(within)]
    }
    set showSeg(max) [ph seg fromHier $obj showSeg_segs]
    boa free $obj
    $canvas delete line
    set showSeg(seg) 0
    showSeg_incr 0
}
proc showSeg_incr inc {
    global canvas showSeg showSeg_segs
    incr showSeg(seg) $inc
    set showSeg(seg) [expr $showSeg(seg) % $showSeg(max)]
    if {$showSeg(seg) < 0} {incr showSeg(seg) $showSeg(max)}
    $canvas delete line
    ph seg show $canvas $showSeg_segs($showSeg(seg)) white
}
proc showSeg_clear {} {
    global canvas normal_binds
    set_bindings $canvas normal_binds
    # clear the canvas
    $canvas delete line
}

# Labels ######################################################################
frame .label-frame
pack .label-frame -fill y -side right -in .center-frame
proc del_label {name} {
    global label_index label_color
    unset label_index($name)
    unset label_color($name)
    destroy .label-$name
}
proc add_label {lc} {
    global label_index label_color current_label label_count
    unpack $lc label color
    if [info exists label_index($label)] {return}
    set label_color($label) $color
    set label_index($label) $label_count
    incr label_count
    radiobutton .label-$label -text $label -variable current_label \
	    -value $label -fg $color
    pack .label-$label -expand 1 -fill x -in .label-frame
    .label-$label invoke
    bind .label-$label <Button-3> [list del_label $label]
}

# create default labels
set default_labels {
  {grass aquamarine} 
  {sky DodgerBlue}
}
set default_labels {
  {building white} 
  {grass aquamarine} 
  {leaves green}
  {person yellow}
  {sky DodgerBlue}
  {street DarkSlateGray}
  {water blue3}
}
set label_count 0
foreach l $default_labels {
  add_label $l
}

# new label entry widget
label .newlabel -width 0
bind .newlabel <Enter> {focus .newlabel}
bind .newlabel <Return> {
  if {[llength [.newlabel get]] < 2} {
    tk_dialog .error Error \
      "Please enter a label and a color,\ne.g. building white" \
      error 0 "Ok"
  } else {
    add_label [.newlabel get]
  }
}

button .savelabels -text "Save" -command save_labels

button .loadlabels -text "Load" -command load_labels

pack .loadlabels .savelabels .newlabel -side bottom -fill x -in .label-frame

# Control buttons #############################################################
label .status -textvar status
pack .status -fill x 

frame .start-frame
pack .start-frame -fill x 
label .start-label -textvar start
pack .start-label -side left -in .start-frame
label .start-entry
bind .start-entry <Return> {
    global ws
    set i [lsearch -exact $ws [.start-entry get]]
    if {$i >= 0} {
	set start $i
	refresh
    }
}
pack .start-entry -fill x  -in .start-frame

frame .scroll-frame
pack .scroll-frame -fill x 
button .up -text Up -command {
    if {$start == 0} return
    if {$start > 0} {set start [expr $start - $total]}
    if {$start < 0} {set start 0}
    refresh
}
pack .up -fill x -expand 1 -side left -in .scroll-frame
button .down -text Down -command {
    set start [expr $start + $total]
    refresh
}
pack .down -fill x -expand 1 -side right -in .scroll-frame

frame .order-frame
pack .order-frame -fill x 
button .shuffle -text Shuffle -command shuffle
pack .shuffle -fill x -expand 1 -side left -in .order-frame
button .canon -text Canonicalize -command canonicalize
pack .canon -fill x -expand 1 -side right -in .order-frame

button .quit -text Quit -command {
    db_destroy
    rename ph ""
    rename boa ""
    exit
}
pack .quit -fill x

# DB Init #####################################################################
# load db list into the menu
set od [boa create [boa type objDir]]
if [catch {boa tcl_var [boa get $od $env(PHOTOBOOK_DB_DIR)/database_list] list}] {
    boa tcl_var [boa get $od $env(PHOTOBOOK_DB_DIR)/_index] list
}
boa free $od
foreach name $list {
    .menu.db.m add radio -label $name -variable db_name \
	    -command "db_open $name"
}
if {[llength $argv] > 0} {
    # load the matching database
    set i [.menu.db.m index [lindex $argv 0]]
    .menu.db.m invoke $i
} else {
    # load the first database in the db menu
    .menu.db.m invoke $tearoff
}
