set gamma 1
load_ph
load-libs lapack blas f2c
# to load extra packages
catch ph
# somehow this makes zooming work later on
#boa get [boa global dlsym] procs/Boa_ImageZoom

# globals:
# members - database in canonical order
# ws - working set (reordering of members)
# ws_size
# od - current directory
# db - db directory
# features - feature directory
# view - current view object
# metric - current comparison object

# left, right - index in the ws

proc dist_combine {left_dists right_dists} {
    global ws_size left right members
    set left_index [lsearch $members $left]
    set right_index [lsearch $members $right]
    # d is the distance between left and right
    set d [expr ([lindex $left_dists $right_index] + \
                 [lindex $right_dists $left_index])/2]
    set result ""
    for {set i 0} {$i < $ws_size} {incr i} {
	set dl [lindex $left_dists $i]
	set dr [lindex $right_dists $i]
	if {$dl <= $d} {
	    if {$dr <= $d} {
		set dm [expr ($d - $dr + $dl)/2]
	    } else {
		set dm [expr ($d - $dr - $dl)/2]
	    }
	} elseif {$dl <= $dr} {
	    set dm [expr ($d - $dr - $dl)/2]
	} else {
	    set dm [expr ($d + $dr + $dl)/2]
	}
	lappend result $dm
    }
    return $result
}

# reorders the working set according to similarity with the 
# left and right images
proc reorder {left right} {
    global members ws ws_size metric distances

    if {![info exists metric]} {
	puts "no metric defined"
	return
    }

    # get a vector of distances to left and to right
    boa tcl_var [boa getu $metric $left] left_dists
    if {$right != ""} {
	boa tcl_var [boa getu $metric $right] right_dists
	# combine the distances
	set distances [dist_combine $left_dists $right_dists]
    } else {
	set distances $left_dists
    }

    set ws $members
    sort -real distances ws
    #unset distances
    refresh
}

# Menu bar ####################################################################
set tearoff 1
frame .menu 
#-relief raised -borderwidth 2
pack .menu -side top -fill x 

menubutton .menu.db -text "Database" -menu .menu.db.m -relief raised
menu .menu.db.m -tearoff $tearoff

menubutton .menu.view -text "View" -menu .menu.view.m -relief raised
menu .menu.view.m -tearoff $tearoff
menubutton .menu.metric -text "Metric" -menu .menu.metric.m -relief raised
menu .menu.metric.m -tearoff $tearoff

pack .menu.db .menu.view .menu.metric -side left -fill x -expand 1

proc db_destroy {} {
    global metric view features db cache distances
    if [info exists metric] {boa free $metric; unset metric}
    if [info exists view] {boa free $view; unset view}
    if [info exists features] {boa free $features}
    if [info exists distances] {unset distances}
    boa free $cache; unset cache
    boa free $db; unset db
    delete_photos
}

proc db_open {name} {
    global db cache features members ws ws_size
    global tearoff env

    if [info exists db] {db_destroy}
    set db [boa create [boa type objDir]]
    boa set $db _path [boa string $env(PHOTOBOOK_DB_DIR)/$name]
    set cache [boa get $db cache]
    set index [boa get $cache index]
    boa hold $index
    boa tcl_var $index members
    set ws $members
    set ws_size [llength $ws]
    catch {set features [boa get $cache feature]}

    create_photos

    # clear existing menu
    .menu.view.m delete 0 last
    # read the views
    if {![catch {boa tcl_var [boa get $db view_list] list}]} {
	puts "views: $list"
	foreach name $list {
	    .menu.view.m add radio -label $name -variable view_name \
		    -command "setview view/$name"
	}
	# set the view to be the first view (calls setview)
	# first menu entry comes after the tearoff button
	.menu.view.m invoke $tearoff
    }

    .menu.metric.m delete 0 last
    # read the metrics
    if {![catch {boa tcl_var [boa get $db metric_list] list}]} {
	puts "metrics: $list"
	foreach name $list {
	    .menu.metric.m add radio -label $name -variable metric_name \
		    -command "setmetric metric/$name"
	}
	# set the metric to be the first metric (calls setmetric)
	.menu.metric.m invoke $tearoff
    }
}
# callback for view menu
proc setview {name} {
    global cache view
    if [info exists view] {boa free $view}
    # should cache these in Tcl instead of in TpmImages
    set view [boa get $cache $name]
    refresh
}
# callback for metric menu
proc setmetric {name} {
    global cache metric features members ws_size
    if [info exists metric] {boa free $metric}
    set metric [boa get $cache $name]
    boa hold $features
    boa set $metric features $features
    # mapCon requires an array, not an index
    set tests [boa parse tcl [boa parse type 0 "(array $ws_size string)"] \{$members\}]
    boa set $metric test $tests
}

# Photo grid ##################################################################
frame .center-frame
pack .center-frame -fill both -expand 1

# remove the highest-numbered photo
proc remove_photo {} {
    global total
    incr total -1
    set index $total

    destroy .frame$index
    destroy .photo$index
    destroy .label$index
    image delete image$index
}
proc add_photo {} {
    global total gamma
    set index $total
    incr total

    # make a frame for image and label
    frame .frame$index 
    #-relief sunken -borderwidth 2
    pack .frame$index -side left -in .center-frame

    # make a button for the image
    image create photo image$index -gamma $gamma
    button .photo$index -image image$index -command "set_left $index"
    bind .photo$index <ButtonRelease-3> "set_right $index"

    # make a label for the image name
    label .label$index
    # canvas should not fill x
    pack .photo$index -in .frame$index
    pack .label$index -in .frame$index -fill x 
}
proc create_photos {} {
    global total ws_size
    set total 0
    while {$total < $ws_size} {
	add_photo
    }
}
proc delete_photos {} {
    global total
    while {$total > 0} {
	remove_photo
    }
}

# Photo callbacks #############################################################

set left ""
set right ""
proc set_left {index} {
    global left right ws
    set left [lindex $ws $index]
    reorder $left $right
}
proc set_right {index} {
    global left right ws
    set right [lindex $ws $index]
    reorder $left $right
}

proc photo_cursor {cursor} {
    global total
    for {set i 0} {$i < $total} {incr i} {
	.photo$i configure -cursor $cursor
    }
}

# error image
image create photo imageE -file $env(PHOTOBOOK_ROOT)/lib/common/error.ppm

# bind images to grid entries, i.e. photos
# if a read error occurs, use the error image instead
proc refresh {} {
    global view start total ws ws_size annotations distances
    global left right
    photo_cursor watch
    # fill in the grid
    for {set i 0} {$i < $total} {incr i} {
	set pos $i
	if {$pos >= $ws_size} {break}
	set name [lindex $ws $pos]
	set text $name
	if {[info exists distances]} {
	    append text "\n[lindex $distances $pos]"
	}
	if {[info exists annotations]} {
	    if {![catch {boa tcl_var [boa getu $annotations $name] ann}]} {
		append text "\n$ann"
	    }
	}
	.label$i configure -text $text

	if {$name == $left} {
	    .label$i configure -bg red
	} elseif {$name == $right} {
	    .label$i configure -bg green
	} else {
	    .label$i configure -bg #d9d9d9
	}

	#puts "image: $name"
	if [catch {set img [boa get $view $name]} msg] {
	    image$i copy imageE -shrink
	    puts stderr $msg
	} else {
	    boa tk_image $img image$i
	}
	update
    }
    photo_cursor ""
}

# DB Init #####################################################################
# load db list into the menu
set od [boa create [boa type objDir]]
boa tcl_var [boa get $od $env(PHOTOBOOK_DB_DIR)/database_list] list
boa free $od
foreach name $list {
    .menu.db.m add radio -label $name -variable db_name \
	    -command "db_open $name"
}
if {[llength $argv] > 0} {
    # load the matching database
    set i [.menu.db.m index [lindex $argv 0]]
    .menu.db.m invoke $i
} else {
    # load the first database in the db menu
    .menu.db.m invoke $tearoff
}
