/* A Seg is a stripped-down Region which covers the entire image with
 * one bitmask.  The advantages are:
 * 1. The same Seg can be used on different-sized images.
 * 2. Segs are smaller and faster than Regions.
 * The disadvantage is that the image width and height must be a multiple
 * of 16.
 *
 * Routines are provided for converting between Regions and Segs.
 */

#include <tpm/polygon.h>

#define SEG_XCELLS 16
#define SEG_YCELLS 16

#define SEG_SZ (SEG_XCELLS*SEG_YCELLS/8)
typedef struct {
  unsigned char mask[SEG_SZ];  
} Seg;

extern Seg SegZero, SegAll;

void SegSet(Seg *seg, int x, int y);
void SegClear(Seg *seg, int x, int y);
int SegIsSet(Seg *seg, int x, int y);
void SegUnion(Seg *result, Seg *seg);
void SegIntersect(Seg *result, Seg *seg);
void SegXor(Seg *result, Seg *seg);
int SegSize(Seg *seg);

#ifdef TPM_REGION_H_INCLUDED
Seg SegFromRegion(Tpm_Region *r, int xmax, int ymax);
Tpm_Region *SegToRegion(Seg *seg, int xmax, int ymax);
#endif

Seg SegFromPolygon(Tpm_Polygon *poly, int xmax, int ymax);
Tpm_Polygon *SegToPolygon(Seg *seg, int xmax, int ymax);
Tpm_Polygon **SegToPolygons(Seg *seg, int xmax, int ymax);

/* t_segmask.c ***************************************************************/
#ifdef BOA_H_INCLUDED
Boa_DefineFunc Boa_Segmask_Define;
extern Type T_Segmask;
#define Boa_Segmask_Value(t) ((Seg*)(t)->value)[0]
#endif
