/*
boa_slink db/bt10/metric/ohta_nhist db/bt10/cache/feature db/bt10/index
*/
#include <tpm/cluster.h>
#include <boa/boa.h>
#include <boa/vector.h>

static int ModelRow(int row, int cols, double *dists, Obj model)
{
  Obj index, query, result;
  Vector d;

#if 0
  assert(Boa_StringGet(model, "_context/test", '/', NULL, &index) == TPM_OK);
  if(Boa_IntGet(index, row, 0, NULL, &query) != TPM_OK) {
    TpmPrependError("no item number %d", row);
    return TPM_ERROR;
  }
  Boa_Release(index);
#else
  query = Boa_Int_Create(row);
#endif
  fprintf(stderr, "query: ");
  Boa_PrintNL(stderr, query);

  if(Boa_ObjGet(model, query, 0, NULL, &result) != TPM_OK) {
    return TPM_ERROR;
  }
  Boa_Release(query);

  memcpy(dists, Boa_Vector_Value(result)->d, cols*sizeof(double));
  Boa_Release(result);
  return TPM_OK;
}

int main(int argc, char *argv[])
{
  Obj od, model, features, indexObj;
  int ntests;
  SpanTree result;

  if(argc < 4) {
    fprintf(stderr, "usage: boa_mst model members index > output\n");
    fprintf(stderr, "outputs a tree where leaves are indices into the index\n");
    exit(1);
  }

  Boa_Startup();
  Boa_Vector_Pkg();
  Boa_Hier_Pkg();
  Boa_Learn_Pkg();
  Boa_Tcl_Pkg();    /* since disk objects may need it. grrr */

  od = Boa_Create(T_ObjDir);
  if(Boa_StringGet(od, argv[1], '/', NULL, &model)) {
    TpmPrependError("Cannot open `%s'", argv[1]);
    TpmPrintError(stderr);
    exit(1);
  }
  if(Boa_StringGet(od, argv[2], '/', NULL, &features)) {
    TpmPrependError("Cannot open `%s'", argv[2]);
    TpmPrintError(stderr);
    exit(1);
  }
  if(Boa_StringGet(od, argv[3], '/', NULL, &indexObj)) {
    TpmPrependError("Cannot open `%s'", argv[3]);
    TpmPrintError(stderr);
    exit(1);
  }

  assert(Boa_StringGetInt(indexObj, "_length", &ntests) == TPM_OK);
  Boa_StringSet(model, "features", BOA_INSERT, features);
  Boa_StringSet(model, "test", BOA_INSERT, indexObj);

  result = Clus_MST(ntests, (Clus_RowFunc*)ModelRow, model);
  if(!result) {
    TpmPrintError(stderr);
    exit(1);
  }
  SpanTreePrint(stderr, result);
  SpanTreeWrite(result, stdout);
  SpanTreeFree(result);

  Boa_Release(model);
  Boa_Release(od);
  Boa_Shutdown();
  MEM_BLOCKS();
  exit(0);
}
