/* Generates a tree corresponding to a set of partition files.
 * Does not check whether the partitions form a valid tree.
 */

#include <tpm/partition.h>
#include <tpm/gtree.h>

void main(int argc, char *argv[])
{
  char *prefix, *fname;
  int from, to, num_samples, i, c;
  Partition p;
  Tree *trees, tree;
  FILE *fp;

  if(argc < 5) {
    printf("Usage: %s <hierarchy_prefix> <num_samples> <from> <to>\n", 
	   argv[0]);
    printf("To create a tree from the partition files\n");
    printf("prefix.from -- prefix.to, each having num_samples elements.\n");
    printf("Tree goes to stdout.\n");
    exit(0);
  }

  prefix = argv[1];
  num_samples = atoi(argv[2]);
  from = atoi(argv[3]);
  to = atoi(argv[4]);

  trees = Allocate(num_samples, Tree);
  for(i=0;i<num_samples;i++) {
    trees[i] = TreeCreate(IntCreate(i+1));
  }

  fname = Allocate(strlen(prefix)+5, char);
  for(;from <= to;from++) {
    /* read partition */
    sprintf(fname, "%s.%d", prefix, from);
    fp = fopen(fname, "r");
    if(!fp) {
      printf("Cannot open partition file `%s'\n", fname);
      exit(1);
    }
    p = ReadPartition(fp, num_samples);
    fclose(fp);

    /* create next level of tree */
    /* loop classes */
    for(c = 1;c <= NumClasses(p);c++) {
      if(ClassSize(p, c) <= 1) continue;
      tree = TreeCreate(IntCreate(from));
      {IterateClass(p,c,e) {
	TreeAddChildUnique(tree, trees[e]);
	trees[e] = tree;
      }}
    }
  }
  free(fname);

  /* put all existing trees under one tree */
  tree = TreeCreate(IntCreate(from));
  for(i=0;i<num_samples;i++) {
    TreeAddChildUnique(tree, trees[i]);
  }

  TreeWrite(stdout, tree, IntWrite);
  TreeFree(tree, free);
}
