/* Compute the weighted eigenvectors of the covariance matrix
 * for every sample in a mixture.
 * The covariance matrix is just a weighted average of the center
 * covariance matrices.
 */
#include <tpm/tpm.h>
#include <tpm/stream.h>
#include <tpm/array.h>

main(int argc, char *argv[])
{
  Matrix center_cov, mbr, cov;
  FILE *fp, *cov_out;
  int s, c, i;

  if(argc < 4) {
    fprintf(stderr, "usage: mixture_cov center_cov mbr cov_out\n");
    exit(1);
  }

  fp = FileOpen(argv[1], "r");
  center_cov = MatrixScan(fp);
  if(!center_cov) {
    fprintf(stderr, "Error reading center covariance matrix\n");
    exit(1);
  }
  FileClose(fp);

  fp = FileOpen(argv[2], "r");
  mbr = MatrixScan(fp);
  if(!mbr) {
    fprintf(stderr, "Error reading membership matrix\n");
    exit(1);
  }
  FileClose(fp);

  cov_out = FileOpen(argv[3], "w");

  i = sqrt(center_cov->cols);
  cov = MatrixCreate(i, i);

  fprintf(stderr, "%d samples, %d centers, %d x %d covariance matrix\n",
	  mbr->rows, center_cov->rows, i, i);

  /* Loop samples */
  for(s=0;s<mbr->rows;s++) {
    if(s % 100 == 0) fprintf(stderr, "sample: %d\n", s);
    VectorSet((Vector)cov, 0);
    /* Loop centers */
    for(c=0;c<center_cov->rows;c++) {
      /* Loop dimensions */
      for(i=0;i<center_cov->cols;i++) {
	cov->d[i] += mbr->d2[s][c] * center_cov->d2[c][i];
      }
    }
    /* Write out cov */
    VectorPrint(cov_out, (Vector)cov);
  }

  MatrixFree(cov);

  FileClose(cov_out);
  MatrixFree(mbr);
  MatrixFree(center_cov);
  exit(0);
}
