/* Turn Euclidean points into distributions. */
/* usage: neighbor_distr < point_matrix > distribution_matrix */
#include <tpm/tpm.h>
#include <tpm/array.h>

double EuclidSqrDist(double *a, double *b, int len)
{
  double d, sum = 0.0;
  while(len--) {
    d = *a++ - *b++;
    sum += d*d;
  }
  return sum; /* no sqrt */
}

#define VARIANCE 1000

main(int argc, char *argv[])
{
  Matrix sample, distr;
  int i,j;

  sample = MatrixScan(stdin);
  if(!sample) {
    fprintf(stderr, "Error reading sample matrix\n");
    exit(1);
  }
  distr = MatrixCreate(sample->rows, sample->rows);

  /* loop points */
  for(i=0;i<sample->rows;i++) {
    double sum = 0;
    if((i % 100) == 0) fprintf(stderr, "%d\n", i);
    /* compute distance to every other point */
    for(j=0;j<sample->rows;j++) {
      double d = EuclidSqrDist(sample->d2[i], sample->d2[j], 
			       sample->cols);
      sum += distr->d2[i][j] = exp(-d/2/VARIANCE);
    }
    /* normalize */
    for(j=0;j<sample->rows;j++) {
      distr->d2[i][j] /= sum;
    }
  }

  MatrixPrint(stdout, distr);
  MatrixFree(distr);

  MatrixFree(sample);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}

