/* Attempts to generate a "random" tree by the following method:
 * At each iteration, a biased coin is flipped. If it is heads, then
 * a leaf is pushed on the stack. If it is tails, then some number of
 * items on the stack, with uniform probability, are merged.
 */

#include <tpm/tpm.h>

void main(int argc, char *argv[])
{
  int num_leaves, stack_depth, chunk_size;
  int *leaves, i;

  if(argc != 3) {
    fprintf(stderr, "usage: %s <num-leaves> <chunk_size>\n", argv[0]);
    fprintf(stderr, "To generate a random tree, favoring chunks of size chunk_size.\n");
    exit(1);
  }

  num_leaves = atoi(argv[1]);
  chunk_size = atoi(argv[2]);
  if(chunk_size < 2) chunk_size = 2;
  leaves = Allocate(num_leaves, int);
  for(i=0;i<num_leaves;i++) leaves[i] = i+1;

  srand(getpid());
  stack_depth = 0;
  while(num_leaves) {
    if((stack_depth < 2) || (RandInt(1,chunk_size) != 1)) {
      /* draw a leaf at random */
      i = RandInt(0, num_leaves-1);
      /* push onto the stack */
      printf("%d\n", leaves[i]);
      stack_depth++;
      /* update leaf array */
      num_leaves--;
      leaves[i] = leaves[num_leaves];
    }
    else {
      i = RandInt(2, stack_depth);
      printf(": %d 0\n", i);
      stack_depth -= i-1;
    }
  }
  /* clean out the stack */
  while(stack_depth > 1) {
    i = RandInt(2, stack_depth);
    printf(": %d 0\n", i);
    stack_depth -= i-1;
  }

  exit(0);
}
