/* Generates a random partition by the following method:
 * For each item, a biased coin is flipped. If it is heads,
 * then the item forms a new class. If it is tails, then
 * the item joins one of the existing classes, each with equal 
 * probability of being chosen.
 */

#include <stdio.h>
#include <time.h>
#include <tpm/partition.h>

void main(int argc, char *argv[])
{
  Partition p;
  int i,n,c;
  int chunk_size;

  if(argc != 3) {
    fprintf(stderr, "%s <num_samples> <chunk_size>\n", argv[0]);
    fprintf(stderr, "To create a random partition of num_samples elements,\n");
    fprintf(stderr, "favoring groups of size chunk_size.\n");
    exit(0);
  }

  n = atoi(argv[1]);
  chunk_size = atoi(argv[2]);

  p = CreatePartition(n);
  ClassOfElement(p, 0) = 1;
  c = 2;
  srand(time(NULL));
  for(i=1;i<n;i++) {
    if(RandInt(1,chunk_size) == 1) {
      ClassOfElement(p, i) = c++;
    }
    else {
      ClassOfElement(p, i) = RandInt(1,c-1);
    }
  }
  NormalizePartition(p);
  WritePartition(stdout, p);
}
