/* Outputs Tcl code to be evaluated on a Tk canvas widget.
 */
#include <tpm/tree.h>

typedef struct PairStruct {
  void *first, *second;
} *Pair;

void *PosCreate(int x, int y);
void NewData(Tree tree, void *pos);
void LabelTree(Tree tree);
void DrawTree(Tree tree);

int Gx,Gy; /* next x and y position for leaf */
int Mx,My; /* maximum x and y positions */
int Xspacing;
int Yspacing;
char **leaf_map;
int show_nodes;
int vertical;

#define Value(n) (*(double*)((Pair)(n)->data)->first)
#define Pos(n) ((int*)((Pair)(n)->data)->second)
#define XPos(n) Pos(n)[0]
#define YPos(n) Pos(n)[1]
#define abs(x) (x > 0) ? x:-x
#define MAX_STR 80

void main(int argc, char *argv[])
{
  Tree tree, t;
  FILE *fp, *leaf_fp;
  int i;

#if 0
  scanargv(argc,argv, "%s [-xs <spacing>|-ys <spacing>|-h|-n] <tree file> [leaf file]", 
	   "[-xs %d|-ys %d|-h %+|-n %+] %r [%r]", 
	   &Xspacing, &Yspacing, &vertical, &show_nodes, &fp, &leaf_fp);
#else
  if(argc < 2) {
    fp = stdin;
#if 0
    fprintf(stderr, "usage: tree2tk tree [leaf_file]\n");
    exit(1);
#endif
  }
  else fp = fopen(argv[1], "r");
  if(argc > 1) leaf_fp = fopen(argv[2], "r");
  else leaf_fp = NULL;
  Xspacing = 0;
  Yspacing = 30;
  vertical = 0;
  show_nodes = 0;
#endif
  if(!Xspacing) Xspacing = 20;
  if(!Yspacing) Yspacing = 30;
  vertical = !vertical;

  tree = TreeRead(fp, DoubleRead);
  fclose(fp);

  /* Read the given name file into leaf_map */
  if(leaf_fp) {
    fp = leaf_fp;
    leaf_map = Allocate(2000, char*);
    for(i=0;;i++) {
      leaf_map[i] = Allocate(MAX_STR,char);
      if(!fgets(leaf_map[i], MAX_STR, fp)) break;
      if(leaf_map[i][strlen(leaf_map[i])-1] == '\n')
	leaf_map[i][strlen(leaf_map[i])-1] = 0;
    }
    fclose(fp);
  }
  else leaf_map = NULL;

  for(t=tree;t;t=t->sibling) {
    Gx = Gy = Mx = My = 0;
    if(vertical) {
      /* make room for text */
      Gx = Xspacing * 5;
    }
    LabelTree(t);
    DrawTree(t);
  }
}

/* line from b(0,1) to a(0,1) */
void DrawLine(int *a, int *b)
{
  printf("$canvas create line %d %d %d %d\n", b[0], b[1], a[0], a[1]);
}

void DrawTree(Tree tree)
{
  int *pos, /* position of corner */
      *pos2; /* position of child */

  if(tree->child) {
    {IterateChildren(tree, child) {
      DrawTree(child);
      if(vertical) {
	pos = PosCreate(XPos(tree)-Xspacing, YPos(child));
	pos2 = PosCreate(XPos(tree), YPos(child));
      }
      else {
	pos = PosCreate(XPos(child), YPos(tree)-Xspacing);
	pos2 = PosCreate(XPos(child), YPos(tree));
      }
      /* parent to corner */
      DrawLine(Pos(tree), pos2);
      /* corner to child */
      DrawLine(pos2, pos);
      if(vertical) {
	if(XPos(child) < pos[0])
	  DrawLine(pos, Pos(child));
      }
      else {
	if(YPos(child) < pos[1])
	  DrawLine(pos, Pos(child));
      }
      free(pos);
      free(pos2);
    }}
    if(show_nodes) {
      printf("$canvas create text %d %d -anchor s -text %g\n", 
	     XPos(tree)+1, YPos(tree)+1, Value(tree));
    }
  }
  else {
    /* leaf */
    if(leaf_map && (YPos(tree) > 40000) && (YPos(tree) < 60000)) {
      if(vertical) {
	printf("$canvas create image %d %d -anchor e -image [image create photo -file %s]\n",
	       XPos(tree)-Xspacing, YPos(tree), leaf_map[(int)Value(tree)]);
      }
      else {
	printf("$canvas create image %d %d -anchor n -image [image create photo -file %s]\n",
	       XPos(tree), YPos(tree), leaf_map[(int)Value(tree)]);
      }
    }
    else {
      if(vertical) {
	printf("$canvas create text %d %d -anchor e -text %g\n",
	       XPos(tree)-Xspacing, YPos(tree), Value(tree));
      }
      else {
	printf("$canvas create text %d %d -anchor n -text %g\n",
	       XPos(tree), YPos(tree), Value(tree));
      }
    }
  }
}

void LabelTree(Tree tree)
{
  int count, yavg, xmax;
  
  if(tree->child) {
    count = yavg = xmax = 0;
    /* Find the average y position and maximum x position of children */
    {IterateChildren(tree, child) {
      LabelTree(child);
      count++;
      if(vertical) {
	yavg += YPos(child);
	if(XPos(child) > xmax) xmax = XPos(child);
      }
      else {
	yavg += XPos(child);
	if(YPos(child) > xmax) xmax = YPos(child);
      }
    }}
    yavg /= count;

    /* Place the parent at yavg, just to the right of xmax */
    if(vertical) {
      NewData(tree, PosCreate(xmax + Xspacing, yavg));
      if(XPos(tree) > Mx) Mx = XPos(tree);
    }
    else {
      NewData(tree, PosCreate(yavg, xmax + Xspacing));
      if(YPos(tree) > My) My = YPos(tree);
    }
  }
  else {
    /* Place leaf node and calculate next leaf position */
    NewData(tree, PosCreate(Gx, Gy));
    if(vertical) Gy += Yspacing;
    else Gx += Yspacing;
  }
}

void NewData(Tree tree, void *pos)
{
  Pair p;

  p = Allocate(1,struct PairStruct);
  p->first = tree->data;
  p->second = pos;
  tree->data = p;
}

void *PosCreate(int x, int y)
{
  int *p;

  p = Allocate(2,int);
  p[0] = x;
  p[1] = y;
  return p;
}
