/* Intersects two trees at a specified depth on the first tree.
 */

#include <tpm/gtree.h>

Tree IntersectLeaves(Tree tree, void **leaves, int nleaves, CmpFunc *cmp);
Tree FirstAtDepth(Tree tree, int depth);
Tree NextAtDepth(Tree tree);

#define IterateAtDepth(t,d,c) \
Tree c;for(c=FirstAtDepth(t,d);c;c=NextAtDepth(c))

void main(int argc, char *argv[])
{
  Tree A,B,C,D,E;
  int depth;
  FILE *fp;
  void *leaves;
  int num_leaves;

  if(argc < 4) {
    printf("Usage: %s <tree1> <tree2> <depth>\n", argv[0]);
    exit(0);
  }

  fp = SafeOpen(argv[1], "r");
  A = TreeRead(fp, RealRead);
  fclose(fp);
  fp = SafeOpen(argv[2], "r");
  B = TreeRead(fp, RealRead);
  fclose(fp);
  depth = atoi(argv[3]);

  for(C=FirstAtDepth(A,depth);C;) {
    D = C->parent;
    E = NextAtDepth(C);
    leaves = ListToPtrArray(TreeLeafList(C), &num_leaves);
    TreeRemoveChild(C);
    TreeAddLeftChild(D, 
		     IntersectLeaves(B, (void**)leaves, num_leaves, RealCmp));
    free(leaves);
    C = E;
  }
  TreeWrite(stdout, A, RealWrite);
}

Tree IntersectLeaves(Tree tree, void **leaves, int nleaves, CmpFunc *cmp)
{
  Tree result;
  int i;
  

  if(TreeLeaf(tree)) {
    for(i=0;i<nleaves;i++) if(!cmp(leaves[i], tree->data)) break;
    if(i == nleaves) return NULL;
    return TreeCreate(tree->data);
  }

  result = TreeCreate(tree->data);
  {IterateChildren(tree, child) {
    TreeAddChild(result, IntersectLeaves(child, leaves, nleaves, cmp));
  }}
  if(TreeLeaf(result)) return NULL;
  return result;
}

Tree FirstAtDepth(Tree tree, int depth)
{
  if(!tree) return NULL;
  if(depth == 0) return tree;
  return FirstAtDepth(tree->child, depth-1);
}

Tree NextAtDepth(Tree tree)
{
  int depth;
  for(depth=0;tree && !tree->sibling;depth++,tree = tree->parent);
  if(!tree) return NULL;
  return FirstAtDepth(tree->sibling, depth);
}
