/* Definition of the Obj type.
 * An Obj is metadata for a C object; it stores reference counts
 * and a meta-Obj (called a Type) which provides access methods.
 */

struct Obj; /* forward decl */
typedef void BoaFreeFunc(struct Obj *obj);

typedef struct Obj {
  long ref_count;        
  BoaFreeFunc *freeFunc; 
  Any value;            
  struct Obj *type;
} *Obj;

typedef Obj Type;

/* Used for cleaning */
typedef void ObjFunc(Obj t, Any data);

/* Boa_Hold/Boa_Release are the reference counting primitives.  When the
 * last holder releases, the object is freed.  In general, passing or
 * returning an object relinquishes your hold on it.
 */
#define Boa_Hold(t) ((t)->ref_count++,(t))
#define Boa_Release(t) if(--(t)->ref_count); else (t)->freeFunc(t)
/* Holds t and stabilizes its data. 
 * Returns NULL if BoaCopy failed.
 */
#define Boa_Stabilize(t) (BoaFreeTest10(t) ? Boa_Hold(t) : BoaCopy(t))

/* obj.c *********************************************************************/

Obj Boa_CreateFromData(Type type, Any value);
Obj Boa_Create(Type type);
void Boa_Free(Obj t);
void Boa_FreeWithType(Obj t);
Obj BoaHold(Obj t);
Obj BoaCopyShallow(Obj t);
Obj BoaCopy(Obj t);
int BoaHash(Obj t);

int BoaFreeTest10(Obj t);
int BoaFreeTest01(Obj t);
void BoaFreeOR10(Obj t);
void BoaFreeOR01(Obj t);
void BoaFreeAND01(Obj t);
void BoaFreeAND10(Obj t);

void GcStartup(void);
void GcShutdown(void);
void GcClean(int count, ...);
