#include <boa/boa.h>

/* Globals *******************************************************************/
Obj Boa_Types = NULL;
Obj Boa_Formatters = NULL;
Obj Boa_Globals = NULL;

/* Prototypes ****************************************************************/
void Boa_Startup(void);
void Boa_Shutdown(void);

int BoaCompare(Obj a, Obj b);
int BoaHash(Obj t);
char *Boa_AsString(Obj t);
int Boa_Print(FILE *fp, Obj t);
int Boa_PrintNL(FILE *fp, Obj t);
Obj Boa_GetFormatter(char *name);
int Boa_StringGetInt(Obj context, char *name, int *value_return);
int Boa_IntGetInt(Obj context, int index, int *value_return);
int Boa_ObjGetInt(Obj context, Obj index, int *value_return);

Type Boa_Type_VaCreate(Type type, ...);
Type Boa_Type_Create(char *name);

/* Functions *****************************************************************/

/* Creates Boa_Globals and fills it with the standard types, formatters,
 * and oddballs. 
 */
static void Boa_Std_Pkg(void)
{
  Obj t, merge;
  /* types */
  Boa_SimpleType_Define();
  Boa_String_Define();
  Boa_Dict_Define();  /* requires T_String */
  Boa_Types = Boa_Create(T_Dict);
  Boa_StringSet(Boa_Types, "stype", BOA_INSERT, T_SimpleType);
  /* test the dictionary */
  assert(Boa_StringGet(Boa_Types, "stype", 0, NULL, &t) == TPM_OK);
  Boa_Release(t);
  Boa_StringSet(Boa_Types, "string", BOA_INSERT, T_String);
  Boa_StringSet(Boa_Types, "dict", BOA_INSERT, T_Dict);

  Boa_StringSet(Boa_Types, "int", BOA_INSERT, Boa_Int_Define());
  Boa_StringSet(Boa_Types, "float", BOA_INSERT, Boa_Float_Define());
  Boa_StringSet(Boa_Types, "double", BOA_INSERT, Boa_Double_Define());
  Boa_StringSet(Boa_Types, "char", BOA_INSERT, Boa_Char_Define());
  Boa_StringSet(Boa_Types, "ptr", BOA_INSERT, Boa_Ptr_Define());
  Boa_StringSet(Boa_Types, "ptype", BOA_INSERT, Boa_ParamType_Define());
  Boa_StringSet(Boa_Types, "array", BOA_INSERT, Boa_Array_Define());
  Boa_StringSet(Boa_Types, "eval", BOA_INSERT, Boa_Eval_Define());
  Boa_StringSet(Boa_Types, "iter", BOA_INSERT, Boa_Iter_Define());
  Boa_StringSet(Boa_Types, "indexer", BOA_INSERT, Boa_Indexer_Define());
  Boa_StringSet(Boa_Types, "index", BOA_INSERT, Boa_Index_Define());
  Boa_StringSet(Boa_Types, "ord", BOA_INSERT, Boa_Ord_Define());
  Boa_StringSet(Boa_Types, "merge", BOA_INSERT, Boa_Merge_Define());
  Boa_StringSet(Boa_Types, "cache", BOA_INSERT, Boa_Cache_Define());
  Boa_StringSet(Boa_Types, "idDict", BOA_INSERT, Boa_IdDict_Define());
  Boa_StringSet(Boa_Types, "obStruct", BOA_INSERT, Boa_ObStruct_Define());
  Boa_StringSet(Boa_Types, "always", BOA_INSERT, Boa_Always_Define());
  Boa_StringSet(Boa_Types, "filter", BOA_INSERT, Boa_Filter_Define());
  Boa_StringSet(Boa_Types, "strStream", BOA_INSERT, Boa_StrStream_Define());
  Boa_StringSet(Boa_Types, "contextProc", BOA_INSERT, Boa_ContextProc_Define());
  Boa_StringSet(Boa_Types, "dlopen", BOA_INSERT, Boa_DLOpen_Define());
  Boa_StringSet(Boa_Types, "dlsym", BOA_INSERT, Boa_DLSym_Define());
  Boa_StringSet(Boa_Types, "atoi", BOA_INSERT, Boa_Atoi_Define());
  Boa_StringSet(Boa_Types, "offset", BOA_INSERT, Boa_Offset_Define());
  Boa_StringSet(Boa_Types, "list", BOA_INSERT, Boa_List_Define());
  Boa_StringSet(Boa_Types, "tuple", BOA_INSERT, Boa_Tuple_Define());
  Boa_StringSet(Boa_Types, "mapCon", BOA_INSERT, Boa_MapCon_Define());
  Boa_StringSet(Boa_Types, "debug", BOA_INSERT, Boa_Debug_Define());
  Boa_StringSet(Boa_Types, "constructMap", BOA_INSERT, Boa_ConstructMap_Define());
  Boa_StringSet(Boa_Types, "spanTree", BOA_INSERT, Boa_SpanTree_Define());

  Boa_StringSet(Boa_Types, "objDir", BOA_INSERT, Boa_ObjDir_Define());
  Boa_StringSet(Boa_Types, "fileHandle", BOA_INSERT, Boa_FileHandle_Define());
  Boa_StringSet(Boa_Types, "diskArray", BOA_INSERT, Boa_DiskArray_Define());
  Boa_StringSet(Boa_Types, "formatBoa", BOA_INSERT, Boa_FormatBoa_Define());
  Boa_StringSet(Boa_Types, "formatAscii", BOA_INSERT, Boa_FormatAscii_Define());
  Boa_StringSet(Boa_Types, "formatBinary", BOA_INSERT, Boa_FormatBinary_Define());
  Boa_StringSet(Boa_Types, "formatArray", BOA_INSERT, Boa_FormatArray_Define());
  Boa_StringSet(Boa_Types, "formatIndex", BOA_INSERT, Boa_FormatIndex_Define());
  Boa_StringSet(Boa_Types, "formatDict", BOA_INSERT, Boa_FormatDict_Define());
  Boa_StringSet(Boa_Types, "formatSpanTree", BOA_INSERT, Boa_FormatSpanTree_Define());

  /* Formatters **************************************************************/
  Boa_Formatters = Boa_Create(T_Dict);
  /* type */
  t = Boa_Create(T_Eval);
  Boa_Hold(Boa_Types);
  Boa_Eval_Context(t) = Boa_Types;
  Boa_StringSet(Boa_Formatters, "type", BOA_INSERT, t);
  /* boa */
  t = Boa_Create(T_FormatBoa);
  Boa_StringSet(Boa_Formatters, "boa", BOA_INSERT, Boa_Hold(t));
  Boa_StringSet(Boa_ObjDir_DefaultFormatter, "_/boa", '/'|BOA_INSERT, t);
  /* ascii */
  merge = Boa_Create(T_Merge);
  t = Boa_Create(T_FormatAscii);
  Boa_StringSet(merge, "_/ascii", '/'|BOA_INSERT, t);
  t = Boa_Create(T_FormatArray);
  Boa_StringSet(t, "_separator", 0, Boa_Char_Create(' '));
  Boa_Hold(merge);
  Boa_StringSet(t, "_formatter", 0, merge);
  Boa_StringSet(merge, "_/array", '/'|BOA_INSERT, t);
  Boa_StringSet(Boa_Formatters, "ascii", BOA_INSERT, merge);
  /* binary */
  merge = Boa_Create(T_Merge);
  t = Boa_Create(T_FormatBinary);
  Boa_StringSet(merge, "_/binary", '/'|BOA_INSERT, t);
  t = Boa_Create(T_FormatArray);
  Boa_Hold(merge);
  Boa_StringSet(t, "_formatter", 0, merge);
  Boa_StringSet(merge, "_/array", '/'|BOA_INSERT, t);
  Boa_StringSet(Boa_Formatters, "binary", BOA_INSERT, merge);
  /* index */
  t = Boa_Create(T_FormatIndex);
  Boa_StringSet(Boa_Formatters, "index", BOA_INSERT, t);
  /* dict */
  t = Boa_Create(T_FormatDict);
  Boa_StringSet(Boa_Formatters, "dict", BOA_INSERT, t);
  /* spanTree */
  t = Boa_Create(T_FormatSpanTree);
  Boa_StringSet(Boa_Formatters, "spanTree", BOA_INSERT, Boa_Hold(t));
  Boa_StringSet(Boa_ObjDir_DefaultFormatter, "_/spanTree", '/'|BOA_INSERT, t);

  /* FormatBoa formatters */
  Boa_StringSet(Boa_FormatBoa_DefaultFormatters, "ascii", BOA_INSERT, 
		Boa_String_Create(strdup("ascii")));
  Boa_StringSet(Boa_FormatBoa_DefaultFormatters, "binary", BOA_INSERT, 
		Boa_String_Create(strdup("binary")));
  Boa_StringSet(Boa_FormatBoa_DefaultFormatters, "dict", BOA_INSERT, 
		Boa_String_Create(strdup("dict")));

  Boa_Release(Boa_FormatArray_DefaultFormatter);
  Boa_FormatArray_DefaultFormatter = Boa_GetFormatter("ascii");
  Boa_Release(Boa_DiskArray_DefaultFormatter);
  Boa_DiskArray_DefaultFormatter = Boa_GetFormatter("binary");

  /* globals */
  Boa_Globals = Boa_Create(T_Dict);
  Boa_StringSet(Boa_Globals, "type", BOA_INSERT, Boa_Types);
  Boa_StringSet(Boa_Globals, "format", BOA_INSERT, Boa_Formatters);
  t = Boa_Create(T_DLOpen);
  Boa_StringSet(Boa_Globals, "dlopen", BOA_INSERT, t);
  t = Boa_Create(T_DLSym);
  Boa_StringSet(Boa_Globals, "dlsym", BOA_INSERT, t);
  t = Boa_Create(T_Atoi);
  Boa_StringSet(Boa_Globals, "atoi", BOA_INSERT, t);
  t = Boa_Cache_Create(Boa_GetFormatter("type"));
  Boa_StringSet(Boa_Globals, "typeCache", BOA_INSERT, t);
}

/* Must be called before using Boa. */
void Boa_Startup(void)
{
  if(Boa_Types) return;   /* don't initialize twice */
#if defined(M_MXFAST) && !defined(sgi) && !defined(sparc)
  if(mallopt(M_MXFAST, 64) || mallopt(M_NLBLKS, 16384)) {
    fprintf(stderr, "mallopt failed\n");
  }
#endif
#if GC
  GcStartup();
#endif
  Boa_Std_Pkg();
}

/* Clean up all Boa objects. */
void Boa_Shutdown(void)
{
#if GC
  /* this will free all Objs, including the Types */
  GcShutdown();
#endif
#if 0
  MEM_BLOCKS();
#endif
  MEM_STATUS();
}

/* Functional version of Boa_Compare */
int BoaCompare(Obj a, Obj b)
{
  if(a == b) return 0;
  return Boa_Compare(a->type, b->type) || Boa_Methods(a)->compare(a,b);
}

/* Functional version of Boa_Hash */
int BoaHash(Obj t)
{
  return Boa_Hash(t);
}

char *Boa_AsString(Obj t)
{
  StrStream ss = StrStreamCreate(16, STRSTREAM_UNLIMITED);
  char *s;
  if(Boa_SPrint(t, ss) != TPM_OK) return NULL;
  s = strdup(StrStreamAsString(ss));
  StrStreamFree(ss);
  return s;
}

int Boa_Print(FILE *fp, Obj t)
{
  StrStream ss = StrStreamCreate(16, STRSTREAM_UNLIMITED);
  if(Boa_SPrint(t, ss) != TPM_OK) {
    StrStreamFree(ss);
    return TPM_ERROR;
  }
  fputs(StrStreamAsString(ss), fp);
  StrStreamFree(ss);
  return TPM_OK;
}

int Boa_PrintNL(FILE *fp, Obj t)
{
  TRY_RETURN(Boa_Print(fp, t));
  fputc('\n', fp);
  return TPM_OK;
}

Obj Boa_GetFormatter(char *name)
{
  Obj t;
  if(Boa_StringGet(Boa_Formatters, name, 0, NULL, &t) != TPM_OK) return NULL;
  return t;
}

int Boa_StringGetInt(Obj context, char *name, int *value_return)
{
  Obj t;
  int status;
  status = Boa_StringGet(context, name, 0, &T_Int, &t);
  if(status == TPM_OK) {
    *value_return = Boa_Int_Value(t);
    Boa_Release(t);
  }
  return status;
}

int Boa_IntGetInt(Obj context, int index, int *value_return)
{
  Obj t;
  int status;
  status = Boa_IntGet(context, index, 0, &T_Int, &t);
  if(status == TPM_OK) {
    *value_return = Boa_Int_Value(t);
    Boa_Release(t);
  }
  return status;
}

int Boa_ObjGetInt(Obj context, Obj index, int *value_return)
{
  Obj t;
  int status;
  status = Boa_ObjGet(context, index, 0, &T_Int, &t);
  if(status == TPM_OK) {
    *value_return = Boa_Int_Value(t);
    Boa_Release(t);
  }
  return status;
}

/* type must be a ParamType.
 * Returns NULL on failure.
 */
Type Boa_Type_VaCreate(Type type, ...)
{
  va_list args;
  int i;
  Boa_ParamType data = (Boa_ParamType)type->value;
  va_start(args, type);
  for(i=0;i<data->num_args;i++) {
    Obj arg = va_arg(args, Obj);
    Obj t;
    int status = Boa_ObjGet(type, arg, 0, NULL, &t);
    Boa_Release(arg);
    if(status != TPM_OK) break;
    Boa_Release(type);
    type = t;
  }
  va_end(args);
  return type;
}

/* Fetches a Type from Boa_Globals given a string name.
 * This is preferred to VaCreate since it caches Types. 
 */
Type Boa_Type_Create(char *name)
{
  Obj cache, t;
  assert(Boa_StringGet(Boa_Globals, "typeCache", 0, NULL, &cache) == TPM_OK);
  if(Boa_StringGet(cache, name, 0, NULL, &t) != TPM_OK) {
    ErrorSet("Unknown type `%s'", name);
    return NULL;
  }
  Boa_Release(cache);
  return t;
}
