/* Standard methods that Boa_Types may use for convenience */
#include <boa/boa.h>

/* Prototypes ****************************************************************/
Boa_InitFunc Empty_Init;
Boa_FinalFunc Empty_Final;
Boa_CleanFunc Empty_Clean;

Boa_FinalFunc Generic_Final;
Boa_SPrintFunc Generic_SPrint;
Boa_CopyFunc Generic_Copy;
Boa_CompareFunc Generic_Compare;
Boa_CompareFunc Identity_Compare;
Boa_HashFunc Generic_Hash;
Boa_FinalFunc Sub_Final;
Boa_CleanFunc Sub_Clean;
Boa_IntGetFunc Sub_IntGet;
Boa_IntSetFunc Sub_IntSet;
Boa_StringGetFunc Sub_StringGet;
Boa_StringSetFunc Sub_StringSet;
Boa_StringGetFunc Generic_StringGet;
Boa_StringSetFunc Generic_StringSet;
Boa_ObjGetFunc Generic_ObjGet;
Boa_ObjSetFunc Generic_ObjSet;
Boa_StringGetFunc Generic_SplitGet;
Boa_StringSetFunc Generic_SplitSet;

/* Always return TPM_ERROR */
Boa_CopyFunc Err_Copy;
Boa_SPrintFunc Err_SPrint;
Boa_StringGetFunc Err_StringGet;
Boa_StringSetFunc Err_StringSet;
Boa_IntGetFunc Err_IntGet;
Boa_IntSetFunc Err_IntSet;
Boa_ObjGetFunc Err_ObjGet;
Boa_ObjSetFunc Err_ObjSet;

/* Functions *****************************************************************/

void Empty_Init(Obj t)
{
  /* do nothing */
}

void Empty_Final(Obj t)
{
  /* do nothing */
}

void Empty_Clean(Obj t, ObjFunc *f, Any data)
{
  /* do nothing */
}

void Generic_Final(Obj t)
{
  free(*(Ptr*)t->value);
}

int Generic_SPrint(Obj self, StrStream ss)
{
  TRY_RETURN(StrStreamPuts(ss, "a "));
  return Boa_SPrint(self->type, ss);
}

int Generic_Copy(Obj t, Ptr value_return)
{
  memcpy(value_return, t->value, TypeSize(t->type));
  return TPM_OK;
}

int Generic_Compare(Obj a, Obj b)
{
  return memcmp(a->value, b->value, TypeSize(a->type));
}

int Identity_Compare(Obj a, Obj b)
{
  return a != b;
}

int Generic_Hash(Obj t)
{
  return HashMem(t->value, TypeSize(t->type));
}

void Sub_Final(Obj self)
{
  Boa_Release(*(Obj*)self->value);
}

void Sub_Clean(Obj self, ObjFunc *f, Any data)
{
  f(*(Obj*)self->value, data);
}

int Sub_IntGet(Obj self, int index, int flags,
	       Type *type_in_out, Obj *value_return)
{
  return Boa_IntGet(*(Obj*)self->value, index, flags, 
		    type_in_out, value_return);
}

int Sub_IntSet(Obj self, int index, int flags, Obj value)
{
  return Boa_IntSet(*(Obj*)self->value, index, flags, value);
}

int Sub_StringGet(Obj self, char *name, int flags,
		  Type *type_in_out, Obj *value_return)
{
  return Boa_StringGet(*(Obj*)self->value, name, flags, 
		       type_in_out, value_return);
}

int Sub_StringSet(Obj self, char *name, int flags, Obj value)
{
  return Boa_StringSet(*(Obj*)self->value, name, flags, value);
}

/* These forward to ObjGet/Set */
int Generic_StringGet(Obj self, char *name, int flags,
		      Type *type_in_out, Obj *value_return)
{
  Obj t;
  int status;
#if 0
  t = Boa_String_00(&name);
#else
  t = Boa_String_Create(strdup(name));
#endif
  status = Boa_ObjGet(self, t, flags, type_in_out, value_return);
  Boa_Release(t);
  if((status == TPM_BOTTOM) && (flags & BOA_SPLIT))
    return Generic_SplitGet(self, name, flags, type_in_out, value_return);
  return status;
}
int Generic_StringSet(Obj self, char *name, int flags, Obj value)
{
  Obj t;
  int status;
  t = Boa_String_00(&name);
  status = Boa_ObjSet(self, t, flags, value);
  Boa_Release(t);
  if((status == TPM_BOTTOM) && (flags & BOA_SPLIT))
    return Generic_SplitSet(self, name, flags, value);
  return status;
}

int Generic_ObjGet(Obj self, Obj index, int flags, 
		   Type *type_in_out, Obj *value_return)
{
  if(index->type == T_String)
    return Boa_StringGet(self, Boa_String_Value(index), flags, 
			 type_in_out, value_return);
  else if(index->type == T_Int)
    return Boa_IntGet(self, Boa_Int_Value(index), flags, 
		      type_in_out, value_return);
  return TPM_BOTTOM;
}
int Generic_ObjSet(Obj self, Obj index, int flags, Obj value)
{
  if(index->type == T_String)
    return Boa_StringSet(self, Boa_String_Value(index), flags, value);
  else if(index->type == T_Int)
    return Boa_IntSet(self, Boa_Int_Value(index), flags, value);
  return TPM_BOTTOM;
}

/* Breaks off the first component of a hierarchical name,
 * gets that name from self, and forwards the rest to the resulting object.
 * Modifies name temporarily.
 */
int Generic_SplitGet(Obj self, char *name, int flags,
		     Type *type_in_out, Obj *value_return)
{
  char *p;
  /* check for hierarchical name */
  for(p=name;*p;p++) {
    if(*p == BOA_SEP(flags)) {
      Obj t;
      int status;
      *p = '\0';
      status = Boa_StringGet(self, name, 0, NULL, &t);
      *p = BOA_SEP(flags);
      if(status != TPM_OK) return status;
      status = Boa_StringGet(t, p+1, flags, type_in_out, value_return);
      Boa_Release(t);
      return status;
    }
  }
  /* not hierarchical */
  return Boa_StringGet(self, name, flags & ~BOA_SPLIT, 
		      type_in_out, value_return);
}
int Generic_SplitSet(Obj self, char *name, int flags, Obj value)
{
  char *p;
  /* check for hierarchical name */
  for(p=name;*p;p++) {
    if(*p == BOA_SEP(flags)) {
      Obj t;
      int status;
      *p = '\0';
      status = Boa_StringGet(self, name, 0, NULL, &t);
      *p = BOA_SEP(flags);
      if(status != TPM_OK) return status;
      status = Boa_StringSet(t, p+1, flags, value);
      Boa_Release(t);
      return status;
    }
  }
  /* not hierarchical */
  return Boa_StringSet(self, name, flags & ~BOA_SPLIT, value);
}

/* Errors ********************************************************************/

int Err_Copy(Obj t, Ptr value_return)
{
  ErrorString = "unsupported method: Copy";
  return TPM_ERROR;
}

int Err_SPrint(Obj t, StrStream ss)
{
  ErrorString = "unsupported method: SPrint";
  return TPM_ERROR;
}

/* these should be called "empty" vs. "error" */
int Err_StringGet(Obj t, char *name, int flags, 
		  Type *type_in_out, Obj *value_return)
{
  ErrorString = "unsupported method: StringGet";
  return TPM_BOTTOM;
}

int Err_StringSet(Obj t, char *name, int flags, Obj value)
{
  ErrorString = "unsupported method: StringSet";
  return TPM_BOTTOM;
}

int Err_IntGet(Obj t, int index, int flags, 
	       Type *type_in_out, Obj *value_return)
{
  ErrorString = "unsupported method: IntGet";
  return TPM_BOTTOM;
}

int Err_IntSet(Obj t, int index, int flags, Obj value)
{
  ErrorString = "unsupported method: IntSet";
  return TPM_BOTTOM;
}

int Err_ObjGet(Obj t, Obj index, int flags, 
	       Type *type_in_out, Obj *value_return)
{
  ErrorString = "unsupported method: ObjGet";
  return TPM_BOTTOM;
}

int Err_ObjSet(Obj t, Obj index, int flags, Obj value)
{
  ErrorString = "unsupported method: ObjSet";
  return TPM_BOTTOM;
}
