#include <boa/boa.h>

#define Err(str) fprintf(stderr, "--> %s\n", str)

int main(int argc, char *argv[])
{
  Obj context, t, iter;
  int i;
  int a[] = {-1,0,1};
  double d;
  char *s = "hello";
  Type type;

  if(argc < 2) {
    fprintf(stderr, "usage: test_context type\n");
    fprintf(stderr, "try with dict, ord, list, obStruct, objDir (in an empty dir)\n");
    exit(1);
  }
  Boa_Startup();

  type = Boa_Type_Create(argv[1]);
  if(!type) {
    Err("unknown type");
    exit(1);
  }
  context = Boa_Create(type);

  /* If the context doesn't accept strings by default, add a decorator. */
  if((type == T_ObStruct) || (type == T_List)) {
    t = Boa_Create(T_Filter);
    Boa_Filter_Context(t) = context;
    context = t;
    Boa_Filter_StringMap(context) = Boa_Create(T_Index);
  }

  if(Boa_PrintNL(stdout, context) != TPM_OK) Err("print failed");

  /* Insert *****************************************/
  /* i = 3 */
  if(Boa_StringSet(context, "i", BOA_INSERT, Boa_Int_Create(3)) != TPM_OK)
    Err("insert i = 3 failed");
  t = Boa_Int_Create(4);
  if(Boa_StringSet(context, "i", BOA_INSERT, t) == TPM_OK) 
    Err("insert i = 4 didn't fail");
  if(Boa_StringSet(context, "j", BOA_REPLACE, t) == TPM_OK) 
    Err("replace j = 4 didn't fail");
  else if(Boa_StringSet(context, "j", 0, t) == TPM_OK) 
    Err("set j = 4 didn't fail");
  else Boa_Release(t);
  
  /* d = 4.5 */
  if(Boa_StringSet(context, "d", BOA_INSERT, Boa_Double_Create(4.5)) != TPM_OK)
    Err("insert d = 4.5 failed");

  /* a = {-1,0,1} */
  t = Boa_CreateFromData(Boa_Type_Create("(array 3 int)"), NULL);
  t->value = a;
  if(Boa_StringSet(context, "a", BOA_INSERT, t) != TPM_OK) 
    Err("insert a = {-1,0,1} failed");

  /* s = "hello" */
  if(Boa_StringSet(context, "s", BOA_INSERT, Boa_String_Create(strdup(s))) != TPM_OK)
    Err("insert s = hello failed");

  if(Boa_PrintNL(stdout, context) != TPM_OK) Err("print failed");

  /* Probe *********************************************/
  if(Boa_StringGet(context, "z", 0, NULL, NULL) == TPM_OK)
    Err("probe z didn't fail");
  if(Boa_StringGet(context, "i", 0, NULL, NULL) != TPM_OK)
    Err("probe x failed");
  if(Boa_StringGet(context, "i", 0, &T_Char, NULL) == TPM_OK)
    Err("probe x didn't fail");
  if(Boa_StringGet(context, "i", 0, &T_Int, NULL) != TPM_OK)
    Err("probe int x failed");
  type = NULL;
  if(Boa_StringGet(context, "i", 0, &type, NULL) != TPM_OK)
    Err("probe type x failed");
  else if(type != T_Int) Err("probe x returned wrong type");
  if(Boa_StringGet(context, "d", 0, &T_Double, NULL) != TPM_OK)
    Err("probe d failed");
  type = Boa_Type_Create("(array 3 int)");
  if(Boa_StringGet(context, "a", 0, &type, NULL) != TPM_OK)
    Err("probe a failed");
  if(Boa_StringGet(context, "s", 0, &T_String, NULL) != TPM_OK)
    Err("probe s failed");
  
  /* Get **************************************************/
  if(Boa_StringGet(context, "z", 0, NULL, &t) == TPM_OK)
    Err("get z didn't fail");
  if(Boa_StringGet(context, "i", 0, NULL, &t) != TPM_OK)
    Err("get i failed");
  else {
    if(Boa_Int_Value(t) != 3) Err("get i returned wrong value");
    Boa_Release(t);
  }
  if(Boa_StringGet(context, "d", 0, &T_Double, &t) != TPM_OK)
    Err("get d failed");
  else {
    if(Boa_Double_Value(t) != 4.5) Err("get d returned wrong value");
    Boa_Release(t);
  }
  if(Boa_StringGet(context, "a", 0, NULL, &t) != TPM_OK)
    Err("get a failed");
  else {
    if(memcmp(t->value, a, 3*sizeof(int))) Err("get a returned wrong value");
    Boa_Release(t);
  }
  if(Boa_StringGet(context, "s", 0, NULL, &t) != TPM_OK)
    Err("get s failed");
  else {
    if(strcmp(Boa_String_Value(t), s)) Err("get s returned wrong value");
    Boa_Release(t);
  }

  /* Set **************************************************/
  if(Boa_StringSet(context, "i", BOA_REPLACE, Boa_Int_Create(4)) != TPM_OK)
    Err("set i failed");
  if(Boa_StringSet(context, "d", BOA_REPLACE, Boa_Double_Create(2.2)) != TPM_OK)
    Err("set d failed");
  if(Boa_StringSet(context, "s", BOA_REPLACE, Boa_String_Create(strdup("there"))) != TPM_OK)
    Err("set s failed");

  if(Boa_PrintNL(stdout, context) != TPM_OK) Err("print failed");

  /* Iterate ***********************************************/
  if(Boa_StringGet(context, "_iter", 0, NULL, &iter) != TPM_OK) {
    Err("no iterator");
  }
  else {
    Obj seek;
    Boa_PrintNL(stdout, iter);

    /* relative addressing */
    if(Boa_IntGet(iter, -1, 0, NULL, &t) != TPM_OK) 
      Err("iter(-1) failed");
    if(Boa_IntGet(iter, 1, 0, NULL, NULL) == TPM_OK)
      Err("iter(1) didn't fail");
    printf("these two lines should be identical:\n");
    Boa_PrintNL(stdout, t); Boa_Release(t);
    if(Boa_IntGet(iter, 3, 0, NULL, &t) != TPM_OK)
      Err("iter(3) failed");
    Boa_PrintNL(stdout, t); Boa_Release(t);

    printf("these two lines should be identical:\n");
    if(Boa_IntGet(iter, 1, 0, NULL, &t) != TPM_OK)
      Err("iter(1) failed");
    Boa_PrintNL(stdout, t); Boa_Release(t);
    if(Boa_IntGet(iter, 0, 0, NULL, NULL) == TPM_OK)
      Err("iter past EOF didn't fail");
    if(Boa_IntGet(iter, -1, 0, NULL, NULL) == TPM_OK)
      Err("backup didn't fail");
    if(Boa_IntGet(iter, -1, 0, NULL, &t) != TPM_OK)
      Err("backup failed");
    Boa_PrintNL(stdout, t); Boa_Release(t);

    /* seeking */
    if(Boa_StringGet(iter, "seek", 0, NULL, &seek) != TPM_OK) {
      Err("no seeking");
    }
    else {
      Boa_PrintNL(stdout, seek);
      
      if(Boa_IntGet(seek, 1, 0, NULL, &t) != TPM_OK)
	Err("seek(1) failed");
      printf("1 : ");
      Boa_PrintNL(stdout, t); Boa_Release(t);
      if(Boa_StringGet(seek, "d", 0, &T_Double, &t) != TPM_OK) 
	Err("seek(d) failed");
      else {
	if(Boa_Double_Value(t) != 2.2) Err("seek(d) returned wrong value");
	Boa_Release(t);
      }
      /* now start relative again */
      if(Boa_IntGet(iter, 0, 0, &T_Double, &t) != TPM_OK)
	Err("iter(0) failed");
      Boa_Release(t);
      Boa_Release(seek);
    }
    Boa_Release(iter);
  }

  /* Delete ************************************************/
  if(Boa_StringGet(context, "z", BOA_DELETE, NULL, NULL) == TPM_OK)
    Err("del z didn't fail");
  if(Boa_PrintNL(stdout, context) != TPM_OK) Err("print failed");
  if(Boa_StringGet(context, "d", BOA_DELETE, NULL, &t) != TPM_OK)
    Err("del d failed");
  else {
    if(Boa_Double_Value(t) != 2.2) Err("del d returned wrong value");
    Boa_Release(t);
  }
  if(Boa_PrintNL(stdout, context) != TPM_OK) Err("print failed");
  if(Boa_StringGet(context, "i", BOA_DELETE, &T_Int, NULL) != TPM_OK)
    Err("del i failed");
  if(Boa_PrintNL(stdout, context) != TPM_OK) Err("print failed");
  if(Boa_StringGet(context, "s", BOA_DELETE, NULL, NULL) != TPM_OK)
    Err("del s failed");

  if(Boa_PrintNL(stdout, context) != TPM_OK) Err("print failed");

  Boa_Release(context);
  Boa_Shutdown();
  exit(0);
}
