/* Ascii formatter.
 * ObjGet with a Stream will return an Obj.
 * ObjSet with a Stream will write the Obj.
 * Dispatches on the return type to specific read/write routines.
 * Behavior object; has no data.
 * Requires: Array, Int, Float, Double, Char, String, SimpleType.
 */
#include <boa/boa.h>

/* Globals *******************************************************************/
Type T_FormatAscii;

/* Functions *****************************************************************/

static int SPrint(Obj self, StrStream ss)
{
  return StrStreamPuts(ss, "aFormatAscii");
}

static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  if(!type_in_out || !*type_in_out) {
    /* could take a guess by peeking one character, but we won't */
    return TPM_BOTTOM;
  }
  if((*type_in_out == T_Char) ||
     (*type_in_out == T_Int) ||
     (*type_in_out == T_Float) ||
     (*type_in_out == T_Double)) {
    return Boa_IntGet(stream, 1, 0, type_in_out, value_return);
  }
  else if(*type_in_out == T_String) {
    return Boa_StringGet(stream, "line", 0, type_in_out, value_return);
  }
  return TPM_BOTTOM;   /* don't know how to read it */
}

static int ObjSet(Obj self, Obj stream, int flags, Obj value)
{
  if((value->type == T_Int) ||
     (value->type == T_Float) ||
     (value->type == T_Double) ||
     (value->type == T_Char) ||
     (value->type == T_String)) {
    /* all streams should support these */
    return Boa_IntSet(stream, 1, flags, value);
  }
  return TPM_BOTTOM;   /* don't know how to write it */
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, ObjSet
};

Obj Boa_FormatAscii_Define(void)
{
  T_FormatAscii = Boa_SimpleType_Create("formatAscii", &methods, 1, 1);
  return T_FormatAscii;
}
