/* Index formatter.
 * Special case of formatDict where the values range from 0..N-1.
 * ObjGet with a Stream will return an Obj.
 * ObjSet with a Stream will write the Obj.
 * Behavior object; has no data.
 * Requires: Array, Int, Double, Char, String, SimpleType.
 */
#include <boa/boa.h>

/* Globals *******************************************************************/
Type T_FormatIndex;

/* Functions *****************************************************************/

static int SPrint(Obj self, StrStream ss)
{
  return StrStreamPuts(ss, "aFormatIndex");
}

static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  int pos = Boa_Tell(stream);
  int i = 0;
  Obj name, t, dict;

  /* Default to a T_Index */
  if(!type_in_out) dict = Boa_Create(T_Index);
  else {
    if(!*type_in_out) *type_in_out = T_Index;
    dict = Boa_Create(*type_in_out);
  }

  t = Boa_Int_Create(0);
  /* format is { line line ... } */
  /* read left brace */
  if(Boa_StringGet(stream, "word", 0, NULL, &name) != TPM_OK) goto error;
  if(Boa_String_Value(name)[0] != '{' || Boa_String_Value(name)[1]) {
    ErrorString = "missing open brace";
    goto error;
  }
  for(;;) {
    Boa_Release(name);
    /* read a line */
    if(Boa_StringGet(stream, "line", 0, NULL, &name) != TPM_OK) {
      ErrorString = "missing close brace";
      goto error;
    }
    /* is it a right brace? */
    if(Boa_String_Value(name)[0] == '}' && !Boa_String_Value(name)[1]) break;
    /* enter it */
    Boa_Int_Value(t) = i++;
    if(Boa_ObjSet(dict, name, BOA_INSERT, Boa_Hold(t)) != TPM_OK) {
      Boa_Release(t);
      Boa_Release(name);
      goto error;
    }
  }
  Boa_Release(t);
  if(value_return) *value_return = dict;
  else Boa_Release(dict);
  return TPM_OK;

 error:
  Boa_Release(t);
  Boa_Release(dict);
  /* reset the stream */
  Boa_Seek(stream, pos, 0);
  return TPM_BOTTOM;
}

static int ObjSet(Obj self, Obj stream, int flags, Obj value)
{
  return TPM_BOTTOM;
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, ObjSet
};

Obj Boa_FormatIndex_Define(void)
{
  T_FormatIndex = Boa_SimpleType_Create("formatIndex", &methods, 1, 1);
  return T_FormatIndex;
}
