/* Formatter for JPEG images. (Patterned after FormatPnm.)
 * ObjGet with a Stream will return an Obj.
 * ObjSet with a Stream will write the Obj.
 * Requires: SimpleType.
 */
#include <boa/boa.h>
#include <boa/image.h>

/* Globals *******************************************************************/
Type T_FormatJpeg;

/* Functions *****************************************************************/

static int SPrint(Obj self, StrStream ss)
{
  return StrStreamPuts(ss, "aFormatJpeg");
}

static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  TpmImage image;
  Jpeg jpeg;
  int pos;
  /* stream must be a fileHandle */
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  jpeg.fp = Boa_FileHandle_Value(stream);
  pos = ftell(jpeg.fp);
  if(pos < 0) { perror("FormatJpeg"); return TPM_BOTTOM; }
  if(JpegReadInit(&jpeg) != TPM_OK) goto error;

  BOA_TYPECHECK_GET(T_Image);
  image = TpmImageFromJpeg(&jpeg);
  jpeg.fp = NULL;
  JpegClose(&jpeg);
  if(!image) goto error;
  if(value_return) *value_return = Boa_Image_Create(image);
  return TPM_OK;

 error:
#if 0
  fprintf(stderr, "FormatJpeg: seeking back to %d\n", pos);
#endif
  fseek(Boa_FileHandle_Value(stream), pos, 0); 
  return TPM_BOTTOM;
}

static int ObjSet(Obj self, Obj stream, int flags, Obj value)
{
  if(value->type != T_Image) return TPM_BOTTOM;
  /* stream must be a fileHandle */
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  TpmImageWriteJpeg(Boa_FileHandle_Value(stream), Boa_Image_Value(value));
  Boa_Release(value);
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, ObjSet
};

Obj Boa_FormatJpeg_Define(void)
{
  T_FormatJpeg = Boa_SimpleType_Create("formatJpeg", &methods, 1, 1);
  return T_FormatJpeg;
}
