/* Formatter for PNM images.
 * ObjGet with a Stream will return an Obj.
 * ObjSet with a Stream will write the Obj.
 * Requires: SimpleType.
 */
#include <boa/boa.h>
#include <boa/image.h>

/* Globals *******************************************************************/
Type T_FormatPnm;

/* Functions *****************************************************************/

static int SPrint(Obj self, StrStream ss)
{
  return StrStreamPuts(ss, "aFormatPnm");
}

static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  TpmImage image;
  Pnm pnm;
  int pos;
  /* stream must be a fileHandle */
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  pnm.fp = Boa_FileHandle_Value(stream);
  pos = ftell(pnm.fp);
  if(pos < 0) { perror("FormatPnm"); return TPM_BOTTOM; }
  if(PnmReadInit(&pnm)) goto error;

  BOA_TYPECHECK_GET(T_Image);
  image = TpmImageFromPnm(&pnm);
  pnm.fp = NULL;
  PnmClose(&pnm);
  if(!image) goto error;
  if(value_return) *value_return = Boa_Image_Create(image);
  return TPM_OK;

 error:
#if 0
  fprintf(stderr, "FormatPnm: seeking back to %d\n", pos);
#endif
  fseek(Boa_FileHandle_Value(stream), pos, 0); 
  return TPM_BOTTOM;
}

static int ObjSet(Obj self, Obj stream, int flags, Obj value)
{
  if(value->type != T_Image) return TPM_BOTTOM;
  /* stream must be a fileHandle */
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  TpmImageWritePnm(Boa_FileHandle_Value(stream), Boa_Image_Value(value));
  Boa_Release(value);
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, ObjSet
};

Obj Boa_FormatPnm_Define(void)
{
  T_FormatPnm = Boa_SimpleType_Create("formatPnm", &methods, 1, 1);
  return T_FormatPnm;
}
