/* Formatter for SGI RGB images.
 * ObjGet with a Stream will return an Obj.
 * ObjSet with a Stream will write the Obj.
 * Requires: SimpleType.
 */
#include <boa/boa.h>
#include <gl/image.h>
#include <boa/image.h>

/* Globals *******************************************************************/
Type T_FormatSgi;

/* Functions *****************************************************************/

static int SPrint(Obj self, StrStream ss)
{
  return StrStreamPuts(ss, "aFormatSgi");
}

static void DoNothing(char *string) 
{
}

static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  TpmImage image;
  FILE *fp;
  IMAGE *img;
  long pos;
  off_t lpos;

  /* stream must be a fileHandle */
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  fp = Boa_FileHandle_Value(stream);
  pos = ftell(fp);
  if(pos < 0) { ErrorSetErrno(); return TPM_BOTTOM; }
  i_seterror(DoNothing);
  /* Force the underlying fileno to be in sync with the FILE* */
  lpos = lseek(fileno(fp), 0, SEEK_CUR);
  lseek(fileno(fp), pos, 0);
  img = fiopen(fileno(fp), "r");
  if(!img) goto error;

  BOA_TYPECHECK_GET(T_Image);
  image = TpmImageFromSgi(img);
  img->file = 0;  /* prevent iclose from closing the file */
  iclose(img);
  if(!image) goto error;
  if(value_return) *value_return = Boa_Image_Create(image);
  return TPM_OK;

 error:
#if 0
  fprintf(stderr, "FormatSgi: seeking back to %d\n", pos);
#endif
  /* Both the FILE* and its fileno must be restored to their original 
   * positions (which may differ in general).
   */
  lseek(fileno(fp), lpos, 0);
  if(fseek(Boa_FileHandle_Value(stream), pos, 0) < 0) {
    fprintf(stderr, "file not seekable\n");
  }
  return TPM_BOTTOM;
}

static int ObjSet(Obj self, Obj stream, int flags, Obj value)
{
  if(value->type != T_Image) return TPM_BOTTOM;
  /* stream must be a fileHandle */
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  TpmImageWriteSgi(Boa_FileHandle_Value(stream), Boa_Image_Value(value));
  Boa_Release(value);
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, ObjSet
};

Obj Boa_FormatSgi_Define(void)
{
  T_FormatSgi = Boa_SimpleType_Create("formatSgi", &methods, 1, 1);
  return T_FormatSgi;
}
