/* SpanTree formatter.
 * GetObj with a Stream will return an Obj.
 * SetObj with a Stream will write the Obj.
 * Requires: SpanTree.
 */
#include <boa/boa.h>
#include <tpm/spanTree.h>

/* no data */

/* Globals *******************************************************************/
Type T_FormatSpanTree;

/* Functions *****************************************************************/

static int SPrint(Obj self, StrStream ss)
{
  return StrStreamPuts(ss, "aFormatSpanTree");
}

/* stream must be a FileHandle */
static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  char *line;
  int i, status;
  Type type;
  Obj t;
  int pos;
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  pos = Boa_Tell(stream);

  /* read the type header */
  line = Boa_GetLine(stream);
  /* determine parameters from the header */
  status = sscanf(line, "%d", &i);
  free(line);
  if(status < 1) goto error;

  /* was a type expected? */
  if(!type_in_out || !*type_in_out) {
    char str[128];
    sprintf(str, "(spanTree %d)", i);
    type = Boa_Type_Create(str);
    if(type_in_out) *type_in_out = type;
  }
  else {
    type = *type_in_out;
    /* type mismatch? */
    if(Boa_SpanTree_Length(type) != i) goto error;
  }

  t = Boa_Create(type);
  if(SpanTreeRead(Boa_SpanTree_Value(t), Boa_FileHandle_Value(stream))) {
    Boa_Release(t); goto error;
  }
  if(value_return) *value_return = t;
  else Boa_Release(t);
  return TPM_OK;

 error:
  Boa_Seek(stream, pos, 0);
  return TPM_BOTTOM;
}

/* stream must be a FileHandle */
static int ObjSet(Obj self, Obj stream, int flags, Obj value)
{
  if(value->type != T_SpanTree) return TPM_BOTTOM;
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  if(SpanTreeWrite(Boa_SpanTree_Value(value), Boa_FileHandle_Value(stream))) {
    return TPM_BOTTOM;
  }
  Boa_Release(value);
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, ObjSet
};

Obj Boa_FormatSpanTree_Define(void)
{
  T_FormatSpanTree = Boa_SimpleType_Create("formatSpanTree", &methods, 1, 1);
  return T_FormatSpanTree;
}
