/* Formatter for TIFF images. (Patterned after FormatPnm.)
 * ObjGet with a Stream will return an Obj.
 * ObjSet with a Stream will write the Obj.
 * Requires: SimpleType.
 */
#include <boa/boa.h>
#include <boa/image.h>

/* Globals *******************************************************************/
Type T_FormatTiff;

/* Functions *****************************************************************/

static int SPrint(Obj self, StrStream ss)
{
  return StrStreamPuts(ss, "aFormatTiff");
}

static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  TpmImage image;
  FILE *fp;
  int pos;
  /* stream must be a fileHandle */
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  fp = Boa_FileHandle_Value(stream);
  pos = ftell(fp);
  if(pos < 0) { perror("FormatTiff"); return TPM_BOTTOM; }
  image = TpmReadTiffStream(fp);
  if(!image) goto error;

  /* XXX if this fails, we must free image */
  BOA_TYPECHECK_GET(T_Image);
  if(value_return) *value_return = Boa_Image_Create(image);
  return TPM_OK;

 error:
#if 0
  fprintf(stderr, "FormatTiff: seeking back to %d\n", pos);
#endif
  fseek(Boa_FileHandle_Value(stream), pos, 0); 
  return TPM_BOTTOM;
}

static int ObjSet(Obj self, Obj stream, int flags, Obj value)
{
  if(value->type != T_Image) return TPM_BOTTOM;
  /* stream must be a fileHandle */
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  TpmImageWriteTiff(Boa_FileHandle_Value(stream), Boa_Image_Value(value));
  Boa_Release(value);
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, ObjSet
};

Obj Boa_FormatTiff_Define(void)
{
  T_FormatTiff = Boa_SimpleType_Create("formatTiff", &methods, 1, 1);
  return T_FormatTiff;
}
