/* Image package.
 * Types:
 *   image - interface to libimage
 * Formatters:
 *   pnm, raw, jpeg, tiff, sgi
 * Pnm, Jpeg, Tiff, and Sgi are made objDir defaults.
 */
#include <boa/boa.h>
#include <boa/image.h>

void Boa_Image_Pkg(void)
{
  Obj t, formatter;
  if(T_Image) return;  /* don't initialize twice */
  Boa_StringSet(Boa_Types, "image", BOA_INSERT, Boa_Image_Define());
  Boa_StringSet(Boa_Types, "formatPnm", BOA_INSERT, Boa_FormatPnm_Define());
  Boa_StringSet(Boa_Types, "formatRaw", BOA_INSERT, Boa_FormatRaw_Define());
  /* Make PNM an automatic format */
  t = Boa_Create(T_FormatPnm); 
  Boa_StringSet(Boa_Formatters, "pnm", BOA_INSERT, Boa_Hold(t));
  Boa_StringSet(Boa_ObjDir_DefaultFormatter, "_/pnm", '/'|BOA_INSERT, t);
#if 0
  /* adding pnm to binary allows us to write arrays of images,
   * but causes single images to be written with BOA binary header. */
  formatter = Boa_GetFormatter("binary");
  Boa_StringSet(formatter, "_/pnm", '/'|BOA_INSERT, t);
  Boa_Release(formatter);
#endif
#ifndef NO_JPEG
  Boa_StringSet(Boa_Types, "formatJpeg", BOA_INSERT, Boa_FormatJpeg_Define());
  /* Make JPEG an automatic format */
  t = Boa_Create(T_FormatJpeg);
  Boa_StringSet(Boa_Formatters, "jpeg", BOA_INSERT, Boa_Hold(t));
  Boa_StringSet(Boa_ObjDir_DefaultFormatter, "_/jpeg", '/'|BOA_INSERT, t);
#endif
#ifndef NO_SGI
  Boa_StringSet(Boa_Types, "formatSgi", BOA_INSERT, Boa_FormatSgi_Define());
  /* Make SGI an automatic format */
  t = Boa_Create(T_FormatSgi);
  Boa_StringSet(Boa_Formatters, "sgi", BOA_INSERT, Boa_Hold(t));
  Boa_StringSet(Boa_ObjDir_DefaultFormatter, "_/sgi", '/'|BOA_INSERT, t);
#endif
#ifndef NO_TIFF
  Boa_StringSet(Boa_Types, "formatTiff", BOA_INSERT, Boa_FormatTiff_Define());
  /* Make TIFF an automatic format */
  t = Boa_Create(T_FormatTiff);
  Boa_StringSet(Boa_Formatters, "tiff", BOA_INSERT, Boa_Hold(t));
  Boa_StringSet(Boa_ObjDir_DefaultFormatter, "_/tiff", '/'|BOA_INSERT, t);
#endif
}

Obj Boa_Image_Create(TpmImage image)
{
  Obj t = Boa_Create(T_Image);
  Boa_Image_Value(t) = image;
  return t;
}
