/* The always context.
 * Any get returns the same object.
 * Any set changes that object.
 */
#include <boa/boa.h>

typedef struct Always {
  Obj value;
} *Always;

/* Globals *******************************************************************/
Type T_Always;

/* Functions *****************************************************************/

static void Final(Obj self)
{
  Always al = (Always)self->value;
  if(al->value) Boa_Release(al->value);
}

static void Clean(Obj self, ObjFunc *func, Any data)
{
  Always al = (Always)self->value;
  if(al->value) func(al->value, data);
}

static int SPrint(Obj self, StrStream ss)
{
  Always al = (Always)self->value;
  StrStreamPuts(ss, "always");
  if(!al->value) return TPM_OK;
  return Boa_SPrint(al->value, ss);
}

static int StringGet(Obj self, char *name, int flags,
		     Type *type_in_out, Obj *value_return)
{
  Always al = (Always)self->value;
  if(!al->value) return TPM_BOTTOM;
  BOA_TYPECHECK_GET(al->value->type);
  if(value_return) {
    Boa_Hold(al->value);
    *value_return = al->value;
    if(flags & BOA_DELETE) {
      Boa_Release(al->value);
      al->value = NULL;
    }
  }
  return TPM_OK;
}
static int IntGet(Obj self, int index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  return StringGet(self, NULL, flags, type_in_out, value_return);
}
static int ObjGet(Obj self, Obj index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  return StringGet(self, NULL, flags, type_in_out, value_return);
}

static int StringSet(Obj self, char *name, int flags, Obj value)
{
  Always al = (Always)self->value;
  if(al->value) Boa_Release(al->value);
  al->value = value;
  return TPM_OK;
}
static int IntSet(Obj self, int index, int flags, Obj value)
{
  return StringSet(self, NULL, flags, value);
}
static int ObjSet(Obj self, Obj index, int flags, Obj value)
{
  return StringSet(self, NULL, flags, value);
}

static struct Boa_Methods methods = {
  Empty_Init, Final, Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  StringGet, StringSet,
  IntGet, IntSet,
  ObjGet, ObjSet
};

Obj Boa_Always_Define(void)
{
  T_Always = Boa_SimpleType_Create("always", &methods, 
				   sizeof(struct Always),
				   sizeof(Ptr));
  return T_Always;
}
