/* Debugging context.  Every request sends diagnostic info to stderr. 
 */
#include <boa/boa.h>

#define ECHO 0

/* Globals *******************************************************************/
Type T_Debug;

/* Functions *****************************************************************/

static int StringGet(Obj self, char *name, int flags,
		     Type *type_in_out, Obj *value_return)
{
  fprintf(stderr, "debug StringGet(%s) flags=%x type_in_out=", name, flags);
  if(!type_in_out) fprintf(stderr, "N");
  else if(!*type_in_out) fprintf(stderr, "(N)");
  else Boa_Print(stderr, *type_in_out);
  fprintf(stderr, "\n");
  ErrorString = BOA_EMETH;
  return TPM_BOTTOM;
}
static int StringSet(Obj self, char *name, int flags, Obj value)
{
  fprintf(stderr, "debug StringSet(%s) flags=%x value=", name, flags);
  Boa_PrintNL(stderr, value);
  return TPM_OK;
}

static int IntGet(Obj self, int index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  fprintf(stderr, "debug IntGet(%d) flags=%x type_in_out=", index, flags);
  if(!type_in_out) fprintf(stderr, "N");
  else if(!*type_in_out) fprintf(stderr, "(N)");
  else Boa_Print(stderr, *type_in_out);
  fprintf(stderr, "\n");
  ErrorString = BOA_EMETH;
  return TPM_BOTTOM;
}
static int IntSet(Obj self, int index, int flags, Obj value)
{
  fprintf(stderr, "debug IntSet(%d) flags=%x value=", index, flags);
  Boa_PrintNL(stderr, value);
  return TPM_OK;
}

static int ObjGet(Obj self, Obj index, int flags,
		     Type *type_in_out, Obj *value_return)
{
  fprintf(stderr, "debug ObjGet(");
  Boa_Print(stderr, index);
  fprintf(stderr, ") flags=%x type_in_out=", flags);
  if(!type_in_out) fprintf(stderr, "N");
  else if(!*type_in_out) fprintf(stderr, "(N)");
  else Boa_Print(stderr, *type_in_out);
  fprintf(stderr, "\n");
#if ECHO
  BOA_TYPECHECK_GET(index->type);
  if(value_return) *value_return = Boa_Hold(index);
  return TPM_OK;
#else
  ErrorString = BOA_EMETH;
  return TPM_BOTTOM;
#endif
}
static int ObjSet(Obj self, Obj index, int flags, Obj value)
{
  fprintf(stderr, "debug ObjSet(");
  Boa_Print(stderr, index);
  fprintf(stderr, ") flags=%x value=", flags);
  Boa_PrintNL(stderr, value);
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  Generic_SPrint, Generic_Copy, Generic_Compare, Generic_Hash,
  StringGet, StringSet,
  IntGet, IntSet,
  ObjGet, ObjSet
};


Obj Boa_Debug_Define(void)
{
  T_Debug = Boa_SimpleType_Create("debug", &methods, 1, 1);
  return T_Debug;
}
