/* The Indexer type makes it easy to create indexer objects.
 * Provide it with data and a int->obj or obj->int function.
 */
#include <boa/boa.h>

/* must synchronize with macros in std.h */
typedef struct Indexer {
  Obj data;
  Boa_IndexerObjFunc *indexObj;
  Boa_IndexerIntFunc *indexInt;
} *Indexer;

/* Globals *******************************************************************/
Type T_Indexer;

/* Functions *****************************************************************/

static Obj DefaultIndexInt(Obj self, int index)
{
  return NULL;
}
static int DefaultIndexObj(Obj self, Obj index)
{
  return NAI;
}

/* set the fcns to always return error */
static void Init(Obj self)
{
  Indexer indexer = (Indexer)self->value;
  indexer->indexObj = DefaultIndexObj;
  indexer->indexInt = DefaultIndexInt;
}

static void Final(Obj self)
{
  Indexer indexer = (Indexer)self->value;
  if(indexer->data) Boa_Release(indexer->data);
}

static void Clean(Obj self, ObjFunc *func, Any data)
{
  Indexer indexer = (Indexer)self->value;
  if(indexer->data) func(indexer->data, data);
}

static int SPrint(Obj self, StrStream ss)
{
  Indexer indexer = (Indexer)self->value;
  StrStreamPuts(ss, "indexer");
  if(indexer->data) return Boa_SPrint(indexer->data, ss);
  return StrStreamPuts(ss, "(NULL)");
}

static int IntGet(Obj self, int index, int flags, 
		  Type *type_in_out, Obj *value_return)
{
  Indexer indexer = (Indexer)self->value;
  /* return the conversion to Obj */
  Obj t = indexer->indexInt(indexer->data, index);
  if(!t) return TPM_BOTTOM;
  BOA_TYPECHECK_GET(t->type);
  if(value_return) *value_return = t;
  return TPM_OK;
}

static int ObjGet(Obj self, Obj index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  Indexer indexer = (Indexer)self->value;
  /* return the conversion to integer */
  int i = indexer->indexObj(indexer->data, index);
  if(i == NAI) return TPM_BOTTOM;
  BOA_TYPECHECK_GET(T_Int);
  if(value_return) *value_return = Boa_Int_Create(i);
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Init, Final, Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Generic_StringGet, Err_StringSet,
  IntGet, Err_IntSet,
  ObjGet, Err_ObjSet
};

Obj Boa_Indexer_Define(void)
{
  T_Indexer = Boa_SimpleType_Create("indexer", &methods,
				    sizeof(struct Indexer),
				    sizeof(Ptr));
  return T_Indexer;
}
