/* List of Objs.
 * Attributes
 *   length
 *   iter
 */
#include <tpm/list.h>
#include <boa/boa.h>

/* Globals *******************************************************************/
Type T_List;

/* Functions *****************************************************************/

static void Init(Obj self)
{
  List list = (List)self->value;
  List new = ListCreate((FreeFunc*)Boa_Free);
  *list = *new;
  free(new);
}

static void Final(Obj self)
{
  List list = (List)self->value;
  ListFlush(list);
}

static void Clean(Obj self, ObjFunc *func, Any data)
{
  List list = (List)self->value;
  Obj t;
  ListIter(p, t, list) {
    func(t, data);
  }
}

/* prints like a Tcl list */
static int SPrint(Obj self, StrStream ss)
{
  List list = (List)self->value;
  Obj t;
  StrStreamPutc(ss, '{');
  {ListIter(p, t, list) {
    TRY_RETURN(Boa_SPrint(t, ss));
    StrStreamPutc(ss, ' ');
  }}
  return StrStreamPutc(ss, '}');
}

static int Compare(Obj self, Obj b)
{
  return ListCompare(Boa_List_Value(self), Boa_List_Value(b), 
		     (CmpFunc*)BoaCompare);
}
static int Hash(Obj self)
{
  int result=1;
  Obj t;
  ListIter(p, t, Boa_List_Value(self)) {
    result += Boa_Hash(t);
  }
  return result * 1103515245;
}

static int StringGet(Obj self, char *name, int flags,
		     Type *type_in_out, Obj *value_return)
{
  List list = (List)self->value;
  if(name[0] == '_') {
    if(!strcmp(name+1, "length")) {
      BOA_TYPECHECK_GET(T_Int);
      if(value_return) *value_return = Boa_Int_Create(ListSize(list));
      return TPM_OK;
    }
#if 0
    else if(!strcmp(name+1, "iter")) {
      Obj iter = Boa_Create(T_Iter);
      Boa_Iter_Data(iter) = self;
      Boa_StringGet(Boa_Globals, "atoi", 0, NULL, &Boa_Iter_Indexer(iter));
      *value_return = iter;
      return TPM_OK;
    }
#endif
  }
  return TPM_BOTTOM;
}

static int IntGet(Obj self, int index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  List list = (List)self->value;
  Obj t;
  if((index < 0) || (index >= ListSize(list))) return TPM_BOTTOM;
  if(flags & BOA_DELETE) {
    ListRemoveAtIndex(list, index, (void**)&t);
    BOA_TYPECHECK_GET(t->type);
    if(value_return) *value_return = t;
    else Boa_Release(t);
    return TPM_OK;
  }
  t = ListValueAtIndex(list, index);
  BOA_TYPECHECK_GET(t->type);
  if(value_return) {
    Boa_Hold(t);
    *value_return = t;
  }
  return TPM_OK;
}

/* replace an element or append to the end */
static int IntSet(Obj self, int index, int flags, Obj value)
{
  List list = (List)self->value;
  ListNode p;
  if((index < 0) || (index > ListSize(list))) return TPM_BOTTOM;
  if(index == ListSize(list)) {
    if(!(flags & BOA_INSERT)) { ErrorString = BOA_EFLAGS; return TPM_BOTTOM; }
    ListAddRear(list, value);
    return TPM_OK;
  }
  if(!(flags & BOA_REPLACE)) { ErrorString = BOA_EFLAGS; return TPM_BOTTOM; }
  p = ListNodeAtIndex(list, index);
  Boa_Release((Obj)p->data);
  p->data = value;
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Init, Final, Clean,
  SPrint, Err_Copy, Compare, Hash,
  StringGet, Err_StringSet,
  IntGet, IntSet,
  Generic_ObjGet, Generic_ObjSet
};

Obj Boa_List_Define(void)
{
  T_List = Boa_SimpleType_Create("list", &methods,
				 sizeof(struct ListStruct), sizeof(Ptr));
  return T_List;
}
