#include <boa/boa.h>
#include <boa/vector.h>
#include <boa/image.h>

/* Globals *******************************************************************/
Type T_Matrix;

/* Functions *****************************************************************/

Obj Boa_Matrix_Create(Matrix v)
{
  Obj t = Boa_Create(T_Matrix);
  Boa_Matrix_Value(t) = v;
  return t;
}

Obj Boa_Matrix_Coerce(Obj t)
{
  if(t->type == T_Image) {
    Matrix m = MatrixFromImage(Boa_Image_Value(t));
    Boa_Release(t);
    t = Boa_Matrix_Create(m);
  }
  else if((t->type == T_Volume) && (Boa_Volume_Value(t)->zdim == 1)) {
    Matrix m = MatrixFromVolume(Boa_Volume_Value(t), 0);
    Boa_Release(t);
    t = Boa_Matrix_Create(m);
  }
  return t;
}

static void Final(Obj self)
{
  if(Boa_Matrix_Value(self)) MatrixFree(Boa_Matrix_Value(self));
}

/* prints like an array of arrays */
static int SPrint(Obj self, StrStream ss)
{
  StrStreamPutc(ss, '{');
  if(Boa_Matrix_Value(self)) {
    Matrix m = Boa_Matrix_Value(self);
    int i,j;
    for(i=0;i<m->rows;i++) {
      StrStreamPutc(ss, '{');
      for(j=0;j<m->cols;j++) {
	StrStreamPutDouble(ss, m->d2[i][j]);
	StrStreamPutc(ss, ' ');
      }
      StrStreamPutc(ss, '}');
    }
  }
  else StrStreamPuts(ss, "{}");
  return StrStreamPutc(ss, '}');
}

static int Copy(Obj self, Ptr value_return)
{
  *(Matrix*)value_return = MatrixCopy(Boa_Matrix_Value(self));
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Final, Empty_Clean,
  SPrint, Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  Generic_ObjGet, Generic_ObjSet
};

Obj Boa_Matrix_Define(void)
{
  T_Matrix = Boa_SimpleType_Create("matrix", &methods, 
				   sizeof(Matrix), sizeof(Ptr));
  return T_Matrix;
}
