/* TclCustom context. All methods are implemented by Tcl commands.
 * Attributes:
 *   interp     Tcl_Interp
 */
#include <boa/boa.h>
#include <boa/tcl.h>

typedef struct Custom {
  Obj tcl;
} *Custom;

/* Globals *******************************************************************/
Type T_TclCustom;

/* Functions *****************************************************************/

static void Init(Obj self)
{
  Custom c = (Custom)self->value;
#if 0
  Boa_StringGet(Boa_Globals, "tcl", 0, NULL, &c->tcl);
#else
  c->tcl = Boa_Create(T_Tcl);
#endif
}

static void Final(Obj self)
{
  Custom c = (Custom)self->value;
  Boa_Release(c->tcl);
}

static void Clean(Obj self, ObjFunc *func, Any data)
{
  Custom c = (Custom)self->value;
  func(c->tcl, data);
}

static int SPrint(Obj self, StrStream ss)
{
  return Generic_SPrint(self, ss);
}

static int Copy(Obj self, Ptr value_return)
{
  Custom c = (Custom)self->value;
  Custom new = (Custom)value_return;
  new->tcl = Boa_Hold(c->tcl);
  return TPM_OK;
}

static int Call(Tcl_Interp *interp, char *command, char *arg1, char *arg2,
		Type *type_in_out, Obj *value_return)
{
  StrStream ss = StrStreamCreate(1024, STRSTREAM_UNLIMITED);
  int status;

  /* call the stringGet command on the name */
  StrStreamPuts(ss, command);
  StrStreamPutc(ss, ' ');
  StrStreamPuts(ss, arg1);
  if(arg2) {
    StrStreamPutc(ss, ' ');
    StrStreamPuts(ss, arg2);
  }
  status = Tcl_Eval(interp, StrStreamAsString(ss));
  StrStreamFree(ss);

  if(status != TCL_OK) {
    ErrorSet("%s", interp->result);
    return TPM_BOTTOM;
  }
  if(type_in_out || value_return) {
    Obj t = BoaTclToObj(interp, interp->result);
    if(!t) return TPM_BOTTOM;
    BOA_TYPECHECK_GET(t->type);
    if(value_return) *value_return = t;
    else Boa_Release(t);
  }
  return TPM_OK;
}

static int StringGet(Obj self, char *name, int flags,
		     Type *type_in_out, Obj *value_return)
{
  Custom c = (Custom)self->value;
  if(flags & BOA_SPLIT) {
    return Generic_SplitGet(self, name, flags, type_in_out, value_return);
  }
  if(name[0] == '_') {
    if(!strcmp(name+1, "interp")) {
      BOA_TYPECHECK_GET(T_Tcl);
      if(value_return) *value_return = Boa_Hold(c->tcl);
      return TPM_OK;
    }
  }
  return Call(Boa_Tcl_Interp(c->tcl), "stringGet", name, NULL, 
	      type_in_out, value_return);
}
static int StringSet(Obj self, char *name, int flags, Obj value)
{
  Custom c = (Custom)self->value;
  Obj t;
  int status;
  if(flags & BOA_SPLIT) {
    return Generic_SplitSet(self, name, flags, value);
  }
  if(name[0] == '_') {
    if(!strcmp(name+1, "interp")) {
      if(value->type != T_Tcl) return TPM_ERROR;
      Boa_Release(c->tcl);
      c->tcl = value;
      return TPM_OK;
    }
  }
  t = BoaObjToTcl(value);
  status = Call(Boa_Tcl_Interp(c->tcl), "stringSet", name, Boa_String_Value(t),
		NULL, NULL);
  Boa_Release(t);
  return status;
}

static int IntGet(Obj self, int index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  return Err_IntGet(self, index, flags, type_in_out, value_return);
}
static int IntSet(Obj self, int index, int flags, Obj value)
{
  return Err_IntSet(self, index, flags, value);
}

static int ObjGet(Obj self, Obj index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  return Err_ObjGet(self, index, flags, type_in_out, value_return);
}
static int ObjSet(Obj self, Obj index, int flags, Obj value)
{
  return Err_ObjSet(self, index, flags, value);
}

static struct Boa_Methods methods = {
  Init, Final, Clean,
  SPrint, Copy, Generic_Compare, Generic_Hash,
  StringGet, StringSet,
  IntGet, IntSet,
  ObjGet, ObjSet
};

Obj Boa_TclCustom_Define(void)
{
  T_TclCustom = Boa_SimpleType_Create("tclCustom", &methods,
				      sizeof(struct Custom), sizeof(Obj));
  return T_TclCustom;
}
