/* Fixed-length array of Objs.
 * Each slot is initially empty.
 * Attributes
 *   iter
 */
#include <boa/boa.h>

typedef Obj *Tuple;

/* Globals *******************************************************************/
Type T_Tuple;

/* Functions *****************************************************************/

#define length Boa_Int_Value(Boa_PType_Item(self->type, 0))

static void Final(Obj self)
{
  Tuple tuple = (Tuple)self->value;
  int i;
  for(i=0;i<length;i++) if(tuple[i]) Boa_Release(tuple[i]);
}

static void Clean(Obj self, ObjFunc *func, Any data)
{
  Tuple tuple = (Tuple)self->value;
  int i;
  for(i=0;i<length;i++) if(tuple[i]) func(tuple[i], data);
}

/* prints like a Tcl list */
static int SPrint(Obj self, StrStream ss)
{
  Tuple tuple = (Tuple)self->value;
  int i;
  StrStreamPutc(ss, '{');
  for(i=0;i<length;i++) {
    if(tuple[i]) TRY_RETURN(Boa_SPrint(tuple[i], ss));
    else StrStreamPuts(ss, "NULL");
    StrStreamPutc(ss, ' ');
  }
  return StrStreamPutc(ss, '}');
}

static int StringGet(Obj self, char *name, int flags, 
		     Type *type_in_out, Obj *value_return)
{
  Tuple tuple = (Tuple)self->value;
  if(name[0] == '_') {
    if(!strcmp(name+1, "iter")) {
      Obj iter = Boa_Create(T_Iter);
      Boa_Iter_Data(iter) = self;
      Boa_StringGet(Boa_Globals, "atoi", 0, NULL, &Boa_Iter_Indexer(iter));
      *value_return = iter;
      return TPM_OK;
    }
  }
  return TPM_BOTTOM;
}

static int IntGet(Obj self, int index, int flags, 
		  Type *type_in_out, Obj *value_return)
{
  Tuple tuple = (Tuple)self->value;
  if((index < 0) || (index >= length)) return TPM_BOTTOM;
  if(!tuple[index]) return TPM_BOTTOM;
  if(flags & BOA_DELETE) { 
    BOA_TYPECHECK_GET(tuple[index]->type);
    if(value_return) *value_return = tuple[index];
    else Boa_Release(tuple[index]);
    tuple[index] = NULL;
    return TPM_OK;
  }
  BOA_TYPECHECK_GET(tuple[index]->type);
  if(value_return) *value_return = Boa_Hold(tuple[index]);
  return TPM_OK;
}

static int IntSet(Obj self, int index, int flags, Obj value)
{
  Tuple tuple = (Tuple)self->value;
  if((index < 0) || (index >= length)) return TPM_BOTTOM;
  if(tuple[index]) {
    /* must do a replace */
    if(!(flags & BOA_REPLACE)) { ErrorString = BOA_EFLAGS; return TPM_ERROR; }
    Boa_Release(tuple[index]);
  }
  else {
    /* must do an insert or replace */
    if(!((flags & BOA_REPLACE) || (flags & BOA_INSERT))) { 
      ErrorString = BOA_EFLAGS; return TPM_ERROR; 
    }
  }
  tuple[index] = value;
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Final, Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  StringGet, Err_StringSet,
  IntGet, IntSet,
  Generic_ObjGet, Generic_ObjSet
};

/* assumes num_args and args have already been filled in */
static void Type_Init(Obj self)
{
  Boa_ParamTypeInstance data = (Boa_ParamTypeInstance)self->value;
  data->methods = &methods;
  data->size = Boa_Int_Value(data->args[0]) * sizeof(Obj);
  data->align = sizeof(Ptr);
}

Obj Boa_Tuple_Define(void)
{
  T_Tuple = Boa_ParamType_Create("tuple", Type_Init, 1, T_Int);
  return T_Tuple;
}
