#include <boa/boa.h>
#include <boa/vector.h>
#include <boa/image.h>

/* Globals *******************************************************************/
Type T_Volume;

/* Functions *****************************************************************/

Obj Boa_Volume_Create(Volume v)
{
  Obj t = Boa_Create(T_Volume);
  Boa_Volume_Value(t) = v;
  return t;
}

Obj Boa_Volume_Coerce(Obj t)
{
  if(t->type != T_Volume) {
    if(t->type == T_Image) {
      Volume v = VolumeFromImage(Boa_Image_Value(t));
      Boa_Release(t);
      t = Boa_Volume_Create(v);
    }
    else {
      /* last resort; make it a matrix first */
      t = Boa_Matrix_Coerce(t);
      if(t->type == T_Matrix) {
	Volume v = VolumeFromMatrix(Boa_Matrix_Value(t), 1);
	Boa_Release(t);
	t = Boa_Volume_Create(v);
      }
    }
  }
  return t;
}

static void Final(Obj self)
{
  if(Boa_Volume_Value(self)) VolumeFree(Boa_Volume_Value(self));
}

/* prints like an array of arrays of arrays */
static int SPrint(Obj self, StrStream ss)
{
  StrStreamPutc(ss, '{');
  if(Boa_Volume_Value(self)) {
    Volume v = Boa_Volume_Value(self);
    int i,j,k;
    for(i=0;i<v->zdim;i++) {
      StrStreamPutc(ss, '{');
      for(j=0;j<v->ydim;j++) {
	StrStreamPutc(ss, '{');
	for(k=0;k<v->xdim;k++) {
	  StrStreamPutDouble(ss, v->d3[i][j][k]);
	  StrStreamPutc(ss, ' ');
	}
	StrStreamPutc(ss, '}');
      }
      StrStreamPutc(ss, '}');
    }
  }
  else StrStreamPuts(ss, "{{}}");
  return StrStreamPutc(ss, ')');
}

static int Copy(Obj self, Ptr value_return)
{
  *(Volume*)value_return = VolumeCopy(Boa_Volume_Value(self));
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Final, Empty_Clean,
  SPrint, Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  Generic_ObjGet, Generic_ObjSet
};

Obj Boa_Volume_Define(void)
{
  T_Volume = Boa_SimpleType_Create("volume", &methods, 
				   sizeof(Volume), sizeof(Ptr));
  return T_Volume;
}
