/* Tcl package.
 * Types:
 *   tcl - interface to a Tcl interpreter
 *   tclCustom - object whose methods are defined in Tcl
 * Formatters:
 *   tcl - converts Tcl strings into values
 *   tclEval - evaluates a Tcl expression to get a value
 *   These use a default Tcl interpreter which shares globals with Boa.
 */
#include <boa/boa.h>
#include <boa/tcl.h>

void Boa_Tcl_Pkg(void)
{
  Obj t, tcl;
  if(T_Tcl) return;  /* don't init twice */

  Boa_StringSet(Boa_Types, "tcl", BOA_INSERT, Boa_Tcl_Define());
  Boa_StringSet(Boa_Types, "formatTcl", BOA_INSERT, Boa_FormatTcl_Define());
  Boa_StringSet(Boa_Types, "formatTclEval", BOA_INSERT, Boa_FormatTclEval_Define());
  Boa_StringSet(Boa_Types, "tclCustom", BOA_INSERT, Boa_TclCustom_Define());

  tcl = Boa_Create(T_Tcl);
  /* expose Boa_Globals in the tcl_interp. this should be done another way. */
  Boa_StringGet(Boa_Globals, "dlopen", 0, NULL, &t);
  Boa_StringSet(tcl, "dlopen", BOA_INSERT, BoaObjToTcl(t));
  Boa_StringGet(Boa_Globals, "dlsym", 0, NULL, &t);
  Boa_StringSet(tcl, "dlsym", BOA_INSERT, BoaObjToTcl(t));
  Boa_StringGet(Boa_Globals, "atoi", 0, NULL, &t);
  Boa_StringSet(tcl, "atoi", BOA_INSERT, BoaObjToTcl(t));

  t = Boa_Create(T_FormatTcl);
  Boa_FormatTcl_Interp(t) = Boa_Hold(tcl);
  Boa_StringSet(Boa_Formatters, "tcl", BOA_INSERT, t);
  t = Boa_Create(T_FormatTclEval);
  Boa_FormatTcl_Interp(t) = Boa_Hold(tcl);
  Boa_StringSet(Boa_Formatters, "tclEval", BOA_INSERT, t);

  Boa_StringSet(Boa_Globals, "tcl", BOA_INSERT, tcl);
}

Obj BoaObjToTcl(Obj t)
{
  char *str = Allocate(20, char);
  sprintf(str, "%ld", (long)t);
  return Boa_String_Create(str);
}

/* Converts from a string containing an integer to an Obj.
 * Caller does not own the resulting object.
 * Returns NULL on failure.
 */
Obj BoaTclToObj(Tcl_Interp *interp, char *s)
{
  Obj t;
  /* convert from string to Obj */
  if(sizeof(Obj) == sizeof(int)) {
    if(Tcl_GetInt(interp, s, (int*)&t) != TCL_OK) return NULL;
  }
  else {
#ifdef __alpha
    if(Tcl_GetDouble(interp, s, (double*)&t) != TCL_OK) return NULL;
    /* the compiler will not allow a direct conversion from double -> ptr */
    t = (Obj)(long)*(double*)&t;
#endif
  }
  return t;
}

Obj BoaTclStringGet(Obj tcl, char *name)
{
  Obj s, t;
  int status = Boa_StringGet(tcl, name, 0, &T_String, &s);
  if(status != TPM_OK) return NULL;
  t = BoaTclToObj(Boa_Tcl_Interp(tcl), Boa_String_Value(s));
  Boa_Release(s);
  return t;
}


/* Set the "boaDir" variable in the tclEval formatter */
int BoaSetTclEvalDir(Obj od)
{
  Obj merge, t;
  /* set tclEval's od */
  merge = Boa_GetFormatter("tclEval");
  Boa_StringGet(merge, "_interp", 0, NULL, &t);
  Boa_Release(merge);
  Boa_StringSet(t, "boaDir", 0, BoaObjToTcl(od));
  Boa_Release(t);
  return TPM_OK;
}
