/* Vector package.
 * Types:
 *   vector, matrix, volume (from libarray)
 * Formatters:
 *   vector, vectorRaw
 * FormatVector is added as a default to binary and an option for boa.
 */
#include <boa/boa.h>
#include <boa/vector.h>

void Boa_Vector_Pkg(void)
{
  Obj t, formatter;
  if(T_Vector) return; /* don't init twice */

  Boa_StringSet(Boa_Types, "vector", BOA_INSERT, Boa_Vector_Define());
  Boa_StringSet(Boa_Types, "matrix", BOA_INSERT, Boa_Matrix_Define());
  Boa_StringSet(Boa_Types, "volume", BOA_INSERT, Boa_Volume_Define());
  Boa_StringSet(Boa_Types, "formatVector", BOA_INSERT, Boa_FormatVector_Define());
  Boa_StringSet(Boa_Types, "formatVectorAscii", BOA_INSERT, Boa_FormatVectorAscii_Define());
  Boa_StringSet(Boa_Types, "formatVectorRaw", BOA_INSERT, Boa_FormatVectorRaw_Define());

  t = Boa_Create(T_FormatVector);
  Boa_Hold(t);
  Boa_StringSet(Boa_Formatters, "vector", BOA_INSERT, t);
  /* adding vector to binary allows us to write arrays of vectors */
  formatter = Boa_GetFormatter("binary");
  Boa_StringSet(formatter, "_/vector", '/'|BOA_INSERT, t);
  Boa_Release(formatter);
  Boa_StringSet(Boa_FormatBoa_DefaultFormatters, "vector", BOA_INSERT, 
		Boa_String_Create(strdup("vector")));

  t = Boa_Create(T_FormatVectorAscii);
  Boa_Hold(t);
  Boa_StringSet(Boa_Formatters, "vectorAscii", BOA_INSERT, t);
  /* adding vector to ascii allows us to write arrays of vectors */
  formatter = Boa_GetFormatter("ascii");
  Boa_StringSet(formatter, "_/vector", '/'|BOA_INSERT, t);
  Boa_Release(formatter);
  Boa_StringSet(Boa_FormatBoa_DefaultFormatters, "vectorAscii", BOA_INSERT, 
		Boa_String_Create(strdup("vectorAscii")));
}
