#include <tpm/tree.h>
#include <tpm/spanTree.h>
#include <tpm/matrix.h>

/* Returns, via dists, a row of the dissimilarity matrix. 
 * Only needs to fill in cols entries of the row.
 */
typedef int Clus_RowFunc(int row, int cols, double *dists, Any data);

/* slink *********************************************************************/
Tree Clus_SingleLink(int nsamples, Clus_RowFunc *dist, Any data);

/* mst ***********************************************************************/
SpanTree Clus_MST(int nsamples, Clus_RowFunc *dist, Any data);

/* em ************************************************************************/
double Clus_EM(Matrix sample, Matrix center, Matrix cov,
	       Matrix mbr, double temperature,
	       char *center_file, char *icov_file, char *mbr_file);
double Clus_EMStepE(Matrix sample, Matrix center, Matrix cov,
		    Matrix mbr, double temperature);
double Clus_EMStepEP(Matrix sample, Matrix center, Matrix cov, Vector prior,
		    Matrix mbr, double temperature);
void Clus_EMStepM(Matrix sample, Matrix center, Matrix mbr);
void Clus_EMStepV(Matrix sample, Matrix center, Matrix cov, Matrix mbr);
void Clus_EMStepP(Vector prior, Matrix mbr);
void VectorPerturb(Vector v, double noise_amp);
void Clus_EMInitialize(Matrix cov);

void Clus_EMClassify(Matrix center, Matrix cov, Vector prior,
		     double temperature,
		     int freq, int scale, Matrix output);
void Clus_EMDistance(Matrix center, Matrix cov, Vector prior,
		     double temperature,
		     Vector point, int scale, Matrix output);

/* particle ******************************************************************/
typedef void ProjectFunc(double *d, double *s, int n, double feedback,
			 int i, Any data);
double SurfaceRepel(Matrix sample, Matrix vel, double noise,
		    int *nsample_p, 
		    ProjectFunc *project, Any data);
