#ifndef GMM_H_INCLUDED
#define GMM_H_INCLUDED

#include <tpm/matrix.h>

typedef struct GaussianMixture {
  /* Each row of center is the mean for a mixture component. */
  Matrix center;
  
  /* Each row of cov is the covariance for a mixture component. */
  /* In the full covariance model, 
   * the covariance is stored as the concatenation of the eigenvectors,
   * weighted by the inverse root of their eigenvalues.
   */
  Matrix cov;

  /* Mixing proportions */
  Vector prior;

  /* Parameter for scaling the covariance */
  double temperature;

  enum {COV_FULL, COV_IDENTITY} cov_type;
} *GaussianMixture;

GaussianMixture GaussianMixtureCreate(int ncenters, int ndims);
void GaussianMixtureFree(GaussianMixture gm);
void GaussianMixtureSample(GaussianMixture gm, Vector sample);
double GaussianMixtureLogProb(GaussianMixture gm, Vector point);
double GaussianMixtureClassify(GaussianMixture gm, Vector point, double *mbr);
double GaussianMixtureDistance(GaussianMixture gm, Vector v1, Vector v2);

void GaussianMixtureIdentityCov(GaussianMixture gm);
void GaussianMixtureMeanCenter(GaussianMixture gm, Matrix points);
double GaussianMixtureTrainEM(GaussianMixture gm, Matrix points);
double GaussianMixtureTrainStepE(GaussianMixture gm, Matrix points, Matrix mbr);
void GaussianMixtureTrainStepM(GaussianMixture gm, Matrix points, Matrix mbr);
void GaussianMixtureTrainStepV(GaussianMixture gm, Matrix points, Matrix mbr);
void GaussianMixtureTrainStepP(GaussianMixture gm, Matrix mbr);

void GaussianMixtureTrainIncremental(GaussianMixture gm, Vector point,
				     double rate);
void GaussianMixtureTrainFast(GaussianMixture gm, Matrix points, int iters);

#endif
