#include <tpm/tpm.h>
#include <tpm/cluster.h>

main(int argc, char *argv[])
{
  int ncenters;
  Matrix sample; /* sample vector */
  Matrix center; /* center mean */
  Matrix cov;    /* covariance of centers */
  Matrix mbr;  /* prob of center given sample */
  double temperature;
  FILE *fp;

  if(argc < 4) {
    fprintf(stderr, "Usage:\n%s k center_output cov_output mbr_output < sample_matrix\n", argv[0]);
    fprintf(stderr, "To generate k clusters of the sample vectors.\n");
    exit(1);
  }

  sample = MatrixScan(stdin);
  if(!sample) {
    fprintf(stderr, "Error reading sample vector matrix\n");
    exit(1);
  }

  ncenters = atoi(argv[1]);
  center = MatrixCreate(ncenters, sample->cols);
  cov = MatrixCreate(ncenters, sample->cols*sample->cols);
  Clus_EMInitialize(cov);
  mbr = MatrixCreate(sample->rows, center->rows);
  /* initialize mbr to uniform */
  VectorSet((Vector)mbr, 1.0/mbr->cols);

  /* Anneal the clustering */
  for(temperature = 1e4; temperature >= 1; temperature = temperature / 10) {
    char center_file[100], cov_file[100], mbr_file[100];
    fprintf(stderr, "temperature = %g\n", temperature);
    sprintf(center_file, "%s.%g", argv[2], temperature);
    sprintf(cov_file, "%s.%g", argv[3], temperature);
    sprintf(mbr_file, "%s.%g", argv[4], temperature);
    Clus_EM(sample, center, cov, mbr, temperature, 
	    center_file, cov_file, mbr_file);
  }
  Clus_EMStepV(sample, center, cov, mbr);

  fp = fopen(argv[2], "w");
  MatrixPrint(fp, center);
  fclose(fp);
  fp = fopen(argv[3], "w");
  MatrixPrint(fp, cov);
  fclose(fp);
  fp = fopen(argv[4], "w");
  MatrixPrint(fp, mbr);
  fclose(fp);
  
  MatrixFree(mbr);
  MatrixFree(center);
  MatrixFree(cov);
  MatrixFree(sample);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}

