/* Usage: matrix_mst < feature_matrix > spantree
 */
#include <tpm/cluster.h>
#include <tpm/matrix.h>

static Vector va = NULL, vb = NULL;

static int DistanceRow(int row, int cols, double *dists, Matrix features)
{
  int i;
  if(!va) {
    /* never get freed */
    va = VectorFromData(features->cols, NULL);
    vb = VectorFromData(features->cols, NULL);
  }
  va->d = features->d2[row];
  for(i=0;i<cols;i++) {
    vb->d = features->d2[i];
    dists[i] = VectorDistanceLp(va, vb, 2);
  }
  return TPM_OK;
}

int main(int argc, char *argv[])
{
  SpanTree tree;
  Matrix features;
  if(argc > 1) features = MatrixScan(stdin);
  else features = MatrixReadBinary(stdin);
  tree = Clus_MST(features->rows, (Clus_RowFunc*)DistanceRow, features);
  MatrixFree(features);
  if(!tree) {
    fprintf(stderr, "Error in %s\n", argv[0]);
    exit(1);
  }
  SpanTreePrint(stderr, tree);
  if(argc > 1) {
    SpanTreePrint(stdout, tree);
  } 
  else {
    SpanTreeWrite(tree, stdout);
  }
  SpanTreeFree(tree);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}
