load-libs tpm matrix cluster
load-libs f2c blas lapack

# Canvas bindings #############################################################
canvas .canvas -background black -width 400 -height 400
pack .canvas -fill both -expand 1

proc dot {x y radius args} {
    set dot [eval .canvas create oval 0 0 0 0 -width 1 -outline white $args]
    oval_pos $dot $x $y $radius $radius
    return $dot
}

# Controls ####################################################################
frame .controlf
pack .controlf -fill x

frame .temp
pack .temp -fill x -expand 1 -in .controlf
label .temp.label -text Temperature
label .temp.value -textvariable temperature
proc set_temp t {
    global temperature
    set temperature [expr exp($t*1.05)]
}
scale .temp.scale -from 0 -to 20 -resolution 0.01 -orient horiz -command set_temp -showvalue no
pack .temp.scale .temp.label .temp.value -side left -in .temp

set temperature 100000

frame .noise
pack .noise -fill x -expand 1 -in .controlf
label .noise.label -text Noise
label .noise.value -textvariable noise
scale .noise.scale -from 0 -to 10 -resolution 0.1 -variable noise -orient horiz -showvalue no
pack .noise.scale .noise.label .noise.value -side left -in .noise

set noise 1

# Surface #####################################################################
if 1 {
    # surface is represented as a collection of lines
    # corr rows of "base" and "offset" matrices define each line
    if 0 {
	# three lines which form a triangle (clockwise)
	set base [matrix create 3 2]
	set offset [matrix create [matrix rows $base] 2]
	matrix set $base {250 200  200 300  150 200}
	matrix set $offset {0.7071 0.7071  -1 0  0.7071 -0.7071}
    } elseif 1 {
	# four lines which form a square (counterclockwise)
	set base [matrix create 4 2]
	set offset [matrix create [matrix rows $base] 2]
	matrix set $base {100 200  200 300  300 200  200 100}
	matrix set $offset {0 1  1 0  0 -1  -1 0}
    } else {
	# four lines which form a square (counterclockwise)
	set base [matrix create 5 2]
	set offset [matrix create [matrix rows $base] 2]
	matrix set $base {100 200  200 300  300 200  200 100  200 200}
	matrix set $offset {0 1  1 0  0 -1  -1 0  0 1}
    }
    
    for {set i 0} {$i < [matrix rows $base]} {incr i} {
	unpack [matrix_row $base $i] x y
	unpack [matrix_row $offset $i] dx dy
	.canvas create line [expr $x-$dx*50] [expr $y-$dy*50] \
		[expr $x+$dx*50] [expr $y+$dy*50] -fill blue -width 3
    }
    
} else {
    #.canvas create oval 100 100 300 300 -outline blue
    .canvas create line 100 100 100 300 300 300 300 100 100 100 -fill blue
}

# Surface dust ################################################################
set ndust 14
set dust [matrix create $ndust 3]
proc init_dust {} {
    global dust
    vector random $dust
    vector scale $dust 400
}
init_dust
if 0 {
set radius [vector create [matrix rows $dust]]
vector fill $radius 100
} else {
set radius 10
}
set vel [matrix create [matrix rows $dust] [matrix cols $dust]]
vector random $vel
vector scale $vel 10

for {set i 0} {$i < [matrix rows $dust]} {incr i} {
    unpack [matrix_row $dust $i] x y
    dot $x $y $radius -fill white -tags dust$i
    .canvas create line 0 0 0 0 -fill gray -tags vel$i
    #.canvas create text $x $y -text $i -fill yellow -tags text$i
}

if 1 {
    set cov [matrix create [matrix rows $base] \
	    [expr [matrix cols $base] * [matrix cols $base]]]
    cluster em_identity $cov
    set mbr [matrix create [matrix rows $dust] [matrix rows $base]]
}

set total 0
label .total -textvariable total
pack .total

proc update_dust {} {
    global ndust dust radius base offset mbr temperature vel noise total
    if 0 {
	# repel
	set ndust [cluster surface_repel $dust $radius $ndust]
	#puts "radius: [vector get $radius]"
    } else {
	global base offset cov mbr
	# compute membership
	#cluster em_E $dust $base $cov $mbr $temperature
	# project onto the surface
	set total [cluster surface_repel $dust $vel $noise $ndust $base $offset $mbr]
    }
    # display
    for {set i 0} {$i < [matrix rows $dust]} {incr i} {
	if {$i < $ndust} {
	    unpack [matrix_row $dust $i] x y r
	    set radius [expr $r/50]
	    oval_pos dust$i $x $y $radius $radius
	    .canvas raise dust$i
	    unpack [matrix_row $vel $i] dx dy
	    .canvas coords vel$i $x $y [expr $x-$dx] [expr $y-$dy]
	    .canvas coords text$i [expr $x+5] [expr $y+5]
	} else {
	    eval oval_pos dust$i 0 0 0 0
	    .canvas coords vel$i 0 0 0 0
	}
    }
    after 10 update_dust
}
button .step -text Step -command update_dust
pack .step -side left -fill x -expand 1

button .init -text Init -command init_dust
pack .init -side left -fill x -expand 1

