#include <tpm/stream.h>
#include "gmm.h"

void main()
{
  GaussianMixture gm = GaussianMixtureCreate(2, 1);
  Matrix m, points;
  Vector v, mbr;
  FileHandle h;
  int i;

  h = FileOpen("test_gmm.center", "r");
  m = MatrixScan(h);
  MatrixMove(gm->center, m);
  MatrixFree(m);
  FileClose(h);

  h = FileOpen("test_gmm.cov", "r");
  m = MatrixScan(h);
  MatrixMove(gm->cov, m);
  MatrixFree(m);
  FileClose(h);
#if 1
  gm->cov_type = COV_FULL;
#else
  gm->cov_type = COV_IDENTITY;
#endif
  gm->temperature = 0.1;

  /* prob of mean with unit cov is 0.3989 for d = 1, and 0.1592 for d = 2 */
  /* prob of 7 with components 7 and 4 is 0.2016 */
  fprintf(stderr, "probability of ");
  v = VectorCreate(1);
  v->d[0] = 7;
  VectorPrint(stderr, v);
  fprintf(stderr, "is %g\n", exp(GaussianMixtureLogProb(gm, v)));

  mbr = VectorCreate(gm->center->rows);
  GaussianMixtureClassify(gm, v, mbr->d);
  fprintf(stderr, "membership is ");
  VectorPrintNL(stderr, mbr);
  VectorFree(mbr);
  VectorFree(v);

  /* Extract samples */
  points = MatrixCreate(1000*gm->center->rows, gm->center->cols);
  v = VectorFromData(gm->center->cols, NULL);
  for(i=0;i<points->rows;i++) {
    v->d = points->d2[i];
    GaussianMixtureSample(gm, v);
  }
  free(v);

  /* Fit the samples with EM */
  fprintf(stderr, "\nEM training\n");
  GaussianMixtureIdentityCov(gm);
  GaussianMixtureMeanCenter(gm, points);
  GaussianMixtureTrainEM(gm, points);
  fprintf(stderr, "center:\n");
  MatrixPrint(stderr, gm->center);
  fprintf(stderr, "cov:\n");
  MatrixPrint(stderr, gm->cov);
  fprintf(stderr, "prior:\n");
  VectorPrintNL(stderr, gm->prior);
  
  /* Fit the samples with Fast */
  fprintf(stderr, "\nIncremental training\n");
  GaussianMixtureIdentityCov(gm);
  GaussianMixtureMeanCenter(gm, points);
  GaussianMixtureTrainFast(gm, points, 20);
  fprintf(stderr, "center:\n");
  MatrixPrint(stderr, gm->center);
  fprintf(stderr, "cov:\n");
  MatrixPrint(stderr, gm->cov);
  fprintf(stderr, "prior:\n");
  VectorPrintNL(stderr, gm->prior);
  
  MatrixFree(points);
  GaussianMixtureFree(gm);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}
