#ifndef	__GL_IMAGE_H__
#define	__GL_IMAGE_H__
#ifdef __cplusplus
extern "C" {
#endif


/*
 *	Defines for image files . . . .
 *
 *  			Paul Haeberli - 1984
 *      Look in /usr/people/4Dgifts/iristools/imgtools for example code!
 *
 */

#include <stdio.h>

#define IMAGIC 	0732
#define IMAGEDEF		/* for backwards compatibility */

#ifdef __linux__
/* tpminka: these come from stdio.h on the SGI */
#define _IOEOF          0020    /* EOF reached on read */
#define _IOERR          0040    /* I/O error from system */
#define _IOREAD         0001    /* currently reading */
#define _IOWRT          0002    /* currently writing */
#define _IORW           0200    /* opened for reading and writing */
#endif

/* colormap of images */
#define CM_NORMAL		0	/* file contains rows of values which 
					 * are either RGB values (zsize == 3) 
					 * or greyramp values (zsize == 1) */
#define CM_DITHERED		1
#define CM_SCREEN		2	/* file contains data which is a screen
					 * image; getrow returns buffer which 
					 * can be displayed directly with 
					 * writepixels */
#define CM_COLORMAP		3	/* a colormap file */

#define TYPEMASK		0xff00
#define BPPMASK			0x00ff
#define ITYPE_VERBATIM		0x0000
#define ITYPE_RLE		0x0100
#define ISRLE(type)		(((type) & 0xff00) == ITYPE_RLE)
#define ISVERBATIM(type)	(((type) & 0xff00) == ITYPE_VERBATIM)
#define BPP(type)		((type) & BPPMASK)
#define RLE(bpp)		(ITYPE_RLE | (bpp))
#define VERBATIM(bpp)		(ITYPE_VERBATIM | (bpp))
#define	IBUFSIZE(pixels)	((pixels+(pixels>>6))<<2)
#define	RLE_NOP			0x00

#define	ierror(p)		(((p)->flags&_IOERR)!=0)
#define	ifileno(p)		((p)->file)
#define	getpix(p)		(--(p)->cnt>=0 ? *(p)->ptr++ : ifilbuf(p))
#define putpix(p,x)		(--(p)->cnt>=0 \
				    ? ((int)(*(p)->ptr++=(unsigned)(x))) \
				    : iflsbuf(p,(unsigned)(x)))

typedef struct {
    unsigned short	imagic;		/* stuff saved on disk . . */
    unsigned short 	type;
    unsigned short 	dim;
    unsigned short 	xsize;
    unsigned short 	ysize;
    unsigned short 	zsize;
    unsigned int 	min;
    unsigned int 	max;
    unsigned int	wastebytes;	
    char 		name[80];
    unsigned int	colormap;

    int 		file;		/* stuff used in core only */
    unsigned short 	flags;
    short		dorev;
    short		x;
    short		y;
    short		z;
    short		cnt;
    unsigned short	*ptr;
    unsigned short	*base;
    unsigned short	*tmpbuf;
    unsigned int	offset;
    unsigned int	rleend;		/* for rle images */
    unsigned int	*rowstart;	/* for rle images */
    int			*rowsize;	/* for rle images */
} IMAGE;

/* row.c *********************************************************************/
int putrow(IMAGE *image, unsigned short *buffer, unsigned int y, unsigned int z);
int getrow(IMAGE *image, unsigned short *buffer, unsigned int y, unsigned int z);

/* open.c ********************************************************************/
/* 
IMAGE *iopen(char *file, char *mode, unsigned int type, unsigned int dim,
		unsigned int xsize, unsigned int ysize, unsigned int zsize);
IMAGE *fiopen(int f, char *mode, unsigned int type, unsigned int dim,
		unsigned int xsize, unsigned int ysize, unsigned int zsize);

IMAGE *iopen(char *file, char *mode, ...);
IMAGE *fiopen(int f, char *mode, ...);
*/
IMAGE *iopen();
IMAGE *fiopen();

unsigned short *ibufalloc(IMAGE *image);
void cvtimage( IMAGE *image );
void cvtints( int *buffer, int n);
void cvtshorts( unsigned short *buffer, int n);

void i_errhdlr(char *fmt, ...);
#if 0
void i_seterror(void (*func)());
#else
void i_seterror();
#endif

/* rle.c *********************************************************************/
int img_getrowsize(IMAGE *image);
void img_setrowsize(IMAGE *image, int cnt, int y, int z);
int img_rle_compact(unsigned short *expbuf, int ibpp,
			unsigned short *rlebuf, int obpp, int cnt);
void img_rle_expand(unsigned short *rlebuf, int ibpp,
			unsigned short *expbuf, int obpp);

/* pix.c *********************************************************************/

/* name.c ********************************************************************/
void isetname(IMAGE *image, char *name);
void isetcolormap(IMAGE *image, int colormap);

/* filbuf.c ******************************************************************/
int ifilbuf(IMAGE *image);

/* flsbuf.c ******************************************************************/
unsigned int iflsbuf(IMAGE *image, unsigned int c);

/* close.c *******************************************************************/
int iclose(IMAGE *image);
int iflush(IMAGE *image);

/* rdwr.c ********************************************************************/
int img_read(IMAGE *image, char *buffer, int count);
int img_write(IMAGE *image, char *buffer,int count);
int img_badrow(IMAGE *image, unsigned int y, unsigned int z);
unsigned int img_seek(IMAGE *image, unsigned int y, unsigned int z);
unsigned int img_optseek(IMAGE *image, unsigned int offset);

#ifdef __cplusplus
}
#endif
#endif	/* !__GL_IMAGE_H__ */
