#include <tpm/tpm.h>
#include <tpm/stream.h>
#include <tpm/matrix.h>
#include <tpm/dmm.h>

void main(int argc, char *argv[])
{
  DirichletMixture dm;
  Matrix m;
  FILE *fp;
  int k, dims = 64;
  int ncenters;

  if(argc < 3) {
    fprintf(stderr, "usage: %s ncenters mixtures\n", argv[0]);
    fprintf(stderr, "mixtures is binary.\n");
    exit(1);
  }
  ncenters = atoi(argv[1]);

  fp = FileOpen(argv[2], "r");
  m = MatrixReadBinary(fp);
  fclose(fp);
  k = m->cols/dims;
  MatrixReshape(m, m->rows*k, dims);
  fprintf(stderr, "clustering %d rows, %d columns\n", m->rows, m->cols);

  dm = DirichletMixtureCreate(ncenters, dims);
#if 1
  dm->temperature = 0.1;
#endif
  DirichletMixtureTrainEM(dm, m);
  MatrixPrint(stdout, dm->center);

  DirichletMixtureFree(dm);
  MatrixFree(m);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}
