#include <tpm/tpm.h>
#include <gl/image.h>
#include <tpm/region.h>
#include <tpm/matrix.h>
#include <tpm/image.h>

#define CUBE 4
#define OHTA 0
#define HSV 1
#define CONE 0

TpmImage TpmReadAny(char *fname)
{
  TpmImage image;
  image = TpmReadPnm(fname);
  if(image) return image;
  image = TpmReadSgi(fname);
  if(image) return image;
  image = TpmReadJpeg(fname);
  if(image) return image;
  image = TpmReadTiff(fname);
  if(image) return image;
  return image;
}

/* Reads image names on stdin and outputs histograms on stdout. */
void main()
{
  char fname[1024];
  TpmImage image, q_image;
#if CUBE
  Vector hist = VectorCreate(CUBE*CUBE*CUBE);
#else
  Vector hist = VectorCreate(3*64);
#endif
  Tpm_Region *region = Tpm_RegionCreate();
  for(;;) {
    if(gets(fname) == NULL) break;
    image = TpmReadAny(fname);
    if(!image) {
      fprintf(stderr, "Cannot open `%s': ", fname);
      perror("");
      continue;
    }
#if OHTA
    TpmImageRGB2Ohta(image);
#elif HSV
    TpmImageRGB2HSV(image);
#elif CONE
    TpmImageRGB2Cone(image);
#endif
#if CUBE
    q_image = TpmImageQuantizeCube(image, CUBE, CUBE, CUBE);
#else
    q_image = TpmImageQuantizeChannels(image, 64);
#endif

    TpmImageHist(q_image, hist);
    VectorScale(hist, 1.0/q_image->len);
    printf("%s ", fname);
    fprintf(stderr, "%s\n", fname);
    VectorPrintNL(stdout, hist);
    if(0) {
      /* check */
      Vector v = VectorCreate(hist->len);
      Tpm_RegionRectangle(region, 0, 0, q_image->width, q_image->height);
      TpmImageRegionHistogram(q_image, region, v);
      VectorScale(v, 1.0/q_image->len);
      VectorPrintNL(stderr, v);
      VectorFree(v);
    }
    TpmImageFree(q_image);
    TpmImageFree(image);
  }
  Tpm_RegionFree(region);
  VectorFree(hist);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}
