/* Given a matrix of mixtures and a mixture, outputs a histogram
 * for each mixture.
 */
#include <tpm/tpm.h>
#include <tpm/stream.h>
#include <tpm/matrix.h>
#include <tpm/dmm.h>

void main(int argc, char *argv[])
{
  DirichletMixture dm;
  Matrix m;
  Vector v, mbr, hist;
  FILE *fp;
  int i, j;
  int k, dims = 64;

  if(argc < 3) {
    fprintf(stderr, "usage: %s mixture_file mixtures\n", argv[0]);
    fprintf(stderr, "mixture_file is ascii while mixtures is binary.\n");
    exit(1);
  }
  fp = FileOpen(argv[1], "r");
  m = MatrixScan(fp);
  fclose(fp);

  dm = DirichletMixtureCreate(m->rows, m->cols);
#if 1
  dm->temperature = 0.1;
#endif
  MatrixMove(dm->center, m);
  MatrixFree(m);

  fp = FileOpen(argv[2], "r");
  m = MatrixReadBinary(fp);
  fclose(fp);
  k = m->cols/dims;

  v = VectorFromData(dims, NULL);
  mbr = VectorCreate(dm->center->rows);
  hist = VectorCreate(dm->center->rows);

  /* loop rows */
  for(i=0;i<m->rows;i++) {
    VectorSet(hist, 0);
    /* loop components of the mixture */
    for(j=0;j<k;j++) {
      v->d = m->d2[i] + j*dims;
      DirichletMixtureClassify(dm, v, mbr->d);
      VectorAdd(hist, mbr);
    }
    VectorScale(hist, 1.0/k);
    VectorPrint(stdout, hist);
  }

  VectorFree(hist);
  VectorFree(mbr);
  free(v);
  DirichletMixtureFree(dm);
  MatrixFree(m);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}
