#include <tpm/tpm.h>
#include <tpm/matrix.h>
#include "image.h"

/* Prototypes ****************************************************************/
TpmImage TpmImageFromMatrix(Matrix m);
Matrix MatrixFromImage(TpmImage image);
void CoocMatrixFromImage(TpmImage image, int dx, int dy, Matrix cooc);

TpmImage TpmImageFromVolume(Volume v);
Volume VolumeFromImage(TpmImage image);

/* Functions *****************************************************************/

TpmImage TpmImageFromMatrix(Matrix m)
{
  TpmImage image = TpmImageCreate(m->cols, m->rows, 1);
  int i;
  for(i=0;i<m->len;i++) image->d[i] = (unsigned char)m->d[i];
  return image;
}

/* Convert a single-channel image to a matrix.
 * If the image is multi-channel, it is converted to gray.
 */
Matrix MatrixFromImage(TpmImage image)
{
  int i;
  Matrix matrix;
  if(image->channels == 1) {
    matrix = MatrixCreate(image->height, image->width);
    for(i=0;i<matrix->len;i++) {
      matrix->d[i] = (double)image->d[i];
    }
  }
  else {
    TpmImage gray = TpmImageGray(image);
    matrix = MatrixFromImage(gray);    /* recursive call */
    TpmImageFree(gray);
  }
  return matrix;
}

TpmImage TpmImageFromVolume(Volume v)
{
  TpmImage image = TpmImageCreate(v->xdim, v->ydim, v->zdim);
  int i;
  for(i=0;i<v->len;i++) image->d[i] = (unsigned char)v->d[i];
  return image;
}

/* Convert a multi-channel image to a volume */
Volume VolumeFromImage(TpmImage image)
{
  Volume volume = VolumeCreate(image->channels, image->height, image->width);
  int i;
  for(i=0;i<volume->len;i++) {
    volume->d[i] = (double)image->d[i];
  }
  return volume;
}

/*****************************************************************************/

/* cooc->d2[m][n] += #times pixel value n occurs displaced by (dx,dy) 
 * from pixel value m.
 * dx and dy must be positive.
 * Does not clear cooc, so that multiple calls will accumulate.
 */
void CoocMatrixFromImage(TpmImage image, int dx, int dy, Matrix cooc)
{
  int x,y;
  int factor = 256 / cooc->rows;
  for(y=0;y<image->height-dy;y++) {
    for(x=0;x<image->width-dx;x++) {
      cooc->d2[image->d2[y][x]/factor][image->d2[y+dy][x+dx]/factor]++;
    }
  }
}

