#include <tpm/tpm.h>
#include <tpm/image.h>
#include <gl/image.h>

/* Prototypes ****************************************************************/
TpmImage TpmReadSgi(char *fname);
TpmImage TpmImageFromSgi(IMAGE *img);
int TpmImageWriteSgi(FILE *fp, TpmImage image);

/* Functions *****************************************************************/

TpmImage TpmReadSgi(char *fname)
{
  TpmImage image;
  IMAGE *img;

  i_seterror(ErrorSet);
  if(!strcmp(fname, "-")) 
    img = fiopen(0, "r");
  else
    img = iopen(fname, "r");
  if(!img) return NULL;
  image = TpmImageFromSgi(img);
  iclose(img);
  return image;
}

TpmImage TpmImageFromSgi(IMAGE *img)
{
  int z, y, x;
  TpmImage image = TpmImageCreate(img->xsize, img->ysize, img->zsize);
  unsigned short *buffer = Allocate(img->xsize, unsigned short);

  /* Loop color planes */
  for(z=0;z<image->channels;z++) {
    for(y=0;y<image->height;y++) {
      getrow(img, buffer, y, z);
      /* getrow gives us an array of shorts which we must convert into chars */
      for(x=0;x<image->width;x++) {
	/* SGI images are stored upside-down so we flip y */
	image->d3[z][image->height-1-y][x] = buffer[x];
      }
    }
  }
  free(buffer);
  return image;
}

int TpmImageWriteSgi(FILE *fp, TpmImage image)
{
  IMAGE *img;
  int z, y, x;
  unsigned short *buffer;

  i_seterror(ErrorSet);
  img = fiopen(fileno(fp), "w", RLE(1), image->channels, 
	       image->width, image->height, image->channels);
  if(!img) return TPM_ERROR;
  buffer = Allocate(img->xsize, unsigned short);

  /* Loop color planes */
  for(z=0;z<image->channels;z++) {
    for(y=0;y<image->height;y++) {
      /* convert chars into an array of shorts */
      for(x=0;x<image->width;x++) {
	/* SGI images are stored upside-down so we flip y */
	buffer[x] = image->d3[z][image->height-1-y][x];
      }
      putrow(img, buffer, y, z);
    }
  }
  iclose(img);
  free(buffer);
  return TPM_OK;
}

