#include "tpm/tpm.h"
#include "xtpm/xtpm.h"
#include "xtpm/app.h"
#include "image.h"

#define TIFF 0
#define PNM 1

int debug = 1;
XtpmAppData appData;
TpmImage image;
Pixmap pixmap;

/* Prototypes ****************************************************************/

static void CBInput(Widget w, void *userData, XmAnyCallbackStruct *callData);
static void CBExpose(Widget w, void *userData, XmAnyCallbackStruct *callData);

/* Functions *****************************************************************/

TpmImage TpmReadAny(char *fname)
{
  TpmImage image;
  image = TpmReadPnm(fname);
  if(image) return image;
  image = TpmReadSgi(fname);
  if(image) return image;
  image = TpmReadJpeg(fname);
  if(image) return image;
  image = TpmReadTiff(fname);
  if(image) return image;
  return image;
}
void main(int argc, char *argv[])
{
  int count;
  Arg arglist[10];
  Widget pic;

  if(argc > 1) {
    image = TpmReadAny(argv[1]);
  }
  else {
    image = TpmReadAny("-");
  }
  if(!image) exit(1);

  appData = XtpmStartup(&argc, argv);
  XtpmSetDepth(appData, 24, 0);

  count = 0;
  XtSetArg(arglist[count], XtNwidth, image->width); count++;
  XtSetArg(arglist[count], XtNheight, image->height); count++;
  pic = XmCreateDrawingArea(appData->toplevel, "picture", arglist, count);
  XtAddCallback(pic, XmNinputCallback, (XtCallbackProc)CBInput, NULL);
  XtAddCallback(pic, XmNexposeCallback, (XtCallbackProc)CBExpose, NULL);
  XtManageChild(pic);

  XtRealizeWidget(appData->toplevel);
  XtpmCreateGC(appData);
  pixmap = TpmImageToPixmap(appData, image);
  XtAppMainLoop(appData->app_context);
}

/* callbacks *****************************************************************/

static void CBInput(Widget w, void *userData, XmAnyCallbackStruct *callData)
{
  if(callData->event->type != ButtonPress) return;
  TpmImageFree(image);
  XFreePixmap(appData->display, pixmap);
  free(appData);
  XC_BLOCKS();
  MEM_STATUS();
  MEM_BLOCKS();
  exit(0);
}

static void CBExpose(Widget w, void *userData, XmAnyCallbackStruct *callData)
{
  XCopyArea(appData->display, pixmap, XtWindow(w), appData->gc,
            0, 0, image->width, image->height, 0, 0);
}
