#include <tpm/tpm.h>
#include "image.h"

TpmImage TpmImageCone2RGB(TpmImage image)
{
  TpmImage result = TpmImageCreate(image->width, image->height, 
				   image->channels);
  int y, x;
  for(y=0;y<image->height;y++) {
    for(x=0;x<image->width;x++) {
      Cone2RGB(image->d3[0][y][x], image->d3[1][y][x], image->d3[2][y][x],
	     &result->d3[0][y][x], &result->d3[1][y][x], &result->d3[2][y][x]);
    }
  }
  return result;
}

void main(int argc, char *argv[])
{
  TpmImage rgb_image, image;
  int x, y;
  if(argc < 2) {
    fprintf(stderr, "usage: %s image.rgb\n", argv[0]);
    exit(1);
  }
  rgb_image = TpmReadSgi(argv[1]);

  if(0) {
    TpmImage ohta_image = TpmImageCopy(rgb_image);
    TpmImageRGB2Ohta(ohta_image);
    image = TpmImageCopy(ohta_image);
    TpmImageOhta2RGB(image);
    for(y=0;y<image->height;y++) {
      for(x=0;x<image->width;x++) {
	if(abs(rgb_image->d3[0][y][x] - image->d3[0][y][x]) > 1) {
	  fprintf(stderr, "Ohta2RGB failed\n");
	  fprintf(stderr, "RGB  (%d, %d, %d)\n", 
		  rgb_image->d3[0][y][x],
		  rgb_image->d3[1][y][x],
		  rgb_image->d3[2][y][x]);
	  fprintf(stderr, "Ohta (%d, %d, %d)\n", 
		  ohta_image->d3[0][y][x],
		  ohta_image->d3[1][y][x],
		  ohta_image->d3[2][y][x]);
	  fprintf(stderr, "RGB  (%d, %d, %d)\n", 
		  image->d3[0][y][x],
		  image->d3[1][y][x],
		  image->d3[2][y][x]);
	  fprintf(stderr, "\n");
	}
      }
    }
    TpmImageFree(image);
    TpmImageFree(ohta_image);
  }

  if(1) {
    TpmImage hsv_image = TpmImageCopy(rgb_image);
    TpmImageRGB2HSV(hsv_image);
    image = TpmImageCopy(hsv_image);
    TpmImageHSV2RGB(image);
    for(y=0;y<rgb_image->height;y++) {
      for(x=0;x<rgb_image->width;x++) {
	int err = abs(rgb_image->d3[0][y][x] - image->d3[0][y][x]) +
	  abs(rgb_image->d3[1][y][x] - image->d3[1][y][x]) +
	  abs(rgb_image->d3[2][y][x] - image->d3[2][y][x]);
	if(err > 2) {
	  fprintf(stderr, "HSV2RGB failed\n");
	  fprintf(stderr, "RGB  (%d, %d, %d)\n", 
		  rgb_image->d3[0][y][x],
		  rgb_image->d3[1][y][x],
		  rgb_image->d3[2][y][x]);
	  fprintf(stderr, "HSV  (%d, %d, %d)\n", 
		  hsv_image->d3[0][y][x],
		  hsv_image->d3[1][y][x],
		  hsv_image->d3[2][y][x]);
	  fprintf(stderr, "RGB  (%d, %d, %d)\n", 
		  image->d3[0][y][x],
		  image->d3[1][y][x],
		  image->d3[2][y][x]);
	  fprintf(stderr, "\n");
	}
      }
    }
    if(0) {
      TpmImage q_image = TpmImageQuantizeCube(hsv_image, 4, 4, 4);
      TpmImageFree(hsv_image);
      hsv_image = TpmImageUnQuantizeCube(q_image, 4, 4, 4);
      TpmImageFree(image);
      image = TpmImageCopy(hsv_image);
      TpmImageHSV2RGB(image);
    }
    TpmImageWritePnm(stdout, image);
    TpmImageFree(image);
    TpmImageFree(hsv_image);
  }

  if(0) {
    TpmImage cone_image = TpmImageCopy(rgb_image);
    TpmImageRGB2Cone(cone_image);
    image = TpmImageCone2RGB(cone_image);
    for(y=0;y<rgb_image->height;y++) {
      for(x=0;x<rgb_image->width;x++) {
	int err = abs(rgb_image->d3[0][y][x] - image->d3[0][y][x]) +
	  abs(rgb_image->d3[1][y][x] - image->d3[1][y][x]) +
	  abs(rgb_image->d3[2][y][x] - image->d3[2][y][x]);
	if(err > 4) {
	  fprintf(stderr, "RGB  (%d, %d, %d)\n", 
		  rgb_image->d3[0][y][x],
		  rgb_image->d3[1][y][x],
		  rgb_image->d3[2][y][x]);
	  fprintf(stderr, "Cone  (%d, %d, %d)\n", 
		  cone_image->d3[0][y][x],
		  cone_image->d3[1][y][x],
		  cone_image->d3[2][y][x]);
	  fprintf(stderr, "RGB  (%d, %d, %d)\n", 
		  image->d3[0][y][x],
		  image->d3[1][y][x],
		  image->d3[2][y][x]);
	  fprintf(stderr, "\n");
	}
      }
    }
    if(1) {
      TpmImage q_image = TpmImageQuantizeCube(cone_image, 4, 4, 4);
      TpmImageFree(cone_image);
      cone_image = TpmImageUnQuantizeCube(q_image, 4, 4, 4);
      TpmImageFree(image);
      image = TpmImageCone2RGB(cone_image);
    }
    TpmImageWritePnm(stdout, image);
    TpmImageFree(image);
    TpmImageFree(cone_image);
  }

  if(0) {
    TpmImage q_image = TpmImageQuantizeCube(rgb_image, 4, 4, 4);
    TpmImageFree(rgb_image);
    rgb_image = TpmImageUnQuantizeCube(q_image, 4, 4, 4);
    TpmImageWritePnm(stdout, rgb_image);
  }

  if(1) {
    unsigned char r, g, b, h, s, v;
    r = 0;
    g = 17;
    b = 0;
    RGB2HSV(r, g, b, &h, &s, &v);
    fprintf(stderr, "RGB  (%d, %d, %d)\n", r, g, b);
    fprintf(stderr, "HSV  (%d, %d, %d)\n", h, s, v);
    HSV2RGB(h, s, v, &r, &g, &b);
    fprintf(stderr, "RGB  (%d, %d, %d)\n", r, g, b);

    r = 0;
    g = 17;
    b = 0;
    RGB2Cone(r, g, b, &h, &s, &v);
    fprintf(stderr, "RGB  (%d, %d, %d)\n", r, g, b);
    fprintf(stderr, "Cone  (%d, %d, %d)\n", h, s, v);
    Cone2RGB(h, s, v, &r, &g, &b);
    fprintf(stderr, "RGB  (%d, %d, %d)\n", r, g, b);
  }

  TpmImageFree(rgb_image);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}
