#include <tk.h>
#include <tpm/tpm.h>
#include "image.h"

/* Prototypes ****************************************************************/
void TpmImageFillTk(Tk_PhotoHandle photo, TpmImage image);
TpmImage TpmImageFromTk(Tk_PhotoHandle photo);

/* Functions *****************************************************************/

void TpmImageFillTk(Tk_PhotoHandle photo, TpmImage image)
{
  int i, image_size = image->width * image->height;
  Tk_PhotoImageBlock block;

#if 1
  /* force the photo to be same size as TpmImage */
  Tk_PhotoSetSize(photo, image->width, image->height);
#else
  Tk_PhotoBlank(photo);
  Tk_PhotoExpand(photo, image->width, image->height);
#endif
  if(!image_size) return;
  block.width = image->width;
  block.height = image->height;
  block.pitch = block.width * image->channels;
  block.pixelSize = image->channels;
  block.offset[0] = 0;
  block.offset[1] = 1;
  block.offset[2] = 2;
  block.pixelPtr = Allocate(image->len, unsigned char);
  if(image->channels == 3) {
    unsigned char *rptr = *image->d3[0];
    unsigned char *gptr = *image->d3[1];
    unsigned char *bptr = *image->d3[2];
    unsigned char *ptr = block.pixelPtr;
    for(i=0;i<image_size;i++) {
      *ptr++ = *rptr++;
      *ptr++ = *gptr++;
      *ptr++ = *bptr++;
    }
  }
  else {
    /* assume one channel */
    memcpy(block.pixelPtr, image->d, image->len);
  }
  Tk_PhotoPutBlock(photo, &block, 0, 0, block.width, block.height);
  free(block.pixelPtr);
}

TpmImage TpmImageFromTk(Tk_PhotoHandle photo)
{
  TpmImage image;
  Tk_PhotoImageBlock block;

  Tk_PhotoGetImage(photo, &block);
  image = TpmImageCreate(block.width, block.height, block.pixelSize);
  if(image->channels == 3) {
    unsigned char *r, *g, *b, *p;
    int i, size = image->width * image->height;
    r = *image->d3[0];
    g = *image->d3[1];
    b = *image->d3[2];
    p = block.pixelPtr;
    for(i=0;i<size;i++) {
      *r++ = *p++; *g++ = *p++; *b++ = *p++;
    }
  }
  else {
    /* assume one channel */
    memcpy(image->d, block.pixelPtr, image->len);
  }
  return image;
}
