/* Learning benchmark.
 * boa_bench partition tree tree ...
 */
#include <tpm/boa_learn.h>
#include <boa/boa.h>

#define PARTITION 1
#if PARTITION
#include <tpm/partition.h>
#endif

int main(int argc, char *argv[])
{
  Obj od, partitionObj, features, classifiers, t;
#if PARTITION
  Partition partition;
#else
  float *partition;
  int p_length;
#endif
  int i;
  FILE *fp, *log;
  char logfile[128];
  Learn_BenchData truth;
  Learn_BenchGroup grp;
  char *partitionFile, *classifierName;

  if(argc < 5) {
    fprintf(stderr, "usage: %s alg partition tree tree ... output\n", argv[0]);
    fprintf(stderr, "and trees are Hiers of ints.\n");
    exit(1);
  }
  classifierName = argv[1];

  Boa_Startup();
  Boa_Hier_Pkg();
  Boa_Learn_Pkg();
  Boa_Vector_Pkg(); /* in case someone needs it */
  Boa_Tcl_Pkg();

  od = Boa_Create(T_ObjDir);

  /* read partition */
  partitionFile = argv[2];
#if PARTITION
  partition = PartitionCreateFromFile(partitionFile);
  if(!partition) {
    ErrorSetErrno();
    ErrorPrepend("Cannot load partition file `%s'", partitionFile);
    ErrorPrint(stderr);
    exit(1);
  }
#else
  if(Boa_StringGet(od, partitionFile, 0, NULL, &partitionObj) != TPM_OK) {
    ErrorPrepend("Cannot load partition file `%s'", partitionFile);
    ErrorPrint(stderr);
    exit(1);
  }
  assert(partitionObj->type->type == T_Array);
  partition = (float*)partitionObj->value;
  p_length = Boa_Array_Length(partitionObj->type);
#endif

  /* read trees */
  features = Boa_Create(T_Ord);
  for(i=3;i<argc-1;i++) {
    if(Boa_StringGet(od, argv[i], 0, NULL, &t) != TPM_OK) {
      ErrorPrepend("Cannot load tree `%s'", argv[i]);
      ErrorPrint(stderr);
      exit(1);
    }
    Boa_StringSet(features, argv[i], BOA_INSERT, t);
  }

  /* create the classifier */
  t = Boa_Type_Create(classifierName);
  classifiers = Boa_MultiClass_Create(Boa_Classifier_Create(
                  t, features, NULL, NULL));

  /* prime it with examples */
  for(;;) {
    char line[128], name[128];
    int member, label;
    Obj mbr;
    float weight;
    if(!getline(line, 128, stdin)) break;
    /* line contains member number, label number, and weight */
    i = sscanf(line, "%d %d %g", &member, &label, &weight);
    if(i < 2) {
      fprintf(stderr, "Examples should be \"member label [weight]\"\n");
      continue;
    }
    if(i < 3) weight = 1;
    fprintf(stderr, "%d %d %g\n", member, label, weight);
    label--;
    while(partition->num_classes > Boa_Ord_Length(classifiers)) {
      Boa_MultiClass_AddClass(classifiers);
    }
    mbr = Boa_Int_Create(member);
    for(i=0;i<partition->num_classes;i++) {
      Obj classifier = Boa_Ord_Item(classifiers, i);
      if(i == label) t = Boa_Float_Create(weight);
      else t = Boa_Float_Create(-weight);
      if(Boa_ObjSet(classifier, mbr, BOA_INSERT, t) != TPM_OK) {
	ErrorPrepend("Classifier set failed for label %d", label);
	ErrorPrint(stderr);
	Boa_Release(t);
      }
    }
    Boa_Release(mbr);
  }

  /* collect a ground-truth list */
  truth = Learn_BenchData_Create();
#if PARTITION
  for(i=0;i<partition->num_elements;i++) {
    if(!partition->class[i]) continue;
    while(partition->class[i]-1 >= Boa_Ord_Length(classifiers)) {
      Boa_MultiClass_AddClass(classifiers);
    }
    Learn_BenchData_AddInstance(truth, partition->class[i]-1, 1,
				Boa_Int_Create(i), NULL);
  }
  PartitionFree(partition);
#else
  grp = NULL;
  for(i=0;i<p_length;i++) {
#if 0
    if(i % 2 == 0) grp = Learn_BenchData_AddGroup(truth);
#endif
    Learn_BenchData_AddInstance(truth, 0, partition[i], 
				Boa_Int_Create(i), grp);
  }
  Boa_Release(partitionObj);
#endif
  
  /* benchmark the classifier */
  fp = fopen(argv[argc-1], "w");
  sprintf(logfile, "%s.log", argv[argc-1]);
  log = fopen(logfile, "w");
  Learn_Benchmark(classifiers, truth, fp, log);
  fclose(log);
  fclose(fp);
  Learn_BenchData_Free(truth);
#if 0
  assert(Boa_StringSet(od, "_path", 0, Boa_String_01("/tmp/tpminka")) 
	 == TPM_OK);
  t = Boa_Ord_Item(classifiers, 0);
  if(Boa_Classifier_Dump(t, od) != TPM_OK) {
    ErrorPrint(stderr);
  }
#endif
  Boa_Release(classifiers);

  Boa_Release(od);
  Boa_Shutdown();
  exit(0);
}
