#include <tpm/list.h>
#include <boa/boa.h>

typedef List InstanceFunc(Any data);

/* XXX kludge */
#define MAX_LABELS 256

/* Ground-truth data structure for Learn_Benchmark */
typedef struct Learn_BenchGroup {
  int *count;   /* [label] */
  List *errors; /* [label] */
} *Learn_BenchGroup;
typedef struct Learn_BenchInstance {
  Obj instance;
  float *weight;  /* [label] */
  Learn_BenchGroup group;
} *Learn_BenchInstance;
typedef struct Learn_BenchData {
  List instances;  /* List of BenchInstance */
  List groups;     /* List of BenchGroup */
} *Learn_BenchData;

/* boa_learn *****************************************************************/
void Boa_Learn_Pkg(void);

Obj Boa_Classifier_Create(Type classifier_t, Obj features, CmpFunc *compare,
			  InstanceFunc *instance);
Obj Boa_Classifier_CreateBlock(Obj classifier);
List Boa_Classifier_AllMatching(Obj filter, Any item, CmpFunc *cf);
int Boa_Classifier_Dump(Obj filter, Obj dest);

Obj Boa_MultiClass_Create(Obj classifier);
Obj Boa_MultiClass_CreateBlock(Obj classifiers);
Obj Boa_MultiClass_AddClass(Obj classifiers);
Obj Boa_MultiClass_CreateExclusive(Obj classifiers);

Learn_BenchData Learn_BenchData_Create(void);
void Learn_BenchData_Free(Learn_BenchData data);
Learn_BenchGroup Learn_BenchData_AddGroup(Learn_BenchData data);
Learn_BenchInstance Learn_BenchData_AddInstance(Learn_BenchData data, 
						int label, float weight, 
						Obj instance, 
						Learn_BenchGroup group);
int Learn_Benchmark(Obj classifiers, Learn_BenchData truth, 
		    FILE *output, FILE *log);

#if 0
List BoaClassifier_AllMatching(Obj self, Obj features, 
			       CmpFunc *compare,
			       Any item, CmpFunc *include_compare);
#endif

/* ssc ***********************************************************************/
Boa_DefineFunc Boa_Classifier_SSC_Define;
extern Type T_Classifier_SSC;

/* sc ************************************************************************/
Boa_DefineFunc Boa_Classifier_SC_Define;
extern Type T_Classifier_SC;

/* belief ********************************************************************/
Boa_DefineFunc Boa_Classifier_Belief_Define;
extern Type T_Classifier_Belief;

/* spanTree ******************************************************************/
Boa_DefineFunc Boa_Classifier_SpanTree_Define;
extern Type T_Classifier_SpanTree;

/* tree **********************************************************************/
Boa_DefineFunc Boa_Classifier_Tree_Define;
extern Type T_Classifier_Tree;

/* stump *********************************************************************/
Boa_DefineFunc Boa_Classifier_Stump_Define;
extern Type T_Classifier_Stump;

/* cover *********************************************************************/
Boa_DefineFunc Boa_Classifier_Cover_Define;
extern Type T_Classifier_Cover;

/* nearest *******************************************************************/
Boa_DefineFunc Boa_Classifier_Nearest_Define;
extern Type T_Classifier_Nearest;

/* kr ************************************************************************/
Boa_DefineFunc Boa_Classifier_KR_Define;
extern Type T_Classifier_KR;

/* CDM ***********************************************************************/
Boa_DefineFunc Boa_Classifier_CDM_Define;
extern Type T_Classifier_CDM;

/* triangle ******************************************************************/
Boa_DefineFunc Boa_Classifier_Triangle_Define;
extern Type T_Classifier_Triangle;

/* kernel ********************************************************************/
Boa_DefineFunc Boa_Classifier_Kernel_Define;
extern Type T_Classifier_Kernel;

/* select ********************************************************************/
Boa_DefineFunc Boa_Classifier_Select_Define;
extern Type T_Classifier_Select;

/* twoLayer ******************************************************************/
Boa_DefineFunc Boa_Classifier_TwoLayer_Define;
extern Type T_Classifier_TwoLayer;

/* butterfly *****************************************************************/
Boa_DefineFunc Boa_Classifier_Butterfly_Define;
extern Type T_Classifier_Butterfly;

/* featureMap ****************************************************************/
Boa_DefineFunc Boa_FeatureMap_Define;
extern Type T_FeatureMap;

/* exclusive *****************************************************************/
Boa_DefineFunc Boa_Exclusive_Define;
extern Type T_Exclusive;
Obj Boa_MultiClassExclusive_Create(Obj classifier, Obj classifiers,
				   Obj posEx);
#define Boa_Exclusive_PosEx(t) ((Obj*)t->value)[2]

/* euclid ********************************************************************/
Boa_DefineFunc Boa_Euclid_Define;
extern Type T_Euclid;

/* cophenetic ****************************************************************/
Boa_DefineFunc Boa_Cophenetic_Define;
extern Type T_Cophenetic;

/* proximity *****************************************************************/
Boa_DefineFunc Boa_Proximity_Define;
extern Type T_Proximity;
