/* Hier formatter.
 * GetObj with a Stream will return an Obj.
 * SetObj with a Stream will write the Obj.
 * Requires: Hier.
 */
#include <boa/boa.h>
#include "hier.h"

/* no data */

/* Globals *******************************************************************/
Type T_FormatHier;

/* Functions *****************************************************************/

static int SPrint(Obj self, StrStream ss)
{
  return StrStreamPuts(ss, "aFormatHier");
}

#if 0
/* stream must be a FileHandle.
 * *type_in_out->type must be T_Hier.
 */
static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  Obj t;
  if(!type_in_out || !*type_in_out || 
     (*type_in_out->type != T_Hier)) return TPM_BOTTOM;
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  t = Boa_Create(*type_in_out);
  if(HierRead((Hier)t->value, Boa_FileHandle_Value(stream))) {
    Boa_Release(t); return TPM_BOTTOM;
  }
  if(value_return) *value_return = t;
  else Boa_Release(t);
  return TPM_OK;
}
#else
/* stream must be a FileHandle */
static int ObjGet(Obj self, Obj stream, int flags,
		  Type *type_in_out, Obj *value_return)
{
  char *line;
  int i,j, status;
  Type type;
  Obj t;
  int pos;
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  pos = Boa_Tell(stream);

  /* read the type header */
  line = Boa_GetLine(stream);
  /* determine parameters from the header */
  status = sscanf(line, "hier %d (%d char)", &i, &j);
  free(line);
  if(status < 2) goto error;

  /* was a type expected? */
  if(!type_in_out || !*type_in_out) {
    char str[128];
    sprintf(str, "(hier %d (array %d char))", i, j);
    type = Boa_Type_Create(str);
    if(type_in_out) *type_in_out = type;
  }
  else {
    type = *type_in_out;
    /* type mismatch? */
    if(Boa_Hier_Length(type) != i) goto error;
    if(TypeSize(Boa_Hier_Subtype(type)) != j) goto error;
  }

  t = Boa_Create(type);
  if(HierRead(Boa_Hier_Value(t), Boa_FileHandle_Value(stream))) {
    Boa_Release(t); goto error;
  }
  if(value_return) *value_return = t;
  else Boa_Release(t);
  return TPM_OK;

 error:
  Boa_Seek(stream, pos, 0);
  return TPM_BOTTOM;
}
#endif

/* stream must be a FileHandle */
static int ObjSet(Obj self, Obj stream, int flags, Obj value)
{
  if(value->type != T_Hier) return TPM_BOTTOM;
  if(stream->type != T_FileHandle) return TPM_BOTTOM;
  if(HierWrite(Boa_Hier_Value(value), Boa_FileHandle_Value(stream))) {
    return TPM_BOTTOM;
  }
  Boa_Release(value);
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Empty_Final, Empty_Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Err_StringGet, Err_StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, ObjSet
};

Obj Boa_FormatHier_Define(void)
{
  T_FormatHier = Boa_SimpleType_Create("formatHier", &methods, 1, 1);
  return T_FormatHier;
}
