#ifndef HIER_H_INCLUDED
#define HIER_H_INCLUDED

#include <tpm/tpm.h>

/* Hier data structure (transparent) */
typedef struct Hier {
  int num_nodes, data_size, pad;
  char *node; /* array of size HierNodeSize(h) */
} *Hier;

typedef struct HierNode {
  int children, parent;
  char data; /* starts here; size unknown; should be aligned for any type */
} *HierNode;

#define HierNodeSize(h) (2*sizeof(int)+(h)->data_size+(h)->pad)
#define HierNodeAt(h, i) ((HierNode)((h)->node + HierNodeSize(h)*(i)))

/* hier.c ********************************************************************/
Hier HierCreate(int num_nodes, int data_size);
void HierFree(Hier hier);
int HierRead(Hier hier, FILE *h);
Hier HierCreateFromStream(FILE *h);
Hier HierCreateFromFile(char *file);
int HierWrite(Hier hier, FILE *h);
int HierRightLeaf(Hier hier, int node);

/* reduce.c ******************************************************************/
void HierRemoveValueAll(Hier hier, void *value, CmpFunc *cf);
void HierReduce(Hier hier);

/* tree.c ********************************************************************/
#ifdef TREE_H_INCLUDED
Tree TreeFromHier(Hier hier);
void HierFromTree(Hier hier, Tree tree);
#endif

/* boa_hier.c ****************************************************************/
void Boa_Hier_Pkg(void);
#ifdef BOA_H_INCLUDED
Boa_DefineFunc Boa_Hier_Define, Boa_FormatHier_Define;
extern Type T_Hier, T_FormatHier;
#define Boa_Hier_Length(t) Boa_Int_Value(Boa_PType_Item(t, 0))
#define Boa_Hier_Subtype(t) Boa_PType_Item(t, 1)
#define Boa_Hier_Value(t) ((Hier)(t)->value)
#endif

#endif
