/* For each node size, computes the pos and neg stats. */

#include <tpm/tpm.h>
#include <tpm/hier.h>
#include "stats.h"
#include <math.h>

int main(int argc, char *argv[])
{
  Hier hier;
  char str[128];
  int i;
  int tpos, tneg;
  int *bpos, *bneg;
  Stats *s;
  FILE *fp;

  if(argc < 2) {
    fprintf(stderr, "usage: %s hier [index]\n", argv[0]);
    fprintf(stderr, "index defaults to 0\n");
    exit(1);
  }
  hier = HierCreateFromFile(argv[1]);
  if(!hier) {
    fprintf(stderr, "Cannot open `%s'\n", argv[1]);
    exit(1);
  }

  sprintf(str, "%s.bias", argv[1]);
  fp = fopen(str, "r");
  if(!fp) {
    fprintf(stderr, "Cannot open `%s'\n", str);
    exit(1);
  }
  /* skip first line */
  while(getc(fp) != '\n');

  /* skip to the desired bias entry */
  if(argc > 2) {
    int n = atoi(argv[2]) * (hier->num_nodes+1)*2*sizeof(int);
    for(i=0;i < n;i++) getc(fp);
  }
  
  bpos = Allocate(hier->num_nodes, int);
  bneg = Allocate(hier->num_nodes, int);
  ReadAdjust(&tpos, TPM_INT, 1, fp);
  fprintf(stderr, "pos total: %d\n", tpos);
  ReadAdjust(bpos, TPM_INT, hier->num_nodes, fp);
  
  ReadAdjust(&tneg, TPM_INT, 1, fp);
  fprintf(stderr, "neg total: %d\n", tneg);
  ReadAdjust(bneg, TPM_INT, hier->num_nodes, fp);

  fclose(fp);

  s = Allocate(hier->num_nodes, Stats);
  for(i=0;i<hier->num_nodes;i++) {
    s[i] = StatsCreate();
  }
  for(i=0;i<hier->num_nodes;i++) {
    /* crude approximation to the node's size */
    int size = HierRightLeaf(hier, i) - i;
    double sz = (double)(size + 1) / hier->num_nodes;
    double p = (double)bpos[i] / tpos, n = (double)bneg[i] / tneg;
#define BIAS 6
#if BIAS == 1
    StatsAddSample(s[size], (double)bpos[i] / (bpos[i] + bneg[i]));
#elif BIAS == 2
    StatsAddSample(s[size], p / n);
#elif BIAS == 3
    StatsAddSample(s[size], p * (1 - n));
#elif BIAS == 4
    StatsAddSample(s[size], p - n);
#elif BIAS == 5
    StatsAddSample(s[size], sz * (1 - sz));
#elif BIAS == 6
    StatsAddSample(s[size], log((p+.5)/(1-p+.5)*(1-n+.5)/(n+.5)));
#endif
  }
  for(i=0;i<hier->num_nodes;i++) {
    if(s[i]->num_samples) printf("%d %g\n", i, s[i]->mean);
  }
}
