#include "hier.h"
#include <tpm/list.h>

typedef struct {
  int **npos, **nneg; /* [class][item] */
  List **ex; /* [class][item] List of Examples, or nil */
  List *leaf; /* [item] Leaves */
  int enabled;
  int nbias; /* number of biases for this tree */
  int *cbias; /* [class] index of bias used by the class */
  int *bptotal, *bntotal; /* [bias] number of pos/neg examples seen */
  int **bpos, **bneg; /* [bias][item] number of pos/neg examples covered */
  float weight;
} TreeData;

typedef struct LearnHandle {
  /* Tree info */
  int ntrees;
  Hier *trees; /* [tree] */
  TreeData *td; /* [tree] */
  List leafCache; /* for holding shared leaf lists */

  /* Class info */
  int nclasses;
  int *updated; /* [class] 0 if covers need to be collected */
  void **classData; /* [class] */

  /* Example info */
  List *examples; /* [class] positive and negative Examples */
  List *pos_ex; /* [class] positive Examples only */

} *LearnHandle;

/* Number of examples for class */
#define TotalEx(class) ListSize(lh->examples[class])
#define TotalPos(class) ListSize(lh->pos_ex[class])
#define TotalNeg(class) (TotalEx(class) - TotalPos(class))

/* learn.c *******************************************************************/

LearnHandle LearnInit(void);
void LearnFreeTrees(LearnHandle lh);
void LearnFree(LearnHandle lh);
void LearnClear(LearnHandle lh);
void LearnAddTree(LearnHandle lh, Hier tree);
int LearnAddClass(LearnHandle lh);
void LearnEnableTree(LearnHandle lh, int tree);
void LearnDisableTree(LearnHandle lh, int tree);
void LearnAddEx(LearnHandle lh, int class, float weight, 
		void *item, CmpFunc *compare);
void LearnUpdate(LearnHandle lh);
List LearnWithClass(LearnHandle lh, int class);
void LearnCommit(LearnHandle lh);

void LearnUpdateClass(LearnHandle lh, int class);
List LearnWithClassMatching(LearnHandle lh, int class, 
			    void *item, CmpFunc *compare);
int LearnReadBias(LearnHandle lh, int tree, char *fname);
void LearnSaveBias(LearnHandle lh, int tree, char *fname);
void LearnUpdateBias(LearnHandle lh);
void LearnSaveTree(LearnHandle lh, int tree, char *fname);

