/* On-line computation of sample statistics */
#include <tpm/tpm.h>
#include "stats.h"
#include <math.h>   /* for sqrt */

/* Prototypes ****************************************************************/
Stats StatsCreate(void);
void StatsAddSample(Stats s, double x);
double StatsStd(Stats s);

/* Functions *****************************************************************/

Stats StatsCreate(void)
{
  Stats s = AllocateC(1, struct Stats);
  s->min = DBL_MAX;
  s->max = -DBL_MAX;
  return s;
}

#define sqr(x) ((x)*(x))
void StatsAddSample(Stats s, double x)
{
  double delta;  /* change in mean */

  s->num_samples++;
  delta = (x - s->mean) / s->num_samples;
  s->mean += delta;
  if(s->num_samples > 1) {
    s->var += sqr(delta) + (sqr(x - s->mean) - s->var) / (s->num_samples - 1);
  }
  if(x < s->min) s->min = x;
  if(x > s->max) s->max = x;
}

double StatsStd(Stats s)
{
  return sqrt(s->var);
}
