#include "hier.h"

/* Prototypes ****************************************************************/

void HierRemoveValueAll(Hier hier, void *value, CmpFunc *cf);
void HierReduce(Hier hier);

/* Functions *****************************************************************/

static void Strip(Hier orig, int *opos, Hier result, int *rpos, int parent,
		  void *value, CmpFunc *cf)
{
  int i, n;
  HierNode n1, n2;

  n1 = HierNodeAt(orig, *opos);
  (*opos)++;
  if(cf && !cf(n1,value)) {
    HierNodeAt(result, parent)->children--;
    /* skip all children */
    for(i=n1->children;i;i--) {
      i += HierNodeAt(orig, *opos)->children;
      (*opos)++;
    }
    return;
  }

  n2 = HierNodeAt(result, *rpos);
  n2->children = n1->children; /* will be decremented later */
  n2->parent = parent;
  memcpy(&n2->data, &n1->data, result->data_size);
  parent = *rpos; (*rpos)++;

  /* store a copy in case n1 is altered during the loop */
  n = n1->children;
  for(i=0;i<n;i++) {
    Strip(orig, opos, result, rpos, parent, value, cf);
  }
}

/* Removes all nodes from hier such that cf(data, value) == 0.
 */
void HierRemoveValueAll(Hier hier, void *value, CmpFunc *cf)
{
  int i,j;
  for(i=j=0;i < hier->num_nodes;) Strip(hier, &i, hier, &j, -1, value, cf);
  hier->num_nodes = j;
}

static void Mark(Hier hier, int *pos, int *keep)
{
  int i;
  HierNode n = HierNodeAt(hier, *pos);
  keep[*pos] = 1;
  (*pos)++;
  /* check children */
  for(i=0;i<n->children;i++) {
    if(HierNodeAt(hier, *pos+i)->children) break;
    keep[*pos+i] = 0;
  }
  if(i < n->children) {
    /* don't prune this node; recurse on children */
    for(i=0;i<n->children;i++) {
      Mark(hier, pos, keep);
    }
  }
  else *pos += n->children;
}

static void StripMarked(Hier orig, int *opos, Hier result, int *rpos, 
			int parent, int *mark)
{
  int i, n;
  HierNode n1, n2;

  n1 = HierNodeAt(orig, *opos);
  (*opos)++;
  if(!mark[(*opos)-1]) {
    HierNodeAt(result, parent)->children--;
    /* skip all children */
    for(i=n1->children;i;i--) {
      i += HierNodeAt(orig, *opos)->children;
      (*opos)++;
    }
    return;
  }

  n2 = HierNodeAt(result, *rpos);
  n2->children = n1->children; /* will be decremented later */
  n2->parent = parent;
  memcpy(&n2->data, &n1->data, result->data_size);
  parent = *rpos; (*rpos)++;

  /* store a copy in case n1 is altered during the loop */
  n = n1->children;
  for(i=0;i<n;i++) {
    StripMarked(orig, opos, result, rpos, parent, mark);
  }
}

/* Prunes the children of all nodes whose children are all leaves. */
void HierReduce(Hier hier)
{
  int i,j;
  int *keep = Allocate(hier->num_nodes, int);
  i=0; Mark(hier, &i, keep);
  for(i=j=0;i < hier->num_nodes;) StripMarked(hier, &i, hier, &j, -1, keep);
  hier->num_nodes = j;
  free(keep);
}
