/* Decorator for maintaining exclusive classes.
 */
#include <tpm/list.h>
#include <boa/boa.h>

/* does not hold classifiers */
typedef struct Exclusive {
  Obj classifier, classifiers, posEx;
} *Exclusive;

/* Globals *******************************************************************/
Type T_Exclusive;

Obj Boa_MultiClassExclusive_Create(Obj classifier, Obj classifiers,
				   Obj posEx);

/* Functions *****************************************************************/

Obj Boa_MultiClassExclusive_Create(Obj classifier, Obj classifiers,
				   Obj posEx)
{
  Obj self = Boa_Create(T_Exclusive);
  Exclusive e = (Exclusive)self->value;
  e->classifier = classifier;
  e->classifiers = classifiers;
  e->posEx = posEx;
  return self;
}

static void Final(Obj self)
{
  Exclusive e = (Exclusive)self->value;
  Boa_Release(e->classifier);
  Boa_Release(e->posEx);
}

static void Clean(Obj self, ObjFunc *func, Any data)
{
  Exclusive e = (Exclusive)self->value;
  func(e->classifier, data);
  func(e->posEx, data);
}

static int SPrint(Obj self, StrStream ss)
{
  Exclusive e = (Exclusive)self->value;
  return StrStreamPuts(ss, "exclusive");
}

/* forward unchanged to the classifier */
static int ObjGet(Obj self, Obj index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  Exclusive e = (Exclusive)self->value;
  return Boa_ObjGet(e->classifier, index, flags, type_in_out, value_return);
}

/* If a positive example,
 * 1. Store in posEx as the class number.
 * 2. Make a negative for all other classifiers.
 */
static int ObjSet(Obj self, Obj index, int flags, Obj value)
{
  Exclusive e = (Exclusive)self->value;
  if(Boa_Float_Value(value) > 0) {
    int i;
    Obj weight = Boa_Float_Create(-Boa_Float_Value(value));
    for(i=0;i<Boa_Ord_Length(e->classifiers);i++) {
      Obj classifier = Boa_Ord_Item(e->classifiers, i);
      if(classifier == self) {
	assert(e->posEx->type == T_List);
	/* XXX use copy instead of hold 
	   since some callers mutate it afterwards */
	ListAddFront(Boa_List_Value(e->posEx), BoaCopy(index));
	continue;
      }
#if VERBOSE
      fprintf(stderr, "adding negative for class %d\n", i);
#endif
      assert(Boa_ObjSet(classifier, index, BOA_INSERT, Boa_Hold(weight)) == TPM_OK);
    }
    Boa_Release(weight);
  }
  return Boa_ObjSet(e->classifier, index, flags, value);
}

static struct Boa_Methods methods = {
  Empty_Init, Final, Clean,
  SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  Sub_StringGet, Sub_StringSet,
  Sub_IntGet, Sub_IntSet,
  ObjGet, ObjSet
};

Obj Boa_Exclusive_Define(void)
{
  T_Exclusive = Boa_SimpleType_Create("exclusive", &methods, 
				      sizeof(struct Exclusive), sizeof(Obj));
  return T_Exclusive;
}
