/* Converts an index into a vector of distances by drawing a row from a 
 * proximity matrix.
 */
#include <boa/boa.h>
#include <boa/vector.h>

typedef struct Proximity {
  Obj matrix;
} *Proximity;

/* Globals *******************************************************************/
Type T_Proximity;

/* Functions *****************************************************************/

static void Final(Obj self)
{
  Proximity c = (Proximity)self->value;
  Boa_Release(c->matrix);
}

/* should have Clean */

static int StringGet(Obj self, char *name, int flags,
		     Type *type_in_out, Obj *value_return)
{
  Proximity c = (Proximity)self->value;
  if(name[0] == '_') {
    if(!strcmp(name+1, "length")) {
      BOA_TYPECHECK_GET(T_Int);
      if(!c->matrix) return TPM_ERROR;
      if(value_return) 
	*value_return = Boa_Int_Create(Boa_Matrix_Value(c->matrix)->rows);
      return TPM_OK;
    }
    else if(!strcmp(name+1, "matrix")) {
      BOA_TYPECHECK_GET(T_Matrix);
      if(!c->matrix) return TPM_ERROR;
      if(value_return) *value_return = Boa_Hold(c->matrix);
      return TPM_OK;
    }
  }
  return TPM_BOTTOM;
}

static int StringSet(Obj self, char *name, int flags, Obj value)
{
  Proximity c = (Proximity)self->value;
  if(name[0] == '_') {
    if(!strcmp(name+1, "matrix")) {
      if(value->type != T_Matrix) return TPM_ERROR;
      if(c->matrix) Boa_Release(c->matrix);
      c->matrix = value;
      return TPM_OK;
    }
  }
  return TPM_BOTTOM;
}

static int ObjGet(Obj self, Obj index, int flags,
		  Type *type_in_out, Obj *value_return)
{
  Proximity c = (Proximity)self->value;
  if(!c->matrix) return TPM_BOTTOM;
  if(index->type != T_Int) return TPM_BOTTOM;
  BOA_TYPECHECK_GET(T_Vector);
  if(value_return) {
    Vector row = MatrixRow(Boa_Matrix_Value(c->matrix), Boa_Int_Value(index));
    *value_return = Boa_Vector_Create(row);
  }
  return TPM_OK;
}

static struct Boa_Methods methods = {
  Empty_Init, Final, Empty_Clean,
  Generic_SPrint, Err_Copy, Generic_Compare, Generic_Hash,
  StringGet, StringSet,
  Err_IntGet, Err_IntSet,
  ObjGet, Generic_ObjSet
};

Obj Boa_Proximity_Define(void)
{
  Boa_Vector_Pkg();
  T_Proximity = Boa_SimpleType_Create("proximity", &methods,
				      sizeof(struct Proximity), sizeof(Obj));
  return T_Proximity;
}
