/* Test TwoLayer on the 2-bit XOR problem. */
#include "twolayer.h"

#define SCALE 10
#define SHIFT 0

main()
{
  int i;
  TwoLayer net = TwoLayerCreate(2, 1, 2);
  TwoLayerNormalized nnet = TwoLayerNormalizedCreate(net);
  Vector vi,vo;

  /* construct examples */
  for(i=0;i<4;i++) {
    vi = VectorCreate(2);
    vi->d[0] = i%2 * SCALE + SHIFT;
    vi->d[1] = i/2 * SCALE + SHIFT;
    vo = VectorCreate(1);
#if 1
    vo->d[0] = (int)vi->d[0] ^ (int)vi->d[1];
#else
    vo->d[0] = (int)vi->d[0];
#endif
    TwoLayerNormalizedAddExample(nnet, TwoLayerExampleCreate(vi, vo));
  }

  /* train the network */
  if(TwoLayerNormalizedTrain(nnet) != TPM_OK) {
    fprintf(stderr, "Training failed\n");
  }
#if 0
  if(TwoLayerTrainSteepest(net, examples, 0.5, 1000) != TPM_OK) {
    fprintf(stderr, "Training failed after 1000 epochs\n");
  }
#elif 0
  if(TwoLayerTrainSCG(net, examples, 1000) != TPM_OK) {
    fprintf(stderr, "Training failed after 1000 epochs\n");
  }
#elif 0
  if(TwoLayerTrainCG(net, examples) != TPM_OK) {
    fprintf(stderr, "Training failed\n");
  }
#endif

  printf("Learned network:\n");
  MatrixPrint(stdout, net->IH);
  printf("\n");
  MatrixPrint(stdout, net->HO);

  /* check it */
  printf("Check:\n");
  vo = VectorCreate(1);
  {TwoLayerExample ex; ListIter(p, ex, nnet->examples) {
    TwoLayerNormalizedEval(nnet, ex->x, vo);
    printf("%g %g -> %g + %g\n", 
	    ex->x->d[0], ex->x->d[1], ex->y->d[0], vo->d[0] - ex->y->d[0]);
  }}
  VectorFree(vo);

  TwoLayerNormalizedFree(nnet);

  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}
