/* Converters for Hierarchies and Trees. Confusing, isn't it? */

#include <tpm/tree.h>
#include "hier.h"

/* Prototypes ****************************************************************/

Tree TreeFromHier(Hier hier);
void HierFromTree(Hier hier, Tree tree);

/* Functions *****************************************************************/

/* Helper for TreeFromHier.
 * Modifies *pos.
 */
static Tree _TreeFromHier(Hier hier, int *pos)
{
  int i;
  HierNode node = HierNodeAt(hier, *pos);
  /* Make a Tree for this node */
  Tree tree;
  void *data = Allocate(hier->data_size, char);
  memcpy(data, &node->data, hier->data_size);
  tree = TreeCreate(data);
  /* Loop children and gather into Trees */
  for(i=0;i<node->children;i++) {
    (*pos)++;
#if 0
    TreeAddLeftChild(tree, _TreeFromHier(hier, pos));
#else
    TreeAddChild(tree, _TreeFromHier(hier, pos));
#endif
  }
  return tree;
}

Tree TreeFromHier(Hier hier)
{
  Tree tree = NULL, t1;
  int i = 0;
  do {
    Tree t2 = _TreeFromHier(hier, &i);
    i++;
    if(!tree) tree = t1 = t2;
    else {
      t1->sibling = t2;
      t1 = t2;
    }
  } while(i < hier->num_nodes);
  return tree;
}

static void __HierFromTree(Hier hier, int *pos, int parent, Tree tree)
{
  int children = 0;
  Tree child, stop;
  HierNode node = HierNodeAt(hier, *pos);
  node->parent = parent;
  parent = (*pos)++;
#if 1
  /* iterate children in reverse order */
  stop = NULL;
  for(stop=NULL;stop!=tree->child;stop=child) {
    for(child=tree->child;child->sibling != stop;child=child->sibling);
    __HierFromTree(hier, pos, parent, child);
    children++;
  }
#else
  {IterateChildren(tree, child) {
    __HierFromTree(hier, pos, parent, child);
    children++;
  }}
#endif
  node->children = children;
  memcpy(&node->data, tree->data, hier->data_size);
}

static void _HierFromTree(Hier hier, int *pos, int parent, Tree tree)
{
  /* call __HierFromTree on the sibling set */
  for(;tree;tree=tree->sibling) {
    __HierFromTree(hier, pos, parent, tree);
  }
}

/* Fills a hier from tree data. Hier must already exist so that we
 * know the number of nodes and the data size.
 */
void HierFromTree(Hier hier, Tree tree)
{
  int i = 0;
  _HierFromTree(hier, &i, -1, tree);
}
