#ifndef TWOLAYER_H_INCLUDED
#define TWOLAYER_H_INCLUDED

#include <tpm/list.h>
#include <tpm/matrix.h>

typedef struct TwoLayer {
  /* weights from input->hidden and hidden->output.
   * IH->rows == #hidden, IH->cols == #input + 1.
   * HO->rows == #output, HO->cols == #hidden + 1.
   * bias connections are in the last column.
   */
  Matrix IH, HO;
} *TwoLayer;

TwoLayer TwoLayerCreate(int num_input, int num_output, int num_hidden);
void TwoLayerFree(TwoLayer net);
void TwoLayerEval(TwoLayer net, Vector x, Vector y);
double TwoLayerGradient(TwoLayer net, Vector x, Vector y, Matrix IHd, Matrix HOd);
void TwoLayerVHessian(TwoLayer net, Vector x, Vector y, 
		      Matrix IHv, Matrix HOv, Matrix IHh, Matrix HOh);

typedef struct TwoLayerExample {
  Vector x, y;
} *TwoLayerExample;

TwoLayerExample TwoLayerExampleCreate(Vector x, Vector y);
void TwoLayerExampleFree(TwoLayerExample ex);
void TwoLayerExampleNormalize(List examples);

int TwoLayerTrainSteepest(TwoLayer net, List examples, double rate, int epochs);
int TwoLayerTrainSCG(TwoLayer net, List examples, int epochs);
int TwoLayerTrainCG(TwoLayer net, List examples);

typedef struct TwoLayerNormalized {
  TwoLayer net;
  List examples;
  Vector input_mean;
  Vector input_var;
} *TwoLayerNormalized;

TwoLayerNormalized TwoLayerNormalizedCreate(TwoLayer net);
void TwoLayerNormalizedFree(TwoLayerNormalized nnet);
void TwoLayerNormalizedAddExample(TwoLayerNormalized nnet, TwoLayerExample ex);
void TwoLayerNormalizedEval(TwoLayerNormalized nnet, Vector x, Vector y);
int TwoLayerNormalizedTrain(TwoLayerNormalized nnet);

#endif
