%module matrix
%{
#include "matrix.h"
%}

struct VectorRec {
  double *d;
  int len;
};
typedef struct VectorRec *Vector;

%inline %{
double VectorAt(Vector v, int i) {
  return v->d[i];
}
%}

/* vector.c ******************************************************************/

Vector VectorCreate(int length);
Vector VectorFromData(int length, double *data);
void VectorFree(Vector v);
Vector VectorCopy(Vector v);
void VectorResize(Vector v, int len);

void VectorSet(Vector v, double x);
void VectorAddValue(Vector v, double x);
void VectorScale(Vector v, double x);
void VectorApply(Vector v, MathFunc *func);

void VectorFlip(Vector v);
double VectorSum(Vector v);
int VectorMaximum(Vector v);
void VectorMaximumK(Vector v, int k, double *value_return);
void VectorMinMax(Vector v, double *min_return, double *max_return);
void VectorRescale(Vector v, double new_min, double new_max);
void VectorTruncate(Vector v, double new_min, double new_max);

void VectorAdd(Vector v1, Vector v2);
void VectorSubtract(Vector v1, Vector v2);
void VectorMultiply(Vector v1, Vector v2);
void VectorDivide(Vector v1, Vector v2);
double VectorDot(Vector v1, Vector v2);
int VectorCompare(Vector v1, Vector v2);

void VectorPack(Vector a, Vector b, Vector c);
void VectorUnpack(Vector c, Vector a, Vector b);

void VectorNormalizePMF(Vector p);
int VectorSample(Vector p);
double VectorChi(Vector v1, Vector v2);

struct MatrixRec {
  double *d;
  int len;
  int cols;
  double **d2;
  int rows;
};
typedef struct MatrixRec *Matrix;

/* matrix.c ******************************************************************/

Matrix MatrixCreate(int rows, int cols);
Matrix MatrixFromData(int rows, int cols, double *data);
void MatrixFree(Matrix m);
Matrix MatrixCopy(Matrix m);
void MatrixIdentity(Matrix m);
Matrix MatrixFromVector(Vector v, int rows);
void MatrixGetRow(Matrix m, int row, Vector v);
void MatrixSetRow(Matrix m, int row, Vector v);
Vector MatrixRow(Matrix m, int row);
Vector MatrixColumn(Matrix m, int col);
Vector MatrixDiagonal(Matrix m);
Matrix MatrixDeleteRow(Matrix m, int row);
void MatrixReshape(Matrix m, int rows, int cols);

void MatrixTranspose(Matrix m);
Matrix MatrixTransposeCopy(Matrix m);
void MatrixFlipV(Matrix m);
void MatrixFlipH(Matrix m);
Matrix MatrixFilter(Matrix m, Matrix f);
void MatrixColumnSum(Matrix m, Vector sum);
void MatrixSubtractRow(Matrix m, double *row);

void MatrixTransposeTimesSelf(Matrix r, Matrix a);
void MatrixSelfTimesTranspose(Matrix r, Matrix a);
void MatrixMultiply(Matrix dest, Matrix a, Matrix b);
void MatrixVectorMultiply(Vector dest, Matrix a, Vector v);
