#ifndef MATRIXI_H_INCLUDED
#define MATRIXI_H_INCLUDED

#include <tpm/vectorI.h>

/* Designed for bytewise compatibility with Vectors.
 * The data is addressable both as a
 * two-dimensional array (m->d2[row][col]) of dimensions (rows, cols)
 * or a row-major array (m->d[index]) of dimension len = rows * cols.
 * m->d == m->d2[0].
 */
typedef struct MatrixIRec {
  int *d;
  int len;
  int cols;
  int **d2;
  int rows;
} *MatrixI;

/* m1 gets the contents of m2. m1 must be already allocated and have exactly
 * the same dimensions as m2.
 */
#define MatrixIMove(m1, m2) memcpy((m1)->d, (m2)->d, (m2)->len*sizeof(int))

/* matrixI.c *****************************************************************/

MatrixI MatrixICreate(int rows, int cols);
MatrixI MatrixIFromData(int rows, int cols, int *data);
void MatrixIFree(MatrixI m);
MatrixI MatrixICopy(MatrixI m);
MatrixI MatrixIFromVector(VectorI v, int rows);
VectorI MatrixIRow(MatrixI m, int row);
VectorI MatrixIColumn(MatrixI m, int col);

/* test if we have a definition for FILE */
#ifdef EOF
void MatrixIPrint(FILE *fp, MatrixI m);
MatrixI MatrixIScan(FILE *fp);
void MatrixIPrintBinary(FILE *fp, MatrixI m);
MatrixI MatrixIReadBinary(FILE *fp);
#endif

#endif
