#include "optim.h"

/* Matlab's banana function.  Minimum is at banana([1 1]) = 0.
 */
double banana(Vector x)
{
  double a = x->d[1] - x->d[0]*x->d[0];
  double b = 1 - x->d[0];
  return 100*a*a + b*b;
}

void bananaGrad(Vector x, Vector g)
{
  double a = x->d[1] - x->d[0]*x->d[0];
  g->d[0] = -400*a*x->d[0] - 2*(1 - x->d[0]);
  g->d[1] = 200*a;
}

main()
{
  Vector x = VectorCreate(2);
  double fmin;

  x->d[0] = -1.9;
  x->d[1] = 2;
  OptCG(x, (OptMScalarFunc*)banana, NULL, (OptVectorFunc*)bananaGrad, NULL, 
	&fmin);
  assert(banana(x) == fmin);
  printf("OptCG finds minimum %g at ", fmin);
  VectorPrint(stdout, x);
  printf("(should be 0 at [1 1])\n");

  x->d[0] = -1.9;
  x->d[1] = 2;
  OptPowell(x, (OptMScalarFunc*)banana, NULL, &fmin);
  assert(banana(x) == fmin);
  printf("OptPowell finds minimum %g at ", fmin);
  VectorPrint(stdout, x);
  printf("(should be 0 at [1 1])\n");

  VectorFree(x);
  MEM_BLOCKS();
  MEM_STATUS();
  exit(0);
}
