#include <tpm/tpm.h>
#include "matrix.h"

main()
{
  Matrix A, T, U;
  Vector V;
  double *B;
  int i,j;

  A = MatrixCreate(2,2);
  A->d2[0][0] = 1.0;
  A->d2[0][1] = 2.0;
  A->d2[1][0] = 3.0;
  A->d2[1][1] = 4.0;
  printf("A:\n");
  MatrixPrint(stdout, A);
  MatrixReshape(A, 4, 1);
  printf("Reshape(4, 1):\n");
  MatrixPrint(stdout, A);
  MatrixReshape(A, 2, 2);
  MatrixTranspose(A);
  printf("Transpose(A):\n");
  MatrixPrint(stdout, A);
  MatrixTranspose(A);
  MatrixFlipV(A);
  printf("FlipV(A):\n");
  MatrixPrint(stdout, A);
  MatrixFlipV(A);
  MatrixFlipH(A);
  printf("FlipH(A):\n");
  MatrixPrint(stdout, A);
  MatrixFlipH(A);
  printf("Determinant: %g\n", MatrixDeterminant(A));
  MatrixPrint(stdout, A);

  T = MatrixCreate(2,2);
  T->d2[0][0] = 3.0;
  T->d2[0][1] = 1.0;
  T->d2[1][0] = -1.0;
  T->d2[1][1] = 2.0;
  printf("T:\n");
  MatrixPrint(stdout, T);
  MatrixMultiply(T, T, A);
  printf("TA:\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  V = VectorCreate(2);
  V->d[0] = 3;
  V->d[1] = 4;
  printf("V: ");
  VectorPrintNL(stdout, V);
  MatrixVectorMultiply(V, A, V);
  printf("MatrixVectorMultiply(A, V): ");
  VectorPrintNL(stdout, V);

  B = Allocate(2, double);
  B[0] = 1.0;
  B[1] = 2.0;
  printf("B: %g %g\n", B[0], B[1]);
  MatrixSolve(A, B);
  printf("Solve(A, B): %g %g\n", B[0], B[1]);

  T = MatrixCopy(A);
  MatrixInvert(T);
  printf("Invert(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  printf("\n");

#ifdef BIHAR
  T = MatrixCopy(A);
  MatrixFourier(T, NULL);
  printf("Fourier(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  T = MatrixCopy(A);
  MatrixTwoDimFourier(T, NULL);
  printf("TwoDimFourier(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);
#endif

  T = MatrixCopy(A);
  VectorDFT((Vector)T, NULL);
  printf("VectorDFT(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  T = MatrixCopy(A);
  MatrixDFT(T, NULL);
  printf("MatrixDFT(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  printf("\n");

#ifdef BIHAR
  T = MatrixCopy(A);
  MatrixFourier(T, (Matrix*)1);
  printf("Magnitude Fourier(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  T = MatrixCopy(A);
  MatrixTwoDimFourier(T, (Matrix*)1);
  printf("Magnitude TwoDimFourier(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);
#endif

  T = MatrixCopy(A);
  VectorDFT((Vector)T, (Vector)1);
  printf("Magnitude VectorDFT(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  T = MatrixCopy(A);
  MatrixDFT(T, (Matrix)1);
  printf("Magnitude MatrixDFT(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  printf("\n");

  A->d2[0][0] = 2.0;
  A->d2[0][1] = 1.0;
  A->d2[1][0] = 1.0;
  A->d2[1][1] = 3.0;
  printf("A:\n");
  MatrixPrint(stdout, A);
  MatrixEigenvalues(A, B, NULL);
  printf("Eigenvalues(A): %g %g\n", B[0], B[1]);
  MatrixCholesky(A);
  printf("Cholesky(A):\n");
  MatrixPrint(stdout, A);
  MatrixFree(A);
  free(B);

  printf("\n");

  A = MatrixCreate(3, 3);
  A->d2[0][0] = 1.0;
  A->d2[0][1] = 2.0;
  A->d2[0][2] = 3.0;
  A->d2[1][0] = 2.0;
  A->d2[1][1] = 1.0;
  A->d2[1][2] = 4.0;
  A->d2[2][0] = 3.0;
  A->d2[2][1] = 4.0;
  A->d2[2][2] = 1.0;
  printf("A:\n");
  MatrixPrint(stdout, A);
  B = Allocate(3, double);
  MatrixEigenvectorsSym(A, B);
  printf("Eigenvectors(A):\n");
  MatrixPrint(stdout, A);
  printf("Eigenvalues(A): %g %g %g\n", B[0], B[1], B[2]);
  MatrixFree(A);
  free(B);

  A = MatrixCreate(3, 3);
  A->d2[0][0] = 1.0;
  A->d2[0][1] = 2.0;
  A->d2[0][2] = 3.0;
  A->d2[1][0] = 2.0;
  A->d2[1][1] = 1.0;
  A->d2[1][2] = 4.0;
  A->d2[2][0] = 3.0;
  A->d2[2][1] = 4.0;
  A->d2[2][2] = 1.0;
  printf("A:\n");
  MatrixPrint(stdout, A);
  B = Allocate(2, double);
  T = MatrixCreate(2, 3);
  MatrixEigenvectorsSymX(A, B, 2, T);
  printf("Eigenvectors(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);
  printf("Eigenvalues(A): %g %g\n", B[0], B[1]);
  MatrixFree(A);
  free(B);

  printf("\n");

  A = MatrixCreate(8, 4);
  for(i=0;i<8;i++) {
    for(j=0;j<4;j++) {
      A->d2[i][j] = sin(i+0.5)*cos(j+0.5);
    }
  }
  printf("A:\n");
  MatrixPrint(stdout, A);

#if 0
  MatrixTranspose(A);
  printf("Transpose(A):\n");
  MatrixPrint(stdout, A);
  MatrixTranspose(A);
#endif

  V = MatrixRow(A, 0);
  VectorDCT(V);
  printf("VectorDCT(A[0]):\n");
  VectorPrintNL(stdout, V);
  VectorFree(V);

  T = MatrixCopy(A);
  MatrixDCT(T);
  printf("MatrixDCT(A):\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);

  MatrixFree(A);

  printf("\n");

  A = MatrixCreate(3, 2);
  A->d2[0][0] = 1;
  A->d2[0][1] = 2;
  A->d2[1][0] = 3;
  A->d2[1][1] = 4;
  A->d2[2][0] = 5;
  A->d2[2][1] = 6;
  B = Allocate(3, double);
  B[0] = 3;
  B[1] = 2;
  B[2] = 1;
  printf("A:\n");
  MatrixPrint(stdout, A);
  printf("B: %g %g %g\n", B[0], B[1], B[2]);
  MatrixLLS(A, B, B);
  /* should be -4 3.5 */
  printf("LLS(A, B): %g %g\n", B[0], B[1]);
  MatrixTranspose(A);
  printf("A:\n");
  MatrixPrint(stdout, A);
  B[0] = 3;
  B[1] = 2;
  printf("B: %g %g\n", B[0], B[1]);
  MatrixLLS(A, B, B);
  /* should be -1.83333 -0.333333 1.16667 */
  printf("LLS(A, B): %g %g %g\n", B[0], B[1], B[2]);
  MatrixFree(A);
  free(B);

  printf("\n");

  A = MatrixCreate(3, 2);
  A->d2[0][0] = 1;
  A->d2[0][1] = 2;
  A->d2[1][0] = 3;
  A->d2[1][1] = 4;
  A->d2[2][0] = 5;
  A->d2[2][1] = 6;
  printf("A:\n");
  MatrixPrint(stdout, A);
  T = MatrixCreate(3, 3);
  MatrixSelfTimesTranspose(T, A);
  printf("AA':\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);
  T = MatrixCreate(2, 2);
  MatrixTransposeTimesSelf(T, A);
  printf("A'A:\n");
  MatrixPrint(stdout, T);
  MatrixFree(T);
  MatrixInvert(A);
  printf("inv(A):\n");
  MatrixPrint(stdout, A);
  MatrixFree(A);

  A = MatrixCreate(3, 2);
  A->d2[0][0] = 1;
  A->d2[0][1] = 2;
  A->d2[1][0] = 3;
  A->d2[1][1] = 4;
  A->d2[2][0] = 5;
  A->d2[2][1] = 6;
  U = MatrixCreate(3, 2);
  T = MatrixCreate(2, 2);
  V = VectorCreate(2);
  MatrixSVD(A, U, T, V);
  printf("U:\n");
  MatrixPrint(stdout, U);
  printf("V:\n");
  MatrixPrint(stdout, T);
  printf("S:\n");
  VectorPrintNL(stdout, V);
  MatrixFree(A);
  MatrixFree(U);
  MatrixFree(T);
  VectorFree(V);

  printf("\n");

  A = MatrixCreate(3, 2);
  A->d2[0][0] = 1;
  A->d2[0][1] = 2;
  A->d2[1][0] = 3;
  A->d2[1][1] = 6;
  A->d2[2][0] = 5;
  A->d2[2][1] = 10;
  printf("A:\n");
  MatrixPrint(stdout, A);
  MatrixPseudoInvert(A);
  printf("pinv(A):\n");
  MatrixPrint(stdout, A);
  MatrixFree(A);

  printf("\n");

  MEM_BLOCKS();
  MEM_STATUS();
}
