#include <tpm/tpm.h>
#include <tpm/matrixSparseP.h>

void MatrixSparsePPrint(FILE *fp, MatrixSparseP m)
{
  int i, j;
  for(i=0;i<m->rows;i++) {
    for(j=0;j<m->cols;j++) {
      void *v = MatrixSparsePAt(m, i, j);
      if(!v) fprintf(fp, "   ");
      else fprintf(fp, "%2d ", v);
    }
    fprintf(fp, "\n");
  }
}

void MatrixSparsePDebug(FILE *fp, MatrixSparseP m)
{
  int i;
  fprintf(fp, "rows: ");
  for(i=0;i<m->rows;i++) fprintf(fp, "(%2d, %2d) ", m->start[i], m->end[i]);
  fprintf(fp, "\ncol: ");
  for(i=0;i<m->len;i++) fprintf(fp, "%2d ", m->col[i]);
  fprintf(fp, "\nd:   ");
  for(i=0;i<m->len;i++) fprintf(fp, "%2d ", m->d[i]);
  fprintf(fp, "\n");
}

main()
{
  int i;
  MatrixSparseP m = MatrixSparsePCreate(4, 0);
  m->cols = 4;
  srand(getpid());
  for(i=0;i<8;i++) {
    MatrixSparsePInsert(m, i/2, (i%2)*2, (void*)(i+1));
  }
  MatrixSparsePPrint(stdout, m);
  MatrixSparsePDebug(stderr, m);
  printf("\n");

  MatrixSparsePInsert(m, 1, 3, (void*)7);
  MatrixSparsePPrint(stdout, m);
  MatrixSparsePDebug(stderr, m);
  printf("\n");

  MatrixSparsePDelete(m, 0, 0);
  MatrixSparsePDelete(m, 3, 2);
  MatrixSparsePDelete(m, 1, 2);
  MatrixSparsePDelete(m, 1, 0);
  MatrixSparsePDelete(m, 2, 2);
  MatrixSparsePPrint(stdout, m);
  MatrixSparsePDebug(stderr, m);
  printf("\n");

  MatrixSparsePInsert(m, 1, 2, (void*)8);
  MatrixSparsePInsert(m, 1, 1, (void*)9);
  MatrixSparsePPrint(stdout, m);
  MatrixSparsePDebug(stderr, m);
  printf("\n");

  MatrixSparsePFree(m);

  MEM_BLOCKS();
  MEM_STATUS();
}
