#ifndef VECTOR_H_INCLUDED
#define VECTOR_H_INCLUDED
/* Simple Vector data type. */
typedef struct VectorRec {
  double *d;
  int len;
} *Vector;

typedef double MathFunc(double x);

/* v1 gets the contents of v2. v1 must be already allocated and exactly
 * as long as v2.
 */
#define VectorMove(v1, v2) memcpy((v1)->d, (v2)->d, (v2)->len*sizeof(double))

#ifdef __cplusplus
extern "C" {
#endif

/* vector.c ******************************************************************/

Vector VectorCreate(int length);
Vector VectorFromData(int length, double *data);
void VectorFree(Vector v);
Vector VectorCopy(Vector v);
void VectorResize(Vector v, int len);

void VectorSet(Vector v, double x);
void VectorAddValue(Vector v, double x);
void VectorScale(Vector v, double x);
void VectorApply(Vector v, MathFunc *func);

void VectorFlip(Vector v);
double VectorSum(Vector v);
int VectorMaximum(Vector v);
void VectorMaximumK(Vector v, int k, double *value_return);
void VectorMinMax(Vector v, double *min_return, double *max_return);
void VectorRescale(Vector v, double new_min, double new_max);
void VectorTruncate(Vector v, double new_min, double new_max);

void VectorAdd(Vector v1, Vector v2);
void VectorSubtract(Vector v1, Vector v2);
void VectorMultiply(Vector v1, Vector v2);
void VectorDivide(Vector v1, Vector v2);
double VectorDot(Vector v1, Vector v2);
int VectorCompare(Vector v1, Vector v2);

void VectorPack(Vector a, Vector b, Vector c);
void VectorUnpack(Vector c, Vector a, Vector b);

void VectorNormalizePMF(Vector p);
int VectorSample(Vector p);
double VectorChi(Vector v1, Vector v2);

#ifdef EOF
void VectorPrint(FILE *fp, Vector v);
void VectorPrintNL(FILE *fp, Vector v);
#endif

/* vmath.c *******************************************************************/

double VectorMag(Vector v);
double VectorDistance(Vector v1, Vector v2);
double VectorWeightedDistance(Vector v1, Vector v2, Vector w);
double VectorDistanceLp(Vector v1, Vector v2, double p);
void VectorHypot(Vector A, Vector B);
double VectorEntropy(Vector v);
double VectorRelEntropy(Vector p, Vector q);

#ifdef __cplusplus
}
#endif

#endif
