#ifndef VECTORI_H_INCLUDED
#define VECTORI_H_INCLUDED

/* Simple Vector data type. */
typedef struct VectorIRec {
  int *d;
  int len;
} *VectorI;

/* v1 gets the contents of v2. v1 must be already allocated and exactly
 * as long as v2.
 */
#define VectorIMove(v1, v2) memcpy((v1)->d, (v2)->d, (v2)->len*sizeof(int))

#ifdef __cplusplus
extern "C" {
#endif

/* vectorI.c *****************************************************************/

VectorI VectorICreate(int length);
VectorI VectorIFromData(int length, int *data);
void VectorIFree(VectorI v);
VectorI VectorICopy(VectorI v);
void VectorIResize(VectorI v, int len);

void VectorISet(VectorI v, int x);
void VectorIAddValue(VectorI v, int x);
void VectorIScale(VectorI v, int x);

double VectorISum(VectorI v);
void VectorIMinMax(VectorI v, int *min_return, int *max_return);
void VectorITruncate(VectorI v, int new_min, int new_max);

void VectorIAdd(VectorI v1, VectorI v2);
void VectorISubtract(VectorI v1, VectorI v2);
void VectorIMultiply(VectorI v1, VectorI v2);
void VectorIDivide(VectorI v1, VectorI v2);

#ifdef _STDIO_INCLUDED
void VectorIPrint(FILE *fp, VectorI v);
#endif

#ifdef __cplusplus
}
#endif

#endif
