/* Definitions for doubly-linked list */
#ifndef DLIST_H_INCLUDED
#define DLIST_H_INCLUDED

#include <tpm/tpm.h>

typedef struct DList {
  Any data;
  struct DList *prev, *next;
} *DList;

typedef struct DListData {
  FreeFunc *ff;
  unsigned size;
} *DListData;

#define DListFront(list) (list)->next
#define DListRear(list) (list)->prev
#define DListSize(list) ((DListData)(list)->data)->size
#define DListFreeFunc(list) ((DListData)(list)->data)->ff
/* A typical use of this iterator is
 * {DListIter(p, d, list) {
 *   Print(d);  or  Print(p->data);
 * }}
 */
#define DListIter(p, d, list) \
  DList p;for(p = DListFront(list);(p!=list)&&(d=p->data,p);p=p->next)

DList DListCreate(FreeFunc *ff);
void DListRemoveAll(DList list);
void DListFree(DList list);

void DListAddFront(DList list, Any value);
void DListAddRear(DList list, Any value);
void DListRemove(DList list, DList node, Any *value_return);
void DListRemoveMatching(DList list, Any value, CmpFunc *compare);

#endif
