#ifndef ERROR_H_INCLUDED
#define ERROR_H_INCLUDED

#include <setjmp.h>

enum { TPM_OK = 0, TPM_ERROR = -1, TPM_BOTTOM = -2 };
#define TRY_RETURN(x) if((x) != TPM_OK) return TPM_ERROR; else

extern char *ErrorFile;
extern int ErrorLine;
extern char *ErrorString;
extern jmp_buf *ErrorJmpBufPtr;

extern char TPM_EEMPTY[], TPM_EFULL[], TPM_EINT[], TPM_EFLOAT[];

#ifdef __cplusplus
extern "C" {
#endif

void ErrorPrint(FILE *fp);
void ErrorClear(void);
void ErrorSet(const char *format, ...);
void ErrorPrepend(const char *format, ...);
void ErrorAppend(const char *format, ...);
void ErrorSetErrno(void);
void ErrorAppendErrno(void);

void ErrorJump(int code);
void ErrorThrow(int code, char *format, ...);

/* The following nifty exception handling code is from Matt Miller.  */
/* ----------------------------------------------------------------- *
 *                                                                   *
 *             Copyright (c) 1995, NEC Research Institute            *
 *                       All Rights Reserved.                        *
 *                                                                   *
 *   Permission to use, copy, and modify this software and its       *
 *   documentation is hereby granted only under the following terms  *
 *   and conditions.  Both the above copyright notice and this       *
 *   permission notice must appear in all copies of the software,    *
 *   derivative works or modified versions, and any portions         *
 *   thereof, and both notices must appear in supporting             *
 *   documentation.                                                  *
 *                                                                   *
 *   Correspondence should be directed to NEC at:                    *
 *                                                                   *
 *                     Ingemar J. Cox                                *
 *                                                                   *
 *                     NEC Research Institute                        *
 *                     4 Independence Way                            *
 *                     Princeton                                     *
 *                     NJ 08540                                      *
 *                                                                   *
 *                     phone:  609 951 2722                          *
 *                     fax:  609 951 2482                            *
 *                     email:  ingemar@research.nj.nec.com (Inet)    *
 *                                                                   *
 *********************************************************************/

#define TRY \
{                                                \
   jmp_buf *prev_ErrorJmpBuf = ErrorJmpBufPtr;   \
   jmp_buf local_ErrorJmpBuf;                    \
   int local_ErrorCode;                          \
   ErrorJmpBufPtr = &local_ErrorJmpBuf;          \
   local_ErrorCode = setjmp(local_ErrorJmpBuf);  \
   if(local_ErrorCode == 0)

#define CLEANUP \
ErrorJmpBufPtr = prev_ErrorJmpBuf;

#define CATCH \
ErrorJmpBufPtr = prev_ErrorJmpBuf; \
if(local_ErrorCode != 0)

/* These two go at the end of the handler */
#define RETURN_ERROR	\
return(local_ErrorCode);

#define RETHROW \
ErrorJump(local_ErrorCode);

#define END_TRY \
  ErrorJmpBufPtr = prev_ErrorJmpBuf; }

#define THROW ErrorFile = __FILE__, ErrorLine = __LINE__, ErrorThrow

#ifdef __cplusplus
}
#endif

#endif
