/* Definition of HashTable */
#ifndef HASH_H_INCLUDED
#define HASH_H_INCLUDED

#include <tpm/tpm.h>

typedef struct HashEntry {
  Any value;
  struct HashEntry *next;
  Any key;
  int bucket;
} HashEntry;

typedef struct HashTable {
  /* Each bucket is a singly-linked list of HashEntries */
  HashEntry **buckets;
  /* occupancy stores the number of items per group of buckets */
  /* useful for finding items by index */
  int *occupancy;
  int num_buckets, num_entries;
  HashFunc *hash;
  CmpFunc *compare;
  FreeFunc *key_free, *value_free;
} *HashTable;

#define HashTableSize(ht) (ht)->num_entries

/* These iterators don't support:
 * 1. Modification of the table.
 * 2. The "break" statement (will only stop the inner loop).
 */
#define HashIterate(b, e, ht) \
  for(b=0; b<(ht)->num_buckets; b++) \
     for(e=(ht)->buckets[b]; e; e=e->next)

#define HashIter(b, e, v, ht) \
  int b; HashEntry *e; for(b=0; b<(ht)->num_buckets; b++) \
          for(e=(ht)->buckets[b]; e && (v=(e)->value,e); e=e->next)

HashTable HashTableCreate(HashFunc *hash, CmpFunc *compare, 
			  FreeFunc *key_free, FreeFunc *value_free);
void HashTableFlush(HashTable ht);
void HashTableFree(HashTable ht);
HashTable HashTableCopy(HashTable ht, 
			CopyFunc *key_copy, CopyFunc *value_copy);
int HashTableCompare(HashTable a, HashTable b, CmpFunc *compare);

HashEntry *HashTableAddEntry(HashTable ht, Any key, int *found_return);
HashEntry *HashTableFindEntry(HashTable ht, Any key);
void HashTableRemoveEntry(HashTable ht, HashEntry *entry);
void HashTableRemoveNoFree(HashTable ht, HashEntry *entry);
int HashTableRemoveValueAll(HashTable ht, Any value, CmpFunc *cf);
HashEntry *HashTableNextEntry(HashTable ht, HashEntry *entry);

int HashTableIndexEntry(HashTable ht, HashEntry *entry);
HashEntry *HashTableEntryAtIndex(HashTable ht, int index);

#endif
