/* Definitions for using Streams and DiskArrays */
#ifndef STREAM_H_INCLUDED
#define STREAM_H_INCLUDED

#include <tpm/tpm.h>

typedef FILE *FileHandle;
FileHandle FileOpen(char *fname, char *mode);
void FileClose(FileHandle h);
FileHandle fdup(FileHandle h);
int getline(char *buffer, int bufsize, FILE *fp);
int FileCheckHeader(FILE *fp, char *data, int length);
size_t FileSize(FileHandle h);

/* A Stream is an abstraction for anything that supports the open,
 * read, close protocol. All Streams support StreamNext, which reads
 * the next piece of data from the Stream. Some Streams also support
 * StreamRead, which provides random access. A DiskArray is a type of Stream.
 */
typedef int StreamReadFunc(void *source, void *data, int index, void *dest);
typedef int StreamCloseFunc(void *source, void *data);

typedef struct {
  void *source;
  void *data;
  StreamReadFunc *readFunc;
  StreamCloseFunc *closeFunc;
} Stream;

#define StreamRead(s, i, d) (s).readFunc((s).source, (s).data, i, d)
#define StreamNext(s, d) (s).readFunc((s).source, (s).data, -1, d)
#define StreamClose(s) (s).closeFunc((s).source, (s).data)

void DiskArrayOpen(Stream *s, FileHandle h, int num_items, int item_size);

typedef int Tpm_ReadCharFunc(Any data);
typedef int Tpm_WriteCharFunc(char c, Any data);

int Tpm_ReadIntBinary(int *value_return, Tpm_ReadCharFunc *readChar, Any data);
int Tpm_WriteIntBinary(int value, Tpm_WriteCharFunc *writeChar, Any data);
int Tpm_ReadFloatBinary(float *value_return, Tpm_ReadCharFunc *readChar,
			Any data);
int Tpm_WriteFloatBinary(float value, Tpm_WriteCharFunc *writeChar, Any data);
int Tpm_ReadDoubleBinary(double *value_return, Tpm_ReadCharFunc *readChar,
			 Any data);
int Tpm_WriteDoubleBinary(double value, Tpm_WriteCharFunc *writeChar, Any data);
int Tpm_ReadStringBinary(char **value_return, Tpm_ReadCharFunc *readChar, 
			 Any data);
int Tpm_WriteStringBinary(char *value, Tpm_WriteCharFunc *writeChar, Any data);

#endif
