/* Definitions for String Streams. 
 * Characters go in at wpos, come out at rpos.
 * Put operations will resize the string, if neccessary, up to a limit.
 */
#ifndef STRSTREAM_H_INCLUDED
#define STRSTREAM_H_INCLUDED

/* StrStream structure (transparent) */
typedef struct StrStream {
  char *data;
  unsigned rpos, wpos;   /* index into data */
  unsigned len;   /* length of data */
  unsigned limit; /* maximum length; 0 means fixed to len (no realloc) */
} *StrStream;

#define STRSTREAM_UNLIMITED ((unsigned)-1)
#define StrStreamEmpty(ss) ((ss)->rpos == (ss)->wpos)
#define StrStreamRewind(ss) (ss)->rpos = (ss)->wpos = 0
#define StrStreamPeek(ss) (ss)->data[(ss)->rpos]

StrStream StrStreamCreate(int len, int limit);
void StrStreamFree(StrStream ss);
StrStream StrStreamFromData(char *s, int len, int limit);
StrStream StrStreamFromString(char *s, int len, int limit);
char *StrStreamAsString(StrStream ss);
int StrStreamSeekR(StrStream ss, long offset, int whence);
int StrStreamSeekW(StrStream ss, long offset, int whence);

int StrStreamPut(StrStream ss, char *data, int len);
int StrStreamPutc(StrStream ss, char c);
int StrStreamPuts(StrStream ss, char *s);
int StrStreamPutInt(StrStream ss, int i);
int StrStreamPutDouble(StrStream ss, double d);

int StrStreamGet(StrStream ss, char *buffer, int len);
int StrStreamGetc(StrStream ss, char *c);
char *StrStreamGetLine(StrStream ss);
char *StrStreamGetWord(StrStream ss);
char *StrStreamGetSexp(StrStream ss);
int StrStreamGetInt(StrStream ss, int *i);
int StrStreamGetDouble(StrStream ss, double *dp);
int StrStreamUnget(StrStream ss, int i);

int StrStreamScanWord(FILE *fp, StrStream ss);
int StrStreamScanSexp(FILE *fp, StrStream ss);
int StrStreamScanLine(FILE *fp, StrStream ss);

#endif
