#ifndef CACHE_H_INCLUDED
#define CACHE_H_INCLUDED

#include <tpm/list.h>
#include <tpm/hash.h>

typedef Any CacheableFunc(Any x, Any data);

/* A Cache is a small buffer for storing the outputs of frequently-used
 * inputs to a function f.
 */
typedef struct Cache {
  CacheableFunc *f;
  Any data;
  CopyFunc *key_copy, *value_copy;
  HashTable dict;
  List queue;
  unsigned item_limit;
  unsigned hits, misses;
} *Cache;

Cache CacheCreate(CacheableFunc *f, Any data,
		  HashFunc *hash, CmpFunc *compare,
		  CopyFunc *key_copy, CopyFunc *value_copy, 
		  FreeFunc *key_free, FreeFunc *value_free,
		  unsigned item_limit);
void CacheFree(Cache c);
Any CacheGet(Any key, Cache c);
Any CachePeek(Any key, Cache c);

#endif
