#include <tpm/tpm.h>
#include <errno.h>

/* Globals *******************************************************************/

char *ErrorFile = NULL;
int ErrorLine;
char *ErrorString = "unknown";
static char ErrorStringBuf[1024];
jmp_buf *ErrorJmpBufPtr = NULL;

#ifdef __GNUC__
extern const char *const sys_errlist[];
#else
extern char *sys_errlist[];
#endif
char TPM_EEMPTY[] = "stream empty";
char TPM_EFULL[] = "stream full";
char TPM_EINT[] = "expected integer";
char TPM_EFLOAT[] = "expected float";

/* Prototypes ****************************************************************/

void ErrorPrint(FILE *fp);
void ErrorClear(void);
void ErrorSet(const char *format, ...);
void ErrorPrepend(const char *format, ...);
void ErrorAppend(const char *format, ...);
void ErrorSetErrno(void);
void ErrorAppendErrno(void);

void ErrorJump(int code);
void ErrorThrow(int code, char *format, ...);

/* Functions *****************************************************************/

void ErrorPrint(FILE *fp)
{
  fprintf(fp, "Error: %s\n", ErrorString);
}

void ErrorClear(void)
{
  strcpy(ErrorStringBuf, "unknown");
  ErrorString = ErrorStringBuf;
}

static void ErrorSetX(const char *format, va_list args)
{
  if(ErrorFile) {
    sprintf(ErrorStringBuf, "%s line %d: ", ErrorFile, ErrorLine);
  } else {
    ErrorStringBuf[0] = 0;
  }
  vsprintf(ErrorStringBuf + strlen(ErrorStringBuf), format, args);
  ErrorString = ErrorStringBuf;
}

void ErrorSet(const char *format, ...)
{
  va_list args;
  va_start(args, format);
  ErrorSetX(format, args);
  va_end(args);
}

void ErrorPrepend(const char *format, ...)
{
  char buf[1024];
  va_list args;

  strcpy(buf, ErrorString);

  va_start(args, format);
  ErrorSetX(format, args);
  va_end(args);

  strcat(ErrorString, ": ");
  strcat(ErrorString, buf);
}

void ErrorAppend(const char *format, ...)
{
  va_list args;
  va_start(args, format);
  if(ErrorString != ErrorStringBuf) {
    strcpy(ErrorStringBuf, ErrorString);
    ErrorString = ErrorStringBuf;
  }
  strcat(ErrorStringBuf, ": ");
  vsprintf(ErrorStringBuf + strlen(ErrorStringBuf), format, args);
}

void ErrorSetErrno(void)
{
  ErrorString = (char*)sys_errlist[errno];
}

void ErrorAppendErrno(void)
{
  ErrorAppend(sys_errlist[errno]);
}

void ErrorJump(int code)
{
  if(!code) return;  /* do nothing */
  if(!ErrorJmpBufPtr) {
    ErrorPrint(stderr);
    /* could also dump core */
    exit(-1);
  }
  longjmp(*ErrorJmpBufPtr, code);
}

void ErrorThrow(int code, char *format, ...)
{
  /* Copy of ErrorSet */
  va_list args;
  va_start(args, format);
  ErrorSetX(format, args);
  va_end(args);

  ErrorJump(code);
}
