#ifndef INDEX_H_INCLUDED
#define INDEX_H_INCLUDED

/* An Index is an O(1) mapping from a key to a unique integer and
 * back again.  Keys are numbered on order of arrival.
 */
#include <tpm/hash.h>

typedef struct Index {
  Any *keys;       /* maps int -> key */
  int length, bufsize;
  HashTable dict;  /* maps key -> int */
  FreeFunc *key_free;
  int missing;     /* first index which is not in the dict */
} *Index;

Index IndexCreate(HashFunc *hash, CmpFunc *compare, FreeFunc *key_free);
void IndexFree(Index ind);
void IndexInsert(Index ind, Any key);
int IndexDelete(Index ind, int i);
Any IndexAtIndex(Index ind, int i);
int IndexAtKey(Index ind, Any key);
#ifdef EOF
void IndexPrint(FILE *fp, Index ind);
#endif

#endif
