#ifndef NAME_H_INCLUDED
#define NAME_H_INCLUDED

#include <tpm/hash.h>

/* transparent */
typedef struct NameContext {
  struct NameContext *parent;
  HashTable entries;
} *NameContext;

/* This function will be called to handle name misses.
 * If name starts with a slash, then only contexts may be
 * returned.
 */
struct NameSpace;
typedef void *NameUnknownFunc(struct NameSpace *ns, NameContext context, 
			      char *name, int *type_return, void *userData);

/* transparent */
typedef struct NameSpace {
  NameContext root, current;
  FreeFunc *ff;
  NameUnknownFunc *unknown;
  void *userData;
} *NameSpace;

#define NameSetUnknown(ns, func, data) { (ns)->unknown = func; (ns)->userData = data; }
#define NameRoot(ns) (ns)->root
#define NameGetContext(ns) (ns)->current
#define NameSetContext(ns, context) ((ns)->current = context)
#define NameCreateContext(ns, name) \
  NameCreateContextInContext(ns, NameGetContext(ns), name)
#define NameCreateObject(ns, name, object) \
  NameCreateObjectInContext(ns, NameGetContext(ns), name, object)
#define NameLookup(ns, name, type) \
  NameLookupInContext(ns, NameGetContext(ns), name, type)
#define NameRemove(ns, name) \
  NameRemoveInContext(ns, NameGetContext(ns), name)

/* name.c ********************************************************************/

NameSpace NameSpaceCreate(FreeFunc *ff);
void NameSpaceFree(NameSpace ns);
NameContext NameCreateContextInContext(NameSpace ns, NameContext context, 
				       char *name);
int NameCreateObjectInContext(NameSpace ns, NameContext context, 
			      char *name, void *object);
void *NameLookupInContext(NameSpace ns, NameContext context,
			  char *name, int *type_return);
int NameRemoveInContext(NameSpace ns, NameContext context, char *name);
void NameDump(FILE *fp, NameContext context);
char *NameOfContext(NameContext context);

#endif
