#include "name.h"

#define Err(str) { fprintf(stderr, str); exit(1); }

void *Resolver(NameSpace ns, NameContext context, char *name, 
	       int *type_return, void *userData)
{
  void *result;
  /* create an object if it starts with an 'f' */
  if(name[0] == 'f') {
    if(type_return) *type_return = 0;
    result = strdup(name);
    if(NameCreateObjectInContext(ns, context, name, result))
      return result;
    free(result);
  }
  /* create a context if it starts with a 'b' */
  else if((name[0] == 'b') || (name[0] == '/' && name[1] == 'b')) {
    if(type_return) *type_return = 1;
    result = NameCreateContextInContext(ns, context, name[0]=='/'?name+1:name);
    if(result) return result;
  }
  return NULL;
}

void main(void)
{
  NameSpace ns;
  NameContext context;
  int status;

  ns = NameSpaceCreate(GenericFree);
  
  /* failure tests */
  context = NameCreateContext(ns, "foo/");
  if(context) Err("NameCreateContext didn't fail\n");
  context = NameCreateContext(ns, "//foo");
  if(context) Err("NameCreateContext didn't fail\n");
  context = NameCreateContext(ns, "/foo//bar");
  if(context) Err("NameCreateContext didn't fail\n");

  context = NameCreateContext(ns, "foo");
  if(!context) Err("NameCreateContext(foo) failed\n");

  /* relative addressing */
  NameSetContext(ns, context);
  context = NameCreateContext(ns, "bar");
  if(!context) Err("NameCreateContext(bar) failed\n");
  context = NameCreateContext(ns, "bar/faz");
  if(!context) Err("NameCreateContext(/bar) failed\n");
  status = NameCreateObject(ns, "ob1", strdup("ob1"));
  if(!status) Err("NameCreateObject(ob1) failed\n");

  /* absolute addressing */
  context = NameCreateContext(ns, "/bar");
  if(!context) Err("NameCreateContext(/bar) failed\n");
  context = NameCreateContext(ns, "/bar/baz");
  if(!context) Err("NameCreateContext(/bar) failed\n");
  status = NameCreateObject(ns, "/ob2", strdup("ob2"));
  if(!status) Err("NameCreateObject(ob2) failed\n");
  status = NameCreateObject(ns, "/bar/ob3", strdup("ob3"));
  if(!status) Err("NameCreateObject(ob3) failed\n");

  /* lookup */
  context = NameLookup(ns, "/faz", &status);
  if(context) Err("NameLookup(/faz) didn't fail\n");
  context = NameLookup(ns, "/faz/foo", &status);
  if(context) Err("NameLookup(/faz/foo) didn't fail\n");
  context = NameLookup(ns, "/foo", &status);
  if(status != 1) Err("NameLookup(/foo) returned wrong type\n");
  if(context != NameGetContext(ns)) Err("NameLookup(/foo) failed\n");
  context = NameLookup(ns, "ob1", &status);
  if(status != 0) Err("NameLookup(ob1) returned wrong type\n");
  if(strcmp((char*)context, "ob1")) Err("NameLookup(ob1) failed\n");

  NameDump(stdout, NameRoot(ns));
  printf("\n");

  /* remove */
  status = NameRemove(ns, "/faz");
  if(status) Err("NameRemove(/faz) didn't fail");
  status = NameRemove(ns, "/bar/ob3");
  if(!status) Err("NameRemove(ob3) failed");
  status = NameRemove(ns, "bar");
  if(!status) Err("NameRemove(bar) failed");

  /* resolver tests */
  NameSetUnknown(ns, Resolver, NULL);
  context = NameLookup(ns, "/ace", &status);
  if(context) Err("NameLookup(/ace) didn't fail");
  context = NameLookup(ns, "/far", &status);
  if(status != 0) Err("NameLookup(/far) returned wrong type\n");
  if(strcmp((char*)context, "far")) Err("NameLookup(/far) failed\n");
  context = NameLookup(ns, "/ballyhoo", &status);
  if(!context) Err("NameLookup(/ballyhoo) failed\n");
  if(status != 1) Err("NameLookup(/ballyhoo) returned wrong type\n");
  context = NameLookup(ns, "/big/frog", &status);
  if(!context) Err("NameLookup(/big/frog) failed\n");
  if(status != 0) Err("NameLookup(/big/frog) returned wrong type\n");
  if(strcmp((char*)context, "frog")) Err("NameLookup(/big/frog) failed\n");
  status = NameRemove(ns, "/faz");
  if(!status) Err("NameRemove(/faz) failed\n");

  NameDump(stdout, NameRoot(ns));

  NameSpaceFree(ns);
  MEM_BLOCKS();
  MEM_STATUS();
}
