/* @(#)popen.c  4.5 (Berkeley) 7/6/84
 * Modified from 4.2, 2.9 BSD popen by Stephen Uitti, PUCC, Nov '85
 * Doesn't invoke shell, saving time, becoming more secure.
 * Calls breakargs to break line into command line arguments,
 * delimited by white space, but ignores globing and quoting
 * characters.  For use where you don't need shell meta character
 * expansion or filename globbing.
 */

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <sys/wait.h>
#include "popen.h"

#define tst(a,b)        (*mode == 'r'? (b) : (a))
#define RDR     0
#define WTR     1
#define SPC '\040'                      /* ascii space */

static  int popen_pid[20];
extern char **environ;
char **breakargs(char*);
char *mynext(char*);

/* no-shell popen */
FILE *
nshpopen(cmd, mode)
char *cmd;
char *mode;
{
        int p[2];
        int myside, hisside, pid;
        char **elist;                           /* the execv list */

        elist = breakargs(cmd);
        if (pipe(p) < 0)
                return (NULL);
        myside = tst(p[WTR], p[RDR]);
        hisside = tst(p[RDR], p[WTR]);
        if ((pid = fork()) == 0) {
                /* myside and hisside reverse roles in child */
                close(myside);
                if (hisside != tst(0, 1)) {     /* 0, 1 => stdin, stdout */
                        dup2(hisside, tst(0, 1));
                        close(hisside);
                }
		
                execve(elist[0], elist, environ);
                perror("execv");
                _exit(1);                       /* bombed execv, child dies */
        }
        free(elist);                            /* discard the broken args */
        if (pid == -1) {
                close(myside);
                close(hisside);
                return (NULL);
        }
        popen_pid[myside] = pid;
        close(hisside);
        return (fdopen(myside, mode));
}

/* close for nshpopen */
int *
nshpclose(ptr)
FILE *ptr;
{
#ifdef  BSD4_2
        int child, pid, status, omask;

        child = popen_pid[fileno(ptr)];
        fclose(ptr);
#define mask(s) (1 << ((s)-1))
        omask = sigblock(mask(SIGINT)|mask(SIGQUIT)|mask(SIGHUP));
        while ((pid = wait(&status)) != child && pid != -1)
                /* empty */;
        (void) sigsetmask(omask);
        return (int *)(pid == -1 ? -1 : status);
#else           /* 2.9 BSD, at least */
        int f, r;
        void (*hstat)(), (*istat)(), (*qstat)();
        int status;

        f = fileno(ptr);
        fclose(ptr);
        istat = signal(SIGINT, SIG_IGN);
        qstat = signal(SIGQUIT, SIG_IGN);
        hstat = signal(SIGHUP, SIG_IGN);
        while((r = wait(&status)) != popen_pid[f] && r != -1)
                ;
        if(r == -1)
                status = -1;
        signal(SIGINT, istat);
        signal(SIGQUIT, qstat);
        signal(SIGHUP, hstat);
        return (int *)(status);
#endif
}

/* breakargs - break a string into a string vector for execv.
 * Returns NULL if can't malloc space for vector, else vector.
 * Note, when done with the vector, merely "free" the vector.
 * Written by Stephen Uitti, PUCC, Nov '85 for the new version
 * of "popen" - "nshpopen", that doesn't use a shell.
 * (used here for the as filters, a newer option).
 *
 * breakargs is copyright (C) Purdue University, 1985
 *
 * put in a fix for cmds lines with "string string" in them
 * Mon Aug 25 13:34:27 EST 1986 (ksb)
 *
 * Permission is hereby given for its free reproduction and
 * modification for non-commercial purposes, provided that this
 * notice and all embedded copyright notices be retained.
 * Commercial organisations may give away copies as part of their
 * systems provided that they do so without charge, and that they
 * acknowledge the source of the software.
 */
#define SPC '\040'                      /* ascii space */

char *
mynext(pch)
register char *pch;
{
        register int fQuote;

        for (fQuote = 0; (*pch != '\000' && *pch != SPC && *pch != '\t')||fQuote
; ++pch) {
                if ('\\' == *pch) {
                        continue;
                }
                switch (fQuote) {
                default:
                case 0:
                        if ('"' == *pch) {
                                fQuote = 1;
                        } else if ('\'' == *pch) {
                                fQuote = 2;
                        }
                        break;
                case 1:
                        if ('"' == *pch)
                                fQuote = 0;
                        break;
                case 2:
                        if ('\'' == *pch)
                                fQuote = 0;
                        break;
                }
        }
        return pch;
}

char **
breakargs(cmd)
char *cmd;
{
        register char *p;               /* tmp                          */
        register char **v;              /* vector of commands returned  */
        register unsigned sum;          /* bytes for malloc             */
        register int i;                 /* number of args               */
        register char *s;               /* save old position            */

        p = cmd;
        while (*p == SPC || *p == '\t')
                p++;
        cmd = p;                        /* no leading spaces            */
        sum = sizeof(char *);
        i = 0;
        while (*p != '\0') {            /* space for argv[];            */
                ++i;
                s = p;
                p = mynext(p);
                sum += sizeof(char *) + 1 + (unsigned)(p - s);
                while (*p == SPC || *p == '\t')
                        p++;
        }
        ++i;
        /* vector starts at v, copy of string follows NULL pointer */
        v = (char **)malloc(sum+1);
        if (v == NULL)
                return (char **)NULL;
        p = (char *)v + i * sizeof(char *); /* after NULL pointer */
        i = 0;                          /* word count, vector index */
        while (*cmd != '\0') {
                v[i++] = p;
                s = cmd;
                cmd = mynext(cmd);
                if (*cmd != '\0')
                        *cmd++ = '\0';
                strcpy(p, s);
                p += strlen(p);
                ++p;
                while (*cmd == SPC || *cmd == '\t')
                        ++cmd;
        }
        v[i] = (char *)NULL;
        return v;
}
