/* Definitions for using Tpm_Region */
#ifndef TPM_REGION_H_INCLUDED
#define TPM_REGION_H_INCLUDED

#include <tpm/polygon.h>

/* A Region is a subset of an image.
 * It is represented by a bounding box and a bitmask.
 * An empty region has data == NULL.
 * Routines are provided for converting between polygons and regions.
 * This is useful for converting a moused outline into a region.
 */

typedef struct {
  int x, y;
  unsigned width, height;
  char **data;        /* bitmap */
} Tpm_Region;

#define Tpm_RegionIter(r, tx, ty) \
for(ty=(r)->y; ty < (r)->y + (r)->height; ty++) \
for(tx=(r)->x; tx < (r)->x + (r)->width; tx++) \
if((r)->data[ty - (r)->y][tx - (r)->x])

Tpm_Region *Tpm_RegionCreate(void);
void Tpm_RegionFree(Tpm_Region *r);
void Tpm_RegionSetDims(Tpm_Region *r, int width, int height);
void Tpm_RegionRectangle(Tpm_Region *r, int x, int y, int width, int height);
Tpm_Region *Tpm_RegionFromPolygon(Tpm_Polygon *p);
Tpm_Polygon *Tpm_RegionPolygon(Tpm_Region *r);
Tpm_Polygon **Tpm_RegionToPolygons(Tpm_Region *r);
Tpm_Region *Tpm_RegionIntersection(Tpm_Region *a, Tpm_Region *b);
int Tpm_RegionArea(Tpm_Region *r);
int Tpm_RegionContains(Tpm_Region *r, int x, int y);

#ifdef EOF
void Tpm_RegionWritePGM(FILE *h, Tpm_Region *r);
void Tpm_RegionPrint(FILE *h, Tpm_Region *r);
void Tpm_RegionScan(FILE *h, Tpm_Region *r);
#endif

#endif
