/* String utilities.
 */
#include "tpm.h"

/* Splits a string into words, a la Tcl_SplitList.
 * split is the separator character.
 * A repeated separator implies a null word.
 * Leading and trailing separators are ignored.
 * *argv_p and the strings in it must be freed by the caller.
 */
void StringSplit(char *string, char split, int *argc_p, char ***argv_p)
{
  char *p, **word;
  /* at most len/2 words in string */
  *argv_p = word = Allocate((strlen(string)+1)/2, char*);
  for(;*string;string++) {
    /* scan until split char */
    for(p=string;*p && *p != split;p++);
    if(p == string) continue;
    /* make a copy */
    *word = AllocCopy(p-string+1, char, string);
    (*word)[p-string] = 0;
    word++;
    if(!*p) break;
    string = p;
  }
  *argc_p = word - *argv_p;
  Reallocate(*argv_p, *argc_p, char*);
}

/* Frees the storage allocated by StringSplit.
 */
void StringSplitFree(int argc, char **argv)
{
  int i;
  for(i=0;i<argc;i++) free(argv[i]);
  free(argv);
}
