/* Miscellaneous commands. Add to Tk with Utiltk_Init(interp).
 * Add to Tk with
 *   require utilTk
 */

#include <tk.h>
#include <tcl_macros.h>
#include <string.h>
#include <stdlib.h> /* for malloc */

/* Prototypes ****************************************************************/
int Utiltk_Init(Tcl_Interp *interp);

/* Functions *****************************************************************/

struct ImageNotifyData {
  Tcl_Interp *interp;
  char *proc;
};

static void changeProc(struct ImageNotifyData *data, 
		       int x, int y, int width, int height,
		       int imageWidth, int imageHeight)
{
  char str[4][10];
  sprintf(str[0], " %d", x);
  sprintf(str[1], " %d", y);
  sprintf(str[2], " %d", width);
  sprintf(str[3], " %d", height);
  if(Tcl_VarEval(data->interp, data->proc, str[0], str[1], str[2], str[3], NULL) != TCL_OK) {
    Tcl_Eval(data->interp, "tkerror {image notification failure}");
  }
}

static int ImageNotifyCmd(ClientData clientData, Tcl_Interp *interp,
			  int argc, char *argv[])
{
  Tk_Window main = Tk_MainWindow(interp);
  Tk_Image image;
  Tk_Window win;
  struct ImageNotifyData *data;

  Err(argc < 4, "usage: image_notify image widget proc");
  GetWindow(argv[2], win);
  data = (struct ImageNotifyData *)malloc(sizeof(struct ImageNotifyData));
  data->interp = interp;
  data->proc = strdup(argv[3]);
  image = Tk_GetImage(interp, win, argv[1], 
		      (Tk_ImageChangedProc*)changeProc, data);
  /* make a destroy binding which releases the image */
  return TCL_OK;
}

static int FormatRawCmd(ClientData clientData, Tcl_Interp *interp,
			int argc, char *argv[])
{
  extern Tk_PhotoImageFormat FormatRaw;
  Tk_CreatePhotoImageFormat(&FormatRaw);
  return TCL_OK;
}

#ifndef NO_TIFF
static int FormatTiffCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[])
{
  extern Tk_PhotoImageFormat FormatTiff;
  Tk_CreatePhotoImageFormat(&FormatTiff);
  return TCL_OK;
}
#endif

#ifndef NO_JPEG
static int FormatJpegCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[])
{
  extern Tk_PhotoImageFormat FormatJpeg;
  Tk_CreatePhotoImageFormat(&FormatJpeg);
  return TCL_OK;
}
#endif

#ifndef NO_SGI
static int FormatSgiCmd(ClientData clientData, Tcl_Interp *interp,
			 int argc, char *argv[])
{
  extern Tk_PhotoImageFormat FormatSgi;
  Tk_CreatePhotoImageFormat(&FormatSgi);
  return TCL_OK;
}
#endif

int Utiltk_Init(Tcl_Interp *interp)
{
#if 0
  if(!Tcl_PkgRequire(interp, "Tk", "4.1", 0)) return TCL_ERROR;
#endif
  Tcl_CreateCommand(interp, "image_notify", ImageNotifyCmd, NULL, NULL);
  Tcl_CreateCommand(interp, "formatRaw", FormatRawCmd, NULL, NULL);
#ifndef NO_TIFF
  Tcl_CreateCommand(interp, "formatTiff", FormatTiffCmd, NULL, NULL);
#endif
#ifndef NO_JPEG
  Tcl_CreateCommand(interp, "formatJpeg", FormatJpegCmd, NULL, NULL);
#endif
#ifndef NO_SGI
  Tcl_CreateCommand(interp, "formatSgi", FormatSgiCmd, NULL, NULL);
#endif
  return Tcl_PkgProvide(interp, "utilTk", "1");
}
