/* Benchmark of learning with an across hier.
across_bench seinfeld db/seinfeld/work/bench.all rgb ohta output
 */
#include <tpm/boa_learn.h>
#include <ph/photobook.h>

int AcrossDataCompare(AcrossData *a, AcrossData *b)
{
  return memcmp(a, b, sizeof(AcrossData));
}

Type AcrossDataType;

List AcrossDataInstance(AcrossData *d)
{
  List instances = ListCreate((FreeFunc*)Boa_Free);
  Obj t = Boa_CreateFromData(AcrossDataType, NULL);
  t->value = d;
  ListAddFront(instances, t);
  return instances;
}

main(int argc, char *argv[])
{
  Obj od, across, member_list, within_list, features, classifiers, t;
  char str[128];
  int i;
  Learn_BenchData truth;
  Learn_BenchGroup grp = NULL;
  FILE *fp, *log;
  int num_words;
  char **words;

  if(argc < 5) {
    fprintf(stderr, "usage: across_bench db truth hier hier ... output < pre\n");
    fprintf(stderr, "truth file should be lines containing\n");
    fprintf(stderr, "\"label\" weight within member segment label-num\n");
    fprintf(stderr, "pre is of the same form.\n");
    exit(1);
  }

  Boa_Startup();
  Boa_Tcl_Pkg();
  Boa_Hier_Pkg();
  Boa_Learn_Pkg();

  od = Boa_Create(T_ObjDir);
  sprintf(str, "%s/%s", getenv("PHOTOBOOK_DB_DIR"), argv[1]);
  Boa_StringSet(od, "_path", BOA_REPLACE, Boa_String_01(str));

  if(Boa_StringGet(od, "index", 0, NULL, &member_list) != TPM_OK) {
    ErrorPrepend("Cannot read index");
    ErrorPrint(stderr);
    exit(1);
  }
  if(Boa_StringGet(od, "within_list", 0, NULL, &within_list) != TPM_OK) {
    ErrorPrepend("Cannot read within_list");
    ErrorPrint(stderr);
    exit(1);
  }

  /* read the across directory and acrosses from the command line */
  if(Boa_StringGet(od, "across", 0, NULL, &across) != TPM_OK) {
    ErrorPrepend("Cannot read across");
    ErrorPrint(stderr);
    exit(1);
  }
  features = Boa_Create(T_Ord);
  for(i=3;i<argc-1;i++) {
    char *fname = argv[i];
    if(Boa_StringGet(across, fname, 0, NULL, &t) != TPM_OK) {
      ErrorPrepend("Cannot open across hier `%s'", fname);
      ErrorPrint(stderr);
      exit(1);
    }
    if(t->type->type != T_Hier) {
      fprintf(stderr, "`%s' is not a Hier\n", fname);
      exit(1);
    }
    Boa_StringSet(features, fname, BOA_INSERT, t);
  }
  Boa_Release(across);

  /* create the classifiers */
  AcrossDataType = Boa_Type_Create("(array 3 int)");
  classifiers = Boa_MultiClass_Create(Boa_Classifier_Create(
		  T_Classifier_SSC, features, (CmpFunc*)AcrossDataCompare,
		  (InstanceFunc*)AcrossDataInstance));

  /* read the ground truth */
  fp = fopen(argv[2], "r");
  if(!fp) {
    fprintf(stderr, "Cannot open `%s'\n", argv[2]);
    exit(1);
  }
  truth = Learn_BenchData_Create();
  for(i=0;;i++,TpmSplitFree(num_words, words)) {
    int label;
    float weight;
    AcrossData *data;
    Obj t;
    char line[128];
    if(!getline(line, 128, fp)) break;
    if(i % 100 == 0) fprintf(stderr, "%s\n", line);
    /* parse into words */
    TpmSplit(line, ' ', &num_words, &words);
    if(num_words == 0) {
      grp = Learn_BenchData_AddGroup(truth);
      continue;
    }
    if(num_words != 6) continue;
    
    label = atoi(words[5]);
#if 1
    if(label < 0) continue;
#endif
    while(label >= Boa_Ord_Length(classifiers)) {
      Boa_MultiClass_AddClass(classifiers);
    }
    weight = atof(words[1]);

    t = Boa_Create(AcrossDataType);
    data = (AcrossData*)t->value;
    /* look up the within number */
    if(Boa_StringGetInt(within_list, words[2], &data->within) != TPM_OK) {
      fprintf(stderr, "unknown within `%s'\n", words[2]);
      continue;
    }
    /* look up the member number */
    if(Boa_StringGetInt(member_list, words[3], &data->member) != TPM_OK) {
      fprintf(stderr, "unknown member `%s'\n", words[3]);
      continue;
    }
    data->segment = atoi(words[4]);

    Learn_BenchData_AddInstance(truth, label, weight, t, grp);
  }
  fclose(fp);

  /* enter the "pre" labels */
  fprintf(stderr, "reading pre labels\n");
  for(i=0;;i++,TpmSplitFree(num_words, words)) {
    int label;
    float weight;
    AcrossData *data;
    Obj t;
    char line[128];
    Obj classifier;
    if(!getline(line, 128, stdin)) break;
    if(i % 100 == 0) fprintf(stderr, "%s\n", line);

    /* parse into words */
    TpmSplit(line, ' ', &num_words, &words);
    if(num_words != 6) continue;
    
    label = atoi(words[5]);
#if 1
    if(label < 0) continue;
#endif
    while(label >= Boa_Ord_Length(classifiers)) {
      Boa_MultiClass_AddClass(classifiers);
    }
    weight = atof(words[1]);

    t = Boa_Create(AcrossDataType);
    data = (AcrossData*)t->value;
    /* look up the within number */
    if(Boa_StringGetInt(within_list, words[2], &data->within) != TPM_OK) {
      fprintf(stderr, "unknown within `%s'\n", words[2]);
      continue;
    }
    /* look up the member number */
    if(Boa_StringGetInt(member_list, words[3], &data->member) != TPM_OK) {
      fprintf(stderr, "unknown member `%s'\n", words[3]);
      continue;
    }
    data->segment = atoi(words[4]);

    classifier = Boa_Ord_Item(classifiers, label);
    if(Boa_ObjSet(classifier, t, BOA_INSERT,
		  Boa_Float_Create(weight)) != TPM_OK) {
      ErrorPrepend("Classifier set failed for label %d", label);
      ErrorPrint(stderr);
    }
    Boa_Release(t);
  }
  Boa_Release(within_list);
  Boa_Release(member_list);

  /* run the benchmark */
  fp = fopen(argv[argc-1], "w");
  sprintf(str, "%s.log", argv[argc-1]);
  log = fopen(str, "w");
  Learn_Benchmark(classifiers, truth, fp, log);
  fclose(log);
  fclose(fp);
  Learn_BenchData_Free(truth);
#if 0
  assert(Boa_StringSet(od, "_path", 0, Boa_String_01("/tmp/tpminka")) 
	 == TPM_OK);
  t = Boa_Ord_Item(classifiers, 3);
  if(Boa_Classifier_Dump(t, od) != TPM_OK) {
    ErrorPrint(stderr);
  }
#endif
  Boa_Release(classifiers);

  Boa_Release(od);
  Boa_Shutdown();
  exit(0);
}

