/* Applies a mapping to a hier. */
#include <ph/photobook.h>

void main(int argc, char *argv[])
{
  Obj od, mapObj;
  char *map;
  int i;
  Hier hier1, hier2;

  if(argc < 2) {
    fprintf(stderr, "usage: apply_map <map>\n");
    fprintf(stderr, "To apply map to the integer node data in Hier.\n");
    fprintf(stderr, "input/output is stdin/stdout\n");
    exit(1);
  }

  Boa_Startup();

  od = Boa_Create(T_ObjDir);
  if(Boa_StringGet(od, argv[1], 0, NULL, &mapObj)) {
    ErrorPrepend("Cannot open `%s'\n", argv[1]);
    ErrorPrint(stderr);
    exit(1);
  }
  Boa_Release(od);

  map = (char*)mapObj->value;
  /* read input hier (should contain ints) */
  hier1 = HierCreateFromFile("-");
  if(hier1->data_size != sizeof(int)) {
    fprintf(stderr, "Input mapping must contain ints\n");
    exit(1);
  }
  /* create output hier for the map range */
  hier2 = HierCreate(hier1->num_nodes, TypeSize(Boa_Array_Subtype(mapObj->type)));

  for(i=0;i<hier1->num_nodes;i++) {
    char *source;
    HierNode n1 = HierNodeAt(hier1, i);
    HierNode n2 = HierNodeAt(hier2, i);
    n2->children = n1->children;
    n2->parent = n1->parent;
    if(n1->children) {
      /* use map[0] for intermediate nodes */
      source = &map[0];
    }
    else {
      int *ip = (int*)&n1->data; 
      /* *ip is index into the map */
      /* hier2->data_size is width of the map */
      source = &map[*ip * hier2->data_size];
    }
    memcpy(&n2->data, source, hier2->data_size);
  }

  HierWrite(hier2, stdout);
  HierFree(hier2);
  HierFree(hier1);
  
  Boa_Release(mapObj);

  Boa_Shutdown();
  exit(0);
}
