/* Creates a within-image segment hierarchy from a segment file and a 
 * hierarchy of segment indices.
 */
#include <boa/boa.h>
#include <tpm/list.h>
#include <tpm/hier.h>
#include <ph/segmask.h>

/* Node structure for the incoming hier of indices */
typedef struct {
  int children, parent, data;
} IntHierNode;

void main(int argc, char *argv[])
{
  Obj od, segmentObj, hierObj;
  List stack;
  int i,j;
  Hier hier, segments, result;

  if(argc < 3) {
    fprintf(stderr, "usage: hier_seg segment_hier int_hier > output\n");
    exit(1);
  }

  Boa_Startup();
  Boa_Hier_Pkg();
  Boa_Tcl_Pkg();    /* since disk objects may need it. grrr */

  od = Boa_Create(T_ObjDir);
  if(Boa_StringGet(od, argv[1], '/', NULL, &segmentObj) != TPM_OK) {
    ErrorPrepend("Cannot open `%s'", argv[1]);
    ErrorPrint(stderr);
    exit(1);
  }
  segments = Boa_Hier_Value(segmentObj);

  if(Boa_StringGet(od, argv[2], '/'|BOA_SPLIT, NULL, &hierObj) != TPM_OK) {
    ErrorPrepend("Cannot open hier `%s'", argv[2]);
    ErrorPrint(stderr);
    exit(1);
  }
  hier = Boa_Hier_Value(hierObj);
  Boa_Release(od);
  result = HierCreate(hier->num_nodes, sizeof(Seg));

  /* loop hier nodes */
  stack = ListCreate(GenericFree); /* will contain Segs */
  for(i=hier->num_nodes;i>0;i--) {
    IntHierNode *n1 = (IntHierNode*)HierNodeAt(hier, i-1);
    HierNode n2 = (HierNode)HierNodeAt(result, i-1);
    Seg *seg;

    n2->children = n1->children;
    n2->parent = n1->parent;

    /* is it a leaf? */
    if(!n1->children) {
      /* push on the stack */
      seg = Allocate(1, Seg);
      *seg = *(Seg*)&HierNodeAt(segments, n1->data)->data;
      ListAddFront(stack, seg);
    }
    else {
      /* merge nodes from the stack */
      ListRemoveFront(stack, (void**)&seg);
      if(!seg) {
	fprintf(stderr, "stack underflow\n");
	exit(1);
      }
      while(--n1->children) {
	Seg *child_seg;
	ListRemoveFront(stack, (void**)&child_seg);
	if(!child_seg) {
	  fprintf(stderr, "stack underflow\n");
	  exit(1);
	}
	SegUnion(seg, child_seg);
	free(child_seg);
      }
      /* push on the stack */
      ListAddFront(stack, seg);
    }
    memcpy(&n2->data, seg, sizeof(Seg));
  }
  /* stack should now contain SegAll */
  if(!ListSize(stack) || 
     memcmp((Seg*)ListFront(stack)->data, &SegAll, SEG_SZ)) {
    fprintf(stderr, "did not unify all segments\n");
  }
  ListFree(stack);

  HierWrite(result, stdout);
  HierFree(result);
  Boa_Release(hierObj);
  Boa_Release(segmentObj);
  Boa_Shutdown();
}
