/* Converts a set of masks for an image into a within-image hierarchy. 
 * reg2seg 320 240 < /net/snaresbrook/data2/masks/m.209.63
 */
#include <tpm/list.h>
#include <tpm/region.h>
#include <tpm/hier.h>
#include <ph/segmask.h>

int main(int argc, char *argv[])
{
  Tpm_Region *r = Tpm_RegionCreate();
  int xmax, ymax;
  int x2, y2, i;
  Seg seg, *segp;
  List segs = ListCreate(GenericFree);
  Hier hier;

  if(argc < 3) {
    fprintf(stderr, "usage: reg2seg xmax ymax < masks > hier\n");
    exit(1);
  }
  xmax = atoi(argv[1]);
  ymax = atoi(argv[2]);

  /* Read the segs into a list */
  for(;;) {
    if(scanf("%d %d %d %d", &r->x, &y2, &x2, &r->y) < 4) break;
    /* eat the newline */
    getchar();
    /* Josh has these measured from the bottom */
    r->y = ymax - 1 - r->y;
    y2 = ymax - 1 - y2;
    Tpm_RegionSetDims(r, x2 - r->x + 1, y2 - r->y + 1);
    Tpm_RegionScan(stdin, r);

    seg = SegFromRegion(r, xmax, ymax);
    /* is the seg non-empty? */
    if(SegSize(&seg)) ListAddRear(segs, AllocCopy(1, Seg, &seg));
  }

  /* Copy the list into a flat Hier of Segs */
  hier = HierCreate(ListSize(segs), sizeof(Seg));
  i = 0;
  {ListIter(p, segp, segs) {
    HierNode n = HierNodeAt(hier, i++);
    n->children = 0;
    n->parent = -1;
    memcpy(&n->data, segp, sizeof(Seg));
  }}
  HierWrite(hier, stdout);
  exit(0);
}
