# usage: [view] < tree
if {$argc > 0} {
    load_ph
    set od [boa create [boa type objDir]]
    set view [boa get $od [lindex $argv 0]]
} else {
  set view 0
}
if {$argc > 1} {
    set label [boa get $od [lindex $argv 1]]
} else {
    set label 0
}

proc min {a b} {
    if {$a < $b} {return $a} else {return $b}
}
# unpack a list into variables
# e.g.  unpack {3 4 5} a b c
# slow; should be part of the language
proc unpack {list args} {
    # args is a list of vars in caller's scope
    set len [min [llength $list] [llength $args]]
    for {set i 0} {$i < $len} {incr i} {
	upvar [lindex $args $i] x
	set x [lindex $list $i]
    }
}

# Make widgets ################################################################
set gamma 1
set canvas .cf.c

frame .cf

# scrollbars
scrollbar .cf.s1 -command "$canvas yview" -relief sunk
scrollbar .cf.s2 -orient horiz -command "$canvas xview" -relief sunk
pack .cf.s2 -side bottom -fill x
pack .cf.s1 -side right -fill y
#$canvas config -xscroll ".cf.s2 set" -yscroll ".cf.s1 set"

# canvas
canvas $canvas -scrollregion {-1c -1c 50c 20000c} \
	-yscrollcommand ".cf.s1 set" \
	-xscrollcommand ".cf.s2 set"
pack .cf.c -expand 1 -fill both
pack .cf -expand 1 -fill both

bind $canvas <2> "$canvas scan mark %x %y"
bind $canvas <B2-Motion> "$canvas scan dragto %x %y"

# Read tree ###################################################################

set stack ""
set yspacing 5
set xspacing 20
set ypos 0
set xpos 0

# loop lines
while {[gets stdin line] != -1} {
    set leaf [lindex $line 0]
    if {$leaf == ":"} {
        set count [lindex $line 1]
        set children [lrange $stack 0 [expr $count-1]]

	# average the y's and max the x's
	set yavg 0
	set xmax 0
	foreach child $children {
	    unpack $child x y
	    incr yavg $y
	    if {$x > $xmax} {set xmax $x}
	}
	set yavg [expr $yavg/$count]
	incr xmax $xspacing

	# connect the children
	foreach child $children {
	    unpack $child x y
	    $canvas create line $x $y $xmax $y
	}
	set ymin [lindex [lindex $children 0] 1]
	set ymax [lindex [lindex $children [expr $count-1]] 1]
	$canvas create line $xmax $ymin $xmax $ymax

	# show the connection weight
	$canvas create text [expr $xmax+4] [expr $yavg-2] \
		-text [lindex $line 2] -anchor sw -font 5x7

	set stack "[list [list $xmax $yavg]] [lrange $stack $count end]"
    } else {
	puts $leaf
	if $view {
	    set img [image create photo -gamma $gamma]
	    set boa_img [boa get $view $leaf]
	    boa tk_image $boa_img $img
	    set item [$canvas create image $xpos $ypos -anchor nw -image $img]
	} else {
	    set item [$canvas create text $xpos $ypos -anchor nw \
		    -text $leaf -font 5x7]
	}
	unpack [$canvas bbox $item] x1 y1 x2 y2
	set yavg [expr ($y1+$y2)/2]
	if $label {
	    if {![catch {boa tcl_var [boa get $label $leaf] text}]} {
		$canvas create text [expr $x2+2] [expr $yavg+2] -text $text \
			-anchor sw -fill yellow
	    }
	}
	set stack "[list [list $x2 $yavg]] $stack"
	set ypos [expr $y2 + $yspacing]
    }
}
