#include <ph/photobook.h>

static Obj AddElement(Obj array, int length, int i, Obj t)
{
  char *data;
  if(!array) {
    /* create array once we know what the subtype is */
    Type type;
    Boa_Hold(T_Array);
    Boa_Hold(t->type);
    type = Boa_Type_VaCreate(T_Array, Boa_Int_Create(length), t->type);
    Boa_PrintNL(stderr, type);
    /* use FromData to prevent initialization */
    data = Allocate(TypeSize(type), char);
    array = Boa_CreateFromData(type, data);
  }
  else data = (char*)array->value + i*TypeSize(t->type);
  /* copy in the new value */
  /* this code is similar to reading an array in format_ascii */
  memcpy(data, t->value, TypeSize(t->type));
  BoaFreeAND01(t);
  Boa_Release(t);
  return array;
}

void main(int argc, char *argv[])
{
  Obj od, model, view, withins, mci, output1, output2, segmask;
  char collect_icov;
  char str[128];

  if(argc < 5) {
    fprintf(stderr, "usage: within_feature model view within mean_output [icov_output] < members\n");
    fprintf(stderr, "To compute image features, according to model,\n");
    fprintf(stderr, "over all segments in the within-image hier,\n");
    fprintf(stderr, "for each member name given via stdin.\n");
    exit(1);
  }
  collect_icov = argc > 5;

  Boa_Startup();
  Boa_Vector_Pkg();
  Boa_Image_Pkg();
  Boa_Hier_Pkg();
  Boa_Region_Pkg();
  Boa_Tcl_Pkg();    /* since disk objects may need it. grrr */

  od = Boa_Create(T_ObjDir);
  if(Boa_StringGet(od, argv[1], '/', NULL, &model) != TPM_OK) {
    ErrorPrepend("Cannot open model `%s'", argv[1]);
    ErrorPrint(stderr);
    exit(1);
  }
  if(Boa_StringGet(od, argv[2], '/', NULL, &view)) {
    ErrorPrepend("Cannot open view `%s'", argv[2]);
    ErrorPrint(stderr);
    exit(1);
  }
  if(!argv[3][0]) withins = NULL;
  else if(Boa_StringGet(od, argv[3], '/', NULL, &withins)) {
    ErrorPrepend("Cannot open within `%s'", argv[3]);
    ErrorPrint(stderr);
    exit(1);
  }
  if(Boa_StringGet(od, argv[4], '/', NULL, &output1)) {
    ErrorPrepend("Cannot open output `%s'", argv[4]);
    ErrorPrint(stderr);
    exit(1);
  }
  if(collect_icov) {
    if(Boa_StringGet(od, argv[5], '/', NULL, &output2)) {
      ErrorPrepend("Cannot open output `%s'", argv[5]);
      ErrorPrint(stderr);
      exit(1);
    }
  }

  sprintf(str, "%s/lib/common/mci.boa", getenv("PHOTOBOOK_ROOT"));
  if(Boa_StringGet(od, str, 0, NULL, &mci) != TPM_OK) {
    ErrorPrepend("Couldn't open lib/common/mci.boa");
    ErrorPrint(stderr);
    exit(1);
  }
  if(withins) {
    /* segmask doesn't own its data */
    segmask = Boa_CreateFromData(T_Segmask, NULL);
    /* we're going to keep the segmask */
    Boa_Hold(segmask);
    Boa_StringSet(mci, "segmask", BOA_INSERT, segmask);
  }

  /* loop members */
  for(;;) {
    Obj imageObj, hierObj;
    int nwithin, i;
    WithinNode *within;
    Obj volume, t, mean_results, icov_results;

    char member[128];
    if(!fgets(member, 128, stdin)) break;

    /* only create if it's not there */
    if(Boa_StringGet(output1, member, 0, NULL, NULL) == TPM_OK) continue;

    if(withins) {
      /* read the hier */
      if(Boa_StringGet(withins, member, 0, NULL, &hierObj)) {
	ErrorPrepend("Cannot get hier for %s", member);
	ErrorPrint(stderr);
	continue;
      }
      nwithin = Boa_Hier_Length(hierObj->type);
      within = (WithinNode*)((Hier)hierObj->value)->node;
    }
    else nwithin = 1;

    /* read the image */
    if(Boa_StringGet(view, member, 0, &T_Image, &imageObj)) {
      if(withins) Boa_Release(hierObj);
      ErrorPrepend("Cannot get image for %s", member);
      ErrorPrint(stderr);
      continue;
    }

    /* apply the model */
    ErrorSet("unknown");
    if(Boa_ObjGet(model, imageObj, 0, &T_Volume, &volume) != TPM_OK) {
      Boa_Release(imageObj);
      ErrorPrepend("model failed for %s", member);
      ErrorPrint(stderr);
      continue;
    }
    Boa_Release(imageObj);
#if 0
    /* extract regions of the resulting feature image */
    Boa_StringSet(mci, "volume", BOA_INSERT|BOA_REPLACE, t);
#endif

    mean_results = NULL;
    if(collect_icov) icov_results = NULL;
    for(i=0;i<nwithin;i++) {
      Obj result;
      if(withins) {
	fprintf(stderr, "segment %d\n", i);
	/* Get the next segmask, by mutation.
	 * This is an efficiency hack to avoid setting "segmask" each time.
	 */
	segmask->value = &within[i].seg;
      }
      
      /* compute features */
      if(Boa_ObjGet(mci, volume, 0, NULL, &result) != TPM_OK) {
	ErrorPrepend("Call failed on segment %d", i);
	ErrorPrint(stderr);
	continue;
      }

      /* remove the result each time */
      if(Boa_StringGet(result, "mean", BOA_DELETE, NULL, &t)) {
	ErrorPrepend("no mean");
	ErrorPrint(stderr);
	continue;
      }
      if(withins) {
	/* add it to our array */
	mean_results = AddElement(mean_results, nwithin, i, t);
      }
      else mean_results = t;

      if(collect_icov) {
	if(Boa_StringGet(result, "icov", BOA_DELETE, NULL, &t)) {
	  ErrorPrepend("no icov");
	  ErrorPrint(stderr);
	  continue;
	}
	if(withins) {
	  icov_results = AddElement(icov_results, nwithin, i, t);
	}
	else icov_results = t;
      }
      Boa_Release(result);
    }
    Boa_Release(volume);

    /* output the results */
    if(mean_results) {
      if(Boa_StringSet(output1, member, BOA_INSERT|BOA_REPLACE, mean_results) 
	 != TPM_OK) {
	ErrorPrepend("Error writing mean");
	ErrorPrint(stderr);
      }
      if(collect_icov && icov_results) {
	if(Boa_StringSet(output2, member, BOA_INSERT|BOA_REPLACE, icov_results)
	   != TPM_OK) {
	  ErrorPrepend("Error writing icov");
	  ErrorPrint(stderr);
	}
      }
    }
    if(withins) Boa_Release(hierObj);
  }
  
  if(withins) {
    Boa_Release(segmask);
    Boa_Release(withins);
  }
  if(collect_icov) Boa_Release(output2);
  Boa_Release(output1);
  Boa_Release(mci);
  Boa_Release(view);
  Boa_Release(model);
  Boa_Release(od);

  Boa_Shutdown();
  exit(0);
} 

