/* To test in Tcl:
boa set $od _path [boa string "db/bt10"]
ph within init [boa get $od within] "8x8/001"
ph within pos .c 0 0
set seg [ph within get]
ph showseg .c $seg black
 */
#include "photobook.h"
#include <tpm/boa_learn.h>

static Obj classifier = NULL;

/* Prototypes ****************************************************************/

void Ph_WithinInit(Obj hiercon, List hiers);
void Ph_WithinDestroy(void);
void Ph_WithinSave(void);
void Ph_WithinClear(void);
Seg Ph_WithinGet(void);
void Ph_WithinEx(int x, int y, float weight);

/* Functions *****************************************************************/

static int PointInSeg(Seg *seg, XPoint *p)
{
  return !SegIsSet(seg, p->x, p->y);
}

/* can't tell if this is faster than an empty cache */
static List PointInstance(Seg *seg)
{
  List instances = ListCreate((FreeFunc*)Boa_Free);
  XPoint p;
  for(p.y=0;p.y<SEG_YCELLS;p.y++) {
    for(p.x=0;p.x<SEG_XCELLS;p.x++) {
      if(SegIsSet(seg, p.x, p.y))
	ListAddFront(instances, Boa_Int_Create(*(int*)&p));
    }
  }
  return instances;
}

static void HierFix(Hier hier)
{
#ifndef TPM_SWAP
  /* Fix up the data in the hier */
  int j;
  for(j=0;j<hier->num_nodes;j++) {
    HierNode n = HierNodeAt(hier, j);
    SwapBuffer(&n->data, SEG_SZ/4, 4);
  }
#endif
}

void Ph_WithinInit(Obj within, List within_list)
{
  Obj features, t;
  char *fname;

  Boa_Learn_Pkg();
  Ph_WithinDestroy();
  features = Boa_Create(T_Ord);
  {ListIter(p, fname, within_list) {
    if(Boa_StringGet(within, fname, '/', NULL, &t)) {
      ErrorPrepend("Cannot open within hier `%s'", fname);
      ErrorPrint(stderr);
      continue;
    }
    if(t->type->type != T_Hier) {
      fprintf(stderr, "`%s' is not a Hier\n", fname);
      continue;
    }
    HierFix(Boa_Hier_Value(t));
    Boa_StringSet(features, fname, BOA_INSERT, t);
  }}
  /* create the classifier */
  classifier = Boa_Classifier_Create(T_Classifier_SC, 
				     features, (CmpFunc*)PointInSeg, 
				     (InstanceFunc*)PointInstance);

  fprintf(stderr, "Learning with ");
  Boa_PrintNL(stderr, Boa_Filter_Context(classifier));
}

void Ph_WithinDestroy(void)
{
  if(!classifier) return;
  Boa_Release(classifier);
  classifier = NULL;
}

void Ph_WithinSave(void)
{
  /* do nothing */
}

void Ph_WithinClear(void)
{
  Type classifierType;
  if(!classifier) return;
  /* just need to recreate the classifier object */
  classifierType = Boa_Filter_Context(classifier)->type;
  Boa_Release(Boa_Filter_Context(classifier));
  Boa_Filter_Context(classifier) = Boa_Create(classifierType);
}

/* Loop all cells and set those which are in the class */
Seg Ph_WithinGet(void)
{
  Seg seg = SegZero;
  XPoint p;
  Obj instance;
  if(!classifier) return seg;
  /* instance will track p */
  instance = Boa_CreateFromData(T_Int, NULL);
  instance->value = &p;
  /* loop all cells */
  for(p.y=0;p.y<SEG_YCELLS;p.y++) {
    for(p.x=0;p.x<SEG_XCELLS;p.x++) {
      Obj t;
      if(Boa_ObjGet(classifier, instance, 0, &T_Float, &t) != TPM_OK) {
	ErrorPrepend("classifier failed on (%d, %d)", p.x, p.y);
	ErrorPrint(stderr);
	continue;
      }
      if(Boa_Float_Value(t) > 0) SegSet(&seg, p.x, p.y);
      Boa_Release(t);
    }
  }
  Boa_Release(instance);
  return seg;
}

void Ph_WithinEx(int x, int y, float weight)
{
  XPoint p;
  Obj instance;
  if(!classifier) {
    fprintf(stderr, "Ph_WithinEx: not initialized\n");
    return;
  }
  /* instance will track p */
  assert(sizeof(XPoint) == sizeof(int));
  instance = Boa_CreateFromData(T_Int, NULL);
  instance->value = &p;
  p.x = x;
  p.y = y;
  Boa_ObjSet(classifier, instance, 0, Boa_Float_Create(weight));
  Boa_Release(instance);
}
