#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <tpm/binary.h>

#if !defined(A2B) && !defined(B2A)
#define A2B
#endif

#ifdef A2B
#define DOCHAR(ch) while((fgetc(fpin) != ch) && !feof(fpin))
#else
#define DOCHAR(ch) sprintf(buffer, "%s%c", buffer, ch)
#endif

#define ADJUST 1

/* BUFSIZE is assumed to be larger than any C data type */
#define BUFSIZE 32*1024

enum { T_CHAR, T_SHORT, T_INT, T_FLOAT, T_DOUBLE, T_STRING };

void main(int argc, char *argv[])
{
  FILE *fpin, *fpout;
  char *format;
  int i, format_length, item_type;
  char f_str[] = "%  ";
  char *buffer, *item_buffer;
  char ch;
  int sizes[5] = { sizeof(char), sizeof(short), sizeof(int), sizeof(float), sizeof(double) };
  char arg, adjust;

  if(argc < 2) {
    printf("Usage:\n\n%s [-n] format\n\n", argv[0]);
#ifdef A2B
    printf("To convert ASCII data from stdin to normalized binary data on stdout.\n");
#else
    printf("To convert normalized binary data from stdin to ASCII data on stdout.\n");
#endif
    printf("<format> is a string of ordinary characters and printf-like data type\n");
    printf("specifiers, such as `item %%d has value %%lf\\n', ");
    printf("where each specifier is one of:\n");
    printf("  %%c           Character\n");
    printf("  %%b           One-byte integer\n");
    printf("  %%h           Two-byte integer\n");
    printf("  %%d           Integer\n");
    printf("  %%f,%%e,%%g     Float\n");
    printf("  %%lf,%%le,%%lg  Double\n");
    printf("\nBinary data is normalized to be little-endian.\n");
    printf("-n  Specifes native format for binary data.\n");
    i = 1;
    printf("This machine is %s.\n", 
	   *(unsigned char*)&i ? "little-endian" : "big-endian");
    exit(1);
  }

  adjust = 1;
  arg = 1;
  if(argc == 3) {
    if(!strcmp(argv[arg], "-n")) adjust = 0;
    else printf("Unknown option %s\n", argv[arg]);
    arg++;
  }

  format = argv[arg];
  format_length = strlen(format);

  fpin = stdin;
  fpout = stdout;

  buffer = (char*)malloc(BUFSIZE);
  item_type = sizes[0];
  for(i=1;i<4;i++) 
    if(sizes[i] > item_type) item_type = sizes[i];
  item_buffer = (char*)malloc(item_type);

  buffer[0] = 0;
  while(!feof(fpin)) {
    if(buffer[0]) fprintf(fpout, "%s", buffer);
    buffer[0] = 0;
    for(i=0;i < format_length;i++) {
      if(format[i] == '%') {
	if(format[++i] == '%') {
	  DOCHAR('%');
	  continue;
	}
	f_str[1] = format[i];
	f_str[2] = 0;
	switch(format[i]) {
	case 'b':
	  f_str[1] = 'd';
	case 'c':
	  item_type = T_CHAR;
	  break;
	case 'h':
	  f_str[1] = 'd';
	  item_type = T_SHORT;
	  break;
	case 'd':
	  item_type = T_INT;
	  break;
	case 'f':
	case 'e':
	case 'g':
	  item_type = T_FLOAT;
	  break;
	case 'l':
	  switch(format[++i]) {
	  case 'f':
	  case 'e':
	  case 'g':
	    item_type = T_DOUBLE;
	    f_str[2] = format[i];
	    f_str[3] = 0;
	    break;
	  default:
	    printf("Unknown data type: %c\n", format[i]);
	    exit(1);
	  }
	  break;
	case 's':
	  item_type = T_STRING;
	  break;
	default:
	  printf("Unknown data type: %c\n", format[i]);
	  exit(1);
	}

	if(item_type < T_STRING) {
#ifdef A2B
	  fscanf(fpin, f_str, item_buffer);
	  if(feof(fpin)) break;
	  if(format[i] == 'b') *(char*)item_buffer = (char)*(int*)item_buffer;
	  if(format[i] == 'h') *(short*)item_buffer = (short)*(int*)item_buffer;
#if ADJUST
	  if(adjust) AdjustBuffer((void*)item_buffer, 1, sizes[item_type]);
#endif
	  fwrite((void*)item_buffer, sizes[item_type], 1, fpout);
#else
	  fread((void*)item_buffer, sizes[item_type], 1, fpin);
	  if(feof(fpin)) break;
	  fprintf(fpout, "%s", buffer);
	  buffer[0] = 0;
#if ADJUST
	  if(adjust) AdjustBuffer((void*)item_buffer, 1, sizes[item_type]);
#endif
	  switch(item_type) {
	  case T_CHAR:
	    fprintf(fpout, f_str, *(unsigned char*)item_buffer);
	    break;
	  case T_SHORT:
	    fprintf(fpout, f_str, *(short*)item_buffer);
	    break;
	  case T_INT:
	    fprintf(fpout, f_str, *(int*)item_buffer);
	    break;
	  case T_FLOAT:
	    fprintf(fpout, f_str, *(float*)item_buffer);
	    break;
	  case T_DOUBLE:
	    fprintf(fpout, f_str, *(double*)item_buffer);
	    break;
	  }
#endif
	}
	else {
	}
      }
      else if(format[i] == '\\') {
	switch(format[++i]) {
	case 'n': ch = '\n'; break;
	case 't': ch = '\t'; break;
	default: ch = format[i];
	}
	DOCHAR(ch);
      }
      else {
	DOCHAR(format[i]);
      }
    }
  }

  fclose(fpin);
  fclose(fpout);
  free(buffer);
  free(item_buffer);
  exit(0);
}
